/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* this file implements a virtual soundcard that does I/O to disks files instead of sound driver.*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/errno.h>
#include <string.h>
#include "iodisk.h"

#define NAME_SIZE 100

static char *buffer1=NULL,*buffer2=NULL;
static unsigned int ts=0,real_ts=0;
static int bufsize;
static int blocksize=160*2; //20 ms

static char dev_selected=0;
static char input_filename[100];
static char output_filename[100];
static FILE *input=NULL,*output=NULL;


static void alarm_handler(int signum);

int audio_select_device_iodisk(char *infile_name, char * outfile_name)
{
	dev_selected=1;
	strncpy(input_filename,	infile_name,NAME_SIZE);
	strncpy(output_filename,	outfile_name,NAME_SIZE);
	return(0);
}		

int open_dsp_iodisk(int rate,int channel,int bits,int bsize)
{
	if (!dev_selected) return(-ENOENT);
	if (bsize<=0) return(-EINVAL);
	if ((bsize>180*2)|| (bsize<150*2)) return(-EOPNOTSUPP);  /*we can simulate approx 20 ms reads: REVISIT*/
	
	if (strlen(input_filename)!=0)
	{
		input=fopen(input_filename,"r");
		if (input==NULL) return(-ENOENT);
	}
	if (strlen(output_filename)!=0)
	{
		output=fopen(output_filename,"w");
		if (output==NULL) return(-ENOENT);
	}
	buffer1=calloc(bsize,sizeof(char));
	buffer2=calloc(bsize,sizeof(char));
	ts=0;
	real_ts=0;
	bufsize=bsize;
	if (input) fread(buffer1,1,bufsize,input);      /* dummy read to go over wav header */
	return(0);
}

int set_source_iodisk(char source)
{
	return(0);
}

int set_in_level_iodisk(int a)
{
	return(0);
}

int set_out_level_iodisk(int a)
{
	return(0);
}

int close_dsp_iodisk()
{
	if (input!=NULL)
	{
		fclose(input);
		input=NULL;
	}
	if (output!=NULL)
	{
		fclose(output);
		output=NULL;
	}
	
	if ((buffer1==NULL)) return(-EFAULT);
	free(buffer1);
	if ((buffer2==NULL)) return(-EFAULT);
	free(buffer2);
	buffer1=NULL;
	buffer2=NULL;
	return(0);
}

char *get_read_iodisk_buffer()
{
	struct itimerval ival;
	ts+=bufsize;
	while (ts-real_ts>=blocksize)
	{  /* sleep 20ms to simulate a read on the soundcard*/
		real_ts+=blocksize;
		ival.it_value.tv_sec=0;
		ival.it_value.tv_usec=20000;
		ival.it_interval.tv_sec=0;
		ival.it_interval.tv_usec=20000;
		setitimer(ITIMER_REAL,&ival,NULL);
		signal(SIGALRM,&alarm_handler);
		/* then we sleep until we are woken up by the alarm*/
		sleep(2);
	}
	if (input) fread(buffer1,1,bufsize,input);
	return(buffer1);
}


char *get_write_iodisk_buffer()
{
	if (output) fwrite(buffer2,1,bufsize,output);
	return(buffer2);
}


void alarm_handler(int signum)
{
	
}


