/***************************************************************************
                          rtprecv.c - reception of rtp packets
                             -------------------
    begin                : ven mar  30
    copyright            : (C) 2001 by Simon MORLAT
    email                : simon.morlat@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <sys/errno.h>
#include <arpa/inet.h>
#include <unistd.h>
#include "rtp.h"
#include "rtp_private.h"
#include "rtp_ptypes.h"



int rtp_recv(RtpSession * handle, void *buffer, int buflen)
{
	int i=0, j, k;
	int bytes_read, data_size;
	struct sockaddr_in from;
	int fromlen;
	uint32_t min_ts = 0xFFFFFFFF, tmpts, wanted_ts;
	int min_ind = -1;
	rtp_packet_t *packet_p=NULL;
	int err, cond = 1, tmp;
	char tmpbuf[UDP_MAX_SIZE];
	int pack_p_udp;

	fromlen = sizeof (from);

	wanted_ts=handle->rcv_last_ts+handle->rcv_tsinc;
	/* while the queue is not filled with jitt_comp elements */
	while ((handle->rcv_count < handle->params.jitt_comp) && (cond))
	{
		/* check for any incoming rtp or rtcp packet */

		/* try to read an RTCP packet */
		bytes_read = recvfrom (handle->rtcp_socket, (void *) tmpbuf,
								buflen, 0, (struct sockaddr *) &from, &fromlen);
		/*if (bytes_read > 0) return (-303);*/

		/* receive rtp packet and stores it */
		bytes_read = recvfrom (handle->rtp_socket, (void *) tmpbuf,
												UDP_MAX_SIZE, 0, (struct sockaddr *) &from,&fromlen);
		if (bytes_read <= 0)
		{
			cond = 0;
		}
		else
		{
			if (bytes_read >= UDP_MAX_SIZE) fprintf (stderr, "serious problem.\n");
			if (handle->params.redundancy > 0)
			{
				/* find how many rtp packets we have in this udp */
				pack_p_udp = bytes_read / (rtp_length[handle->params.profile]);
				bytes_read = rtp_length[handle->params.profile];
			}
			else pack_p_udp = 1;
			for (k = 0; k < pack_p_udp; k++)
			{
				handle->local_stats.received++;
				/* find first availlable place to store the packet */
				i = 0;
				while (handle->rcv_stack[i].packet_size > 0) i++;
				memcpy (&handle->rcv_stack[i].packet, &tmpbuf[bytes_read * k],
								bytes_read);
				packet_p=&handle->rcv_stack[i].packet;
				/* do the network to host conversion  for seq, ts, ssrc, csrc's*/
				packet_p->seq_number = ntohs (packet_p->seq_number);
				packet_p->timestamp= ntohl (packet_p->timestamp);
				packet_p->ssrc = ntohl (packet_p->ssrc);
				for (j = 0; j < packet_p->cc ; j++)
				{
					((uint32_t *) (&packet_p->data))[j]= ntohl (((uint32_t *) (&packet_p->data))[j]);
				}
				handle->rcv_stack[i].p_payload = &packet_p->data[packet_p->cc * 4];
				handle->rcv_stack[i].packet_size = bytes_read - RTP_FIXED_HEADER_SIZE - (packet_p->cc * 4);
				/* give debug information if needed */
				if (handle->dbg_file != NULL)
				{
					fprintf (handle->dbg_file, "Receiving rtp packet:\n");
					fprintf (handle->dbg_file, "  Total Length : %i\n",bytes_read);
					fprintf (handle->dbg_file, "  From : %s port %i\n",
									inet_ntoa (from.sin_addr), ntohs (from.sin_port));
					fprintf (handle->dbg_file,"  Version : %i   PaddingBit : %i  ExtBit : %i  \n ",
									packet_p->version, packet_p->padbit, packet_p->extbit);
					fprintf (handle->dbg_file,"  CC : %i   MarkBit : %i   PayloadType : %i\n",
									packet_p->cc, packet_p->markbit, packet_p->paytype);
					fprintf (handle->dbg_file,"  SeqNumber : %i   TimeStamp : %i   SSRC : %i\n",
									packet_p->seq_number, packet_p->timestamp, packet_p->ssrc);
					for (j = 0; j < packet_p->cc; j++)
						fprintf (handle->dbg_file, "  CSRC : %i\n",((uint32_t*)(packet_p->data))[j]);
				}
				handle->rcv_count++;
				/* check if it is a duplicated packet */
				for (j = 0; j < handle->rcv_count; j++)
				{
					if (((packet_p->seq_number == handle->rcv_stack[j].packet.seq_number)
							&& (handle->rcv_stack[j].packet_size > 0) && (i != j)) || (packet_p->seq_number == handle->rcv_last_seq))
					{
						/* this is a duplicated packet, discard it */
						handle->rcv_stack[i].packet_size = 0;
						handle->rcv_count--;
						if (handle->dbg_file != NULL)
						{
							fprintf (handle->dbg_file,"Discarding rtp packet (duplicated):\n");
							fprintf (handle->dbg_file,"  SeqNumber : %i   TimeStamp : %i\n",
											packet_p->seq_number, packet_p->timestamp);
						}
					}
				}
				/* check if it is a late packet */
				if ((((int)(packet_p->timestamp - wanted_ts))<0) && (!(handle->rcv_firsts))
						&& (handle->rcv_stack[i].packet_size!=0))
				{
					/* discard this packet, it is late */
					handle->rcv_stack[i].packet_size = 0;
					handle->rcv_count--;
					handle->local_stats.outdated++;
					if ((handle->rcv_last_seq - packet_p->seq_number)<0) handle->rcv_last_seq=packet_p->seq_number;
					if (handle->dbg_file != NULL)
					{
						fprintf (handle->dbg_file,"Discarding rtp packet (too late):\n");
						fprintf (handle->dbg_file,"  SeqNumber : %i   TimeStamp : %i   WantedTS : %i\n",
										packet_p->seq_number, packet_p->timestamp,wanted_ts);
					}
				}
				else if (handle->rcv_firsts)
				{
					/* update the minimum timestamp to find who is the first packet */
					if (((int32_t)(packet_p->timestamp-handle->rcv_first_ts)<0) || (handle->rcv_count==1))
					{
						handle->rcv_first_ts=packet_p->timestamp;
						handle->rcv_first_packet=i;
					}
				}
			}
		}
	}
	if (handle->rcv_firsts)
	{
		if (handle->rcv_count >= handle->params.jitt_comp)
		{
			/* there is enough packets in the queue to start transmission to the application */
			handle->rcv_firsts = 0;
			wanted_ts=handle->rcv_first_ts;
			if (handle->dbg_file != NULL)
			{
				fprintf (handle->dbg_file,"The first packet of the transmission has:\n");
				fprintf (handle->dbg_file,"  SeqNumber : %i   TimeStamp : %i\n",
								handle->rcv_stack[handle->rcv_first_packet].packet.seq_number,
								handle->rcv_first_ts);
			}
		}
		else
		{
			/* there is not enough packets in the queue to start returning payloads
			to the application */
			if (handle->dbg_file != NULL)
			{
				fprintf (handle->dbg_file,"Returning empty RTP payload to the application : need more incoming packets.\n");
			}
			return (0);
		}
	}
	/* check if there is something in the queue*/
	if (handle->rcv_count == 0)
	{
		/* there is no packets availlable in the queue, so return an empty packet */
		if (handle->dbg_file != NULL)
		{
			fprintf (handle->dbg_file,
			"Returning empty payload : the queue is empty. (WantedTS = %i)\n",wanted_ts);
		}
		handle->rcv_last_ts = wanted_ts;
		return (0);
	}
	/* find the wanted timestamp and eliminate old ones*/
	i=0;
	while((i< handle->params.stacksize))
	{
		if ((handle->rcv_stack[i].packet.timestamp == wanted_ts) && (handle->rcv_stack[i].packet_size > 0))
		{
			min_ts = tmpts;
			min_ind = i;
		}
		else if ((((int)(handle->rcv_stack[i].packet.timestamp - wanted_ts))<0) &&
							(handle->rcv_stack[i].packet_size > 0))
		{
			/* discard this packet, it is late */
			handle->rcv_stack[i].packet_size = 0;
			handle->rcv_count--;
			handle->local_stats.outdated++;
			if ((handle->rcv_last_seq - handle->rcv_stack[i].packet.seq_number)<0)
				handle->rcv_last_seq=handle->rcv_stack[i].packet.seq_number;
			if (handle->dbg_file != NULL)
			{
				fprintf (handle->dbg_file,"Discarding rtp packet (too late) (note: second check):\n");
				fprintf (handle->dbg_file,"  SeqNumber : %i   TimeStamp : %i   WantedTS : %i\n",
										packet_p->seq_number, packet_p->timestamp,wanted_ts);
			}
		}	
		i++;
	}
	
	if (min_ind == -1)
	{
		/* no packet found for this timestamp, return empty packet */
		memset (buffer, 0, buflen);
		if (handle->dbg_file != NULL)
		{
			fprintf (handle->dbg_file,"Missing rtp packet with TimeStamp : %i\n",
							wanted_ts);
		}
		handle->rcv_last_ts = wanted_ts;
		return (0);
	}
	/* the appropriate packet is found, but check for missing packets */
	if ((tmp =(uint16_t) (handle->rcv_stack[min_ind].packet.seq_number -
			handle->rcv_last_seq)) > 1)
	{
		handle->local_stats.lost += tmp;
		if (handle->dbg_file != NULL)
		{
			fprintf (handle->dbg_file,"Rtp packet(s) with SeqNumber %i..%i was(were) missing.\n",
							(int16_t) (handle->rcv_last_seq + 1),(uint16_t) (handle->rcv_stack[min_ind].packet.seq_number - 1));
		}
	}
	data_size = handle->rcv_stack[min_ind].packet_size;
	if (data_size > buflen) return (-302);
	memcpy (buffer, handle->rcv_stack[min_ind].p_payload, data_size);
	/* mark place as free */
	handle->rcv_stack[min_ind].packet_size = 0;
	handle->rcv_last_seq = handle->rcv_stack[min_ind].packet.seq_number;
	handle->rcv_last_ts = wanted_ts;
	handle->rcv_count--;
	if (handle->dbg_file != NULL)
	{
		fprintf (handle->dbg_file, "Returning packet to application.\n");
		fprintf (handle->dbg_file, "  SeqNumber : %i   TimeStamp : %i\n",
						handle->rcv_stack[min_ind].packet.seq_number,handle->rcv_stack[min_ind].packet.timestamp);
	}
	return (data_size);
}

