/***************************************************************************
                          rtpsend.c - emmission of rtp packets
                             -------------------
    begin                : ven mar  30
    copyright            : (C) 2001 by Simon MORLAT
    email                : simon.morlat@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rtp.h"
#include "rtp_private.h"
#include "rtp_ptypes.h"
#include <sys/errno.h>


int rtp_send(RtpSession *handle,void *buffer,int buflen)
{
  int err,i=0;
  rtp_packet_t *rtppacket;
  q_element_t *el;
  char udpbuf[UDP_MAX_SIZE];
  int udp_size=0;

  if (buflen>rtp_length[handle->params.profile]) return(-EINVAL);
  if (handle->snd_firsts)
    {
      handle->snd_last_ts=choose_uint32();
      handle->snd_last_seq=choose_uint16();
      handle->snd_firsts=0;
    }
  handle->snd_last_ts+=handle->snd_tsinc;

  /* find a free element to compose rtp packet */
  while(handle->snd_stack[i].packet_size!=0) i++;

  el=&handle->snd_stack[i];
  el->packet_size=buflen+RTP_FIXED_HEADER_SIZE;
  handle->snd_last_seq++;
  rtppacket=&el->packet;
  rtppacket->ssrc=htonl(handle->params.ssrc);
  rtppacket->timestamp=htonl(handle->snd_last_ts);
  rtppacket->seq_number=htons(handle->snd_last_seq);
  memcpy(rtppacket->data,buffer,buflen);
  /* now this rtp packet is ready.*/
	/* for debug: simulate a lost packet*/
	if (buflen==0) return(0);
  /* send rtp packet in an udp packet, maybe several rtp packet in a single
     udp*/
  if (handle->params.redundancy==0)
    {
    		/*single packet */
    		/* use for simulation of lost packets:*/
    		if (buflen==0) return(0);
      err=sendto(handle->rtp_socket,(void*)rtppacket,el->packet_size,0,&handle->rtp_rem_addr,sizeof(struct sockaddr_in));
      el->packet_size=0;
      if (err==-1) buflen=-errno;
      return(buflen);
    }
  else
    {
    		/*several packets in a single udp */
      /* find availlable packets to send */
      for(i=0;i<=handle->params.redundancy;i++)
	{
	  el=&handle->snd_stack[i];
	  if (el->packet_size!=0)
	    {
	      if ((handle->snd_last_seq- ntohs(el->packet.seq_number))<
		  handle->params.redundancy)
		{
		  /* this packet is good to be sent */
		  memcpy(&udpbuf[udp_size],&el->packet,el->packet_size);
		  udp_size+=el->packet_size;
		}
	      else 
		{
		  memcpy(&udpbuf[udp_size],&el->packet,el->packet_size);
		  udp_size+=el->packet_size;
		  /* this packet is the oldest, it must not be sent anymore*/
		  el->packet_size=0;
		}
	    }
	}
      /* and send them in a single udp*/
      err=send(handle->rtp_socket,(void*)udpbuf,udp_size,0);
      if (err==-1) return(-errno);
      else return(udp_size);
    }
}

