/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "msavdecoder.h"

static MSAVDecoderClass *ms_avdecoder_class=NULL;

MSFilter *ms_h263decoder_new()
{
	return ms_AVdecoder_new_with_codec(&h263_decoder);
}

MSFilter *ms_mpeg_decoder_new()
{
	return ms_AVdecoder_new_with_codec(&mpeg_decoder);
}

MSFilter * ms_AVdecoder_new_with_codec(AVCodec *codec)
{
	MSAVDecoder *enc;

	enc=g_malloc(sizeof(MSAVDecoder));
	if (ms_avdecoder_class==NULL)
	{
		 ms_avdecoder_class=g_malloc(sizeof(MSAVDecoderClass));
		 ms_AVdecoder_class_init(ms_avdecoder_class);
	}
	MS_FILTER(enc)->klass=(MSFilterClass*)ms_avdecoder_class;
	ms_AVdecoder_init(enc,codec);
	return MS_FILTER(enc);
}


void ms_AVdecoder_init(MSAVDecoder *enc, AVCodec *codec)
{
	gint error;
	
	ms_filter_init(MS_FILTER(enc));
	MS_FILTER(enc)->inqueues=enc->q_inputs;
	MS_FILTER(enc)->outqueues=enc->q_outputs;
	error=avcodec_open(&enc->av_context, codec);
	if (error!=0) g_warning("avcodec_open() failed: %i",error);
}

void ms_AVdecoder_class_init(MSAVDecoderClass *klass)
{
	ms_filter_class_init(MS_FILTER_CLASS(klass));
	ms_filter_class_set_name( MS_FILTER_CLASS(klass),"AVdecoder");
	MS_FILTER_CLASS(klass)->max_qinputs=MSAVDECODER_MAX_INPUTS;
	MS_FILTER_CLASS(klass)->max_qoutputs=MSAVDECODER_MAX_INPUTS;
	MS_FILTER_CLASS(klass)->r_maxgran=0;
	MS_FILTER_CLASS(klass)->w_maxgran=0;
	MS_FILTER_CLASS(klass)->destroy=(MSFilterDestroyFunc)ms_AVdecoder_destroy;
	MS_FILTER_CLASS(klass)->process=(MSFilterProcessFunc)ms_AVdecoder_process;
}

void ms_AVdecoder_uninit(MSAVDecoder *enc)
{
	avcodec_close(&enc->av_context);
}
void ms_AVdecoder_destroy( MSAVDecoder *obj)
{
	ms_AVdecoder_uninit(obj);
	g_free(obj);
}

void ms_AVdecoder_process(MSAVDecoder *r)
{
	AVPicture avpict;
	MSQueue *inq,*outq;
	MSMessage *inm,*outm;
	gint error;
	gint osize,got_picture;
	
	inq=r->q_inputs[0];
	outq=r->q_outputs[0];
	
	/* get a picture from the input queue */
	inm=ms_queue_get(inq);
	g_return_if_fail(inm!=NULL);

	
	ms_trace("ms_AVdecoder_process: bitrate=%i",r->av_context.bit_rate);
	osize=avpicture_get_size(r->output_pix_fmt,r->width,r->height);
	outm=ms_message_new(osize); /*REVIST: how to know the output buffer size ? */
	avpicture_fill(&avpict,outm->data,r->output_pix_fmt,r->width,r->height);
    error=avcodec_decode_video(&r->av_context,&avpict,&got_picture,inm->data,inm->size);
    if (error<0) ms_warning("ms_AVdecoder_process: error %i.",error);
    if (got_picture) {
    	ms_queue_put(outq,outm);
    	ms_trace("ms_AVdecoder_process: got_picture=1");
    }
    else{
    	ms_message_destroy(outm);
    	ms_trace("ms_AVdecoder_process: got_picture=0");
    }
    ms_message_destroy(inm);
}
