/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "msavencoder.h"

static MSAVEncoderClass *ms_avencoder_class=NULL;

MSFilter *ms_h263encoder_new()
{
	return ms_AVencoder_new_with_codec(&h263_encoder);
}
MSFilter * ms_AVencoder_new_with_codec(AVCodec *codec)
{
	MSAVEncoder *enc;

	enc=g_malloc(sizeof(MSAVEncoder));
	if (ms_avencoder_class==NULL)
	{
		 ms_avencoder_class=g_malloc(sizeof(MSAVEncoderClass));
		 ms_AVencoder_class_init(ms_avencoder_class);
	}
	MS_FILTER(enc)->klass=(MSFilterClass*)ms_avencoder_class;
	ms_AVencoder_init(enc,codec);
	return MS_FILTER(enc);
}


void ms_AVencoder_init(MSAVEncoder *enc, AVCodec *codec)
{
	gint error;
	
	ms_filter_init(MS_FILTER(enc));
	error=avcodec_open(&enc->av_context, codec);
	if (error!=0) g_warning("avcodec_open() failed: %i",error);
}

void ms_AVencoder_class_init(MSAVEncoderClass *klass)
{
	ms_filter_class_init(MS_FILTER_CLASS(klass));
	MS_FILTER_CLASS(klass)->max_qinputs=MSAVENCODER_MAX_INPUTS;
	MS_FILTER_CLASS(klass)->max_qoutputs=MSAVENCODER_MAX_INPUTS;
	MS_FILTER_CLASS(klass)->r_maxgran=0;
	MS_FILTER_CLASS(klass)->w_maxgran=0;
	MS_FILTER_CLASS(klass)->destroy=(MSFilterDestroyFunc)ms_AVencoder_destroy;
	MS_FILTER_CLASS(klass)->process=(MSFilterProcessFunc)ms_AVencoder_process;
}

void ms_AVencoder_uninit(MSAVEncoder *enc)
{
	avcodec_close(&enc->av_context);
}
void ms_AVencoder_destroy( MSAVEncoder *obj)
{
	ms_AVencoder_uninit(obj);
	g_free(obj);
}

void ms_AVencoder_process(MSAVEncoder *r)
{
	AVPicture avpict;
	MSQueue *inq,*outq;
	MSMessage *inm,*outm;
	gint error;
	gint osize;
	
	inq=r->q_inputs[0];
	outq=r->q_outputs[0];
	
	/* get a picture from the input queue */
	inm=ms_queue_get(inq);
	g_return_if_fail(inm!=NULL);

	avpicture_fill(&avpict,inm->data,r->input_pix_fmt,r->width,r->height);
	ms_trace("ms_AVencoder_process: bitrate=%i",r->av_context.bit_rate);
	osize=ms_message_size(inm);
	outm=ms_message_new(osize); /*REVIST: how to know the output buffer size ? */
    error=avcodec_encode_video(&r->av_context,outm->data,osize,&avpict);
    if (error!=0) ms_warning("ms_AVencoder_process: error %i.",error);

    ms_message_destroy(inm);
}

