/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "msossread.h"
#include "mssync.h"
#include <unistd.h>
#include <errno.h>

static MSOssReadClass *msossreadclass=NULL;

MSFilter * ms_oss_read_new(gint devid)
{
	MSOssRead *w;
	
  if (msossreadclass==NULL)
	{
		msossreadclass=g_new(MSOssReadClass,1);
		ms_oss_read_class_init( msossreadclass );
	}
	/* it must exists only one MSOssRead object */
	if (msossreadclass->dev_instances[devid]>0) return NULL;
	w=g_new(MSOssRead,1);
	MS_FILTER(w)->klass=MS_FILTER_CLASS(msossreadclass);
	ms_oss_read_init(w);
	w->devid=devid;
	MS_OSS_READ_CLASS(MS_FILTER(w)->klass)->dev_instances[devid]++;
	return(MS_FILTER(w));
}

/* FOR INTERNAL USE*/
void ms_oss_read_init(MSOssRead *w)
{
	ms_filter_init(MS_FILTER(w));
	MS_FILTER(w)->outfifos=w->f_outputs;
	MS_FILTER(w)->outfifos[0]=NULL;
}
	

void ms_oss_read_class_init(MSOssReadClass *klass)
{
	int i;
	ms_filter_class_init(MS_FILTER_CLASS(klass));
	MS_FILTER_CLASS(klass)->max_foutputs=1;  /* one fifo output only */
	MS_FILTER_CLASS(klass)->process= (MSFilterProcessFunc)ms_oss_read_process;
	MS_FILTER_CLASS(klass)->destroy= (MSFilterDestroyFunc)ms_oss_read_destroy;
	MS_FILTER_CLASS(klass)->w_maxgran=MS_OSS_READ_DEF_GRAN;
	ms_filter_class_set_name(MS_FILTER_CLASS(klass),"OssRead");
	ms_filter_class_set_attr( MS_FILTER_CLASS(klass),FILTER_CAN_SYNC|FILTER_IS_SOURCE);
	klass->oss_manager=&oss_mgr;
	for(i=0;i<MSOSS_MAX_DEVICES;i++) klass->dev_instances[i]=0;
}

void ms_oss_read_destroy( MSOssRead *obj)
{
	MS_OSS_READ_CLASS(MS_FILTER(obj)->klass)->dev_instances[obj->devid]--;
	g_free(obj);
}

void ms_oss_read_process(MSOssRead *f)
{
	MSFifo *fifo;
	void *p;
	gint fd=ms_oss_read_getfd(f);
	gint gran=f->bsize;
	int bytesread;
	if (fd>0)
	{
		fifo=f->f_outputs[0];
		ms_fifo_get_write_ptr(fifo,gran,&p);
		
		bytesread=read(fd,p,gran);
		if (bytesread!=gran){
			if (bytesread<0) g_warning("ms_oss_read_process: Read failed: %s\n",strerror(errno));
			else {
				gint index=0;
				gint cond=gran-bytesread;
				/* fix for some obscure drivers (in BSD) that don't return the number of bytes that they should */
   				while(cond>0){
   					bytesread=read(fd,p+index,cond);
   					index+=bytesread;
   					cond=cond-bytesread;
   				}
   			}
   		}
	}
}

void ms_oss_read_start(MSOssRead *r)
{
	g_return_if_fail(MS_FILTER(r)->sync!=NULL);
	/* open the device for an audio telephony signal with minimum latency */
	ms_oss_opendev(MS_OSS_READ_CLASS(MS_FILTER(r)->klass)->oss_manager,r->devid,16,0,8000,256);
	r->bsize=ms_oss_read_getbsize(r);
	ms_sync_set_samples_per_tick(MS_FILTER(r)->sync,r->bsize);
}

void ms_oss_read_stop(MSOssRead *w)
{
	ms_sync_set_samples_per_tick(MS_FILTER(w)->sync,0); 	/* very bad !*/
	/* this blocks the sync (all filter chains atached to it !)*/
	/* it would be better that the chain be removed of sync work list */
	/* using ms_sync_dettach() */
	ms_oss_closedev(MS_OSS_READ_CLASS(MS_FILTER(w)->klass)->oss_manager,w->devid);
}

void ms_oss_read_set_level(MSOssRead *w,gint a)
{
	ms_oss_set_rlevel(MS_OSS_READ_CLASS(MS_FILTER(w)->klass)->oss_manager,
			w->devid,a);
}

