/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "msosswrite.h"
#include "mssync.h"
#include <unistd.h>

static MSOssWriteClass *msosswriteclass=NULL;

MSFilter * ms_oss_write_new(gint devid)
{
	MSOssWrite *w;
	
  if (msosswriteclass==NULL)
	{
		msosswriteclass=g_new(MSOssWriteClass,1);
		ms_oss_write_class_init( msosswriteclass );
	}
	/* it must exists only one MSOssWrite object */
	if (msosswriteclass->dev_instances[devid]>0) return NULL;
	w=g_new(MSOssWrite,1);
	MS_FILTER(w)->klass=MS_FILTER_CLASS(msosswriteclass);
	ms_oss_write_init(w);
	w->devid=devid;
	MS_OSS_WRITE_CLASS(MS_FILTER(w)->klass)->dev_instances[devid]++;
	return(MS_FILTER(w));
}

/* FOR INTERNAL USE*/
void ms_oss_write_init(MSOssWrite *w)
{
	ms_filter_init(MS_FILTER(w));
	MS_FILTER(w)->infifos=w->f_inputs;
	MS_FILTER(w)->infifos[0]=NULL;
	MS_FILTER(w)->r_mingran=256;  /* very few cards can do that...*/
}
	

void ms_oss_write_class_init(MSOssWriteClass *klass)
{
	int i;
	ms_filter_class_init(MS_FILTER_CLASS(klass));
	MS_FILTER_CLASS(klass)->max_finputs=1;  /* one fifo input only */
	MS_FILTER_CLASS(klass)->r_maxgran=MS_OSS_WRITE_DEF_GRAN;
	MS_FILTER_CLASS(klass)->process= (MSFilterProcessFunc)ms_oss_write_process;
	MS_FILTER_CLASS(klass)->destroy= (MSFilterDestroyFunc)ms_oss_write_destroy;
	ms_filter_class_set_name(MS_FILTER_CLASS(klass),"OssWrite");
	ms_filter_class_set_attr( MS_FILTER_CLASS(klass),FILTER_CAN_SYNC);
	klass->oss_manager=&oss_mgr;
	for(i=0;i<MSOSS_MAX_DEVICES;i++) klass->dev_instances[i]=0;
}

void ms_oss_write_destroy( MSOssWrite *obj)
{
	MS_OSS_WRITE_CLASS(MS_FILTER(obj)->klass)->dev_instances[obj->devid]--;
	g_free(obj);
}

void ms_oss_write_process(MSOssWrite *f)
{
	MSFifo *fifo;
	void *p;
	gint fd=ms_oss_write_getfd(f);
	gint gran=ms_filter_get_mingran(MS_FILTER(f));
	if (fd>0)
	{
		fifo=f->f_inputs[0];
		ms_fifo_get_read_ptr(fifo,gran,&p);
		write(fd,p,gran);
	}
}

void ms_oss_write_start(MSOssWrite *w)
{
	gint bsize;
	g_return_if_fail(MS_FILTER(w)->sync!=NULL);
	/* open the device for an audio telephony signal with minimum latency */
	ms_oss_opendev(MS_OSS_WRITE_CLASS(MS_FILTER(w)->klass)->oss_manager,w->devid,16,0,8000,256);
	bsize=ms_oss_write_getbsize(w);
	ms_filter_set_mingran(MS_FILTER(w),bsize);
	ms_sync_set_samples_per_tick(MS_FILTER(w)->sync,bsize);
}

void ms_oss_write_stop(MSOssWrite *w)
{
	ms_sync_set_samples_per_tick(MS_FILTER(w)->sync,0);
	ms_oss_closedev(MS_OSS_WRITE_CLASS(MS_FILTER(w)->klass)->oss_manager,w->devid);
}

void ms_oss_write_set_level(MSOssWrite *w,gint a)
{
	ms_oss_set_plevel(MS_OSS_WRITE_CLASS(MS_FILTER(w)->klass)->oss_manager,
			w->devid,a);
}
