/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "msread.h"
#include "mssync.h"
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <string.h>
#include <errno.h>

static MSReadClass *ms_read_class=NULL;

MSFilter * ms_read_new(char *name)
{
	MSRead *r;
	int fd=-1;
	
	r=g_new(MSRead,1);
	ms_read_init(r);
	if (ms_read_class==NULL)
	{
		ms_read_class=g_new(MSReadClass,1);
		ms_read_class_init(ms_read_class);
	}
	MS_FILTER(r)->klass=MS_FILTER_CLASS(ms_read_class);
	if ((name!=NULL) && (strlen(name)!=0))
	{
		fd=open(name,O_RDONLY);
		if (fd<0) g_error("ms_read_new: failed to open %s.\n",name);
	}else g_warning("ms_read_new: Bad file name");
	r->fd=fd;
	return(MS_FILTER(r));
}
	

/* FOR INTERNAL USE*/
void ms_read_init(MSRead *r)
{
	ms_filter_init(MS_FILTER(r));
	MS_FILTER(r)->outfifos=r->foutputs;
	MS_FILTER(r)->outqueues=r->qoutputs;
	memset(r->foutputs,0,sizeof(MSFifo*)*MSREAD_MAX_OUTPUTS);
	memset(r->qoutputs,0,sizeof(MSQueue*)*MSREAD_MAX_OUTPUTS);
	r->fd=-1;
	r->gran=MSREAD_DEF_GRAN;
}

void ms_read_class_init(MSReadClass *klass)
{
	ms_filter_class_init(MS_FILTER_CLASS(klass));
	ms_filter_class_set_name(MS_FILTER_CLASS(klass),"dskreader");
	ms_filter_class_set_attr(MS_FILTER_CLASS(klass),FILTER_IS_SOURCE);
	MS_FILTER_CLASS(klass)->max_foutputs=MSREAD_MAX_OUTPUTS;
	MS_FILTER_CLASS(klass)->max_qoutputs=MSREAD_MAX_OUTPUTS;
	MS_FILTER_CLASS(klass)->w_maxgran=MSREAD_DEF_GRAN;
	MS_FILTER_CLASS(klass)->destroy=(MSFilterDestroyFunc)ms_read_destroy;
	MS_FILTER_CLASS(klass)->process=(MSFilterProcessFunc)ms_read_process;
}
	
void ms_read_process(MSRead *r)
{
	MSFifo *f;
	MSQueue *q;
	MSMessage *msg=NULL;
	int i,j,err;
	gint gran=ms_sync_get_samples_per_tick(MS_SYNC( MS_FILTER(r)->sync));
	void *p,*fp=NULL;
	
	
	/* process output fifos*/
	for (i=0,j=0;(i<MS_FILTER(r)->klass->max_foutputs)&&(j<MS_FILTER(r)->foutputs);i++)
	{
		f=r->foutputs[i];
		if (f!=NULL)
		{
			err=ms_fifo_get_write_ptr(f,gran,&p);
			if (err>0)
			{
				if (j==0)
				{
					err=read(r->fd,p,gran);
					fp=p;
					if (err==0)
					{
						ms_sync_set_samples_per_tick(MS_FILTER(r)->sync,0);
						g_message("DiskReader: end of file.");
					}
					else if (err<0)
					{
						/* temp: */
						g_error("ms_read_process: failed to read: %s.\n",strerror(errno));
					}
				}
				else 
				{	/* copy from the first fifo*/
					memcpy(p,fp,gran);
				}
			}
			j++;
		}
	}
	/* process output queues*/
	for (i=0,j=0;(i<MS_FILTER(r)->klass->max_qoutputs)&&(j<MS_FILTER(r)->qoutputs);i++)
	{
		q=r->qoutputs[i];
		if (q!=NULL)
		{
			if (j==0) 
			{
				msg=ms_message_new(r->gran);
				read(r->fd,msg->data,r->gran);
			}
			ms_queue_put(q,msg);
			j++;	
		}				
	}	
}

void ms_read_destroy( MSRead *obj)
{
	if (obj->fd!=0) close(obj->fd);
	g_free(obj);
}

