   /*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ms.h"

#include "mswrite.h"
#include "msread.h"
#include "msavencoder.h"
#include "msnosync.h"

#include <signal.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>

static int cond=1;

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

void stop_handler(int signum)
{
	cond=0;
}
#ifdef HAVE_AVCODEC

int main()
{
	MSFilter *play,*enc,*rec;
	MSSync *dummysync;	

	ms_init();
	signal(SIGINT,stop_handler);
	
	play=ms_read_new("/tmp/video.rgb");
	rec=ms_write_new("/tmp/video.h263");
	enc=ms_h263encoder_new();
	dummysync=ms_nosync_new();

	ms_filter_add_link(play,0,enc,0,LINK_QUEUE);
	ms_filter_add_link(enc,0,rec,0,LINK_QUEUE);
	ms_sync_attach(dummysync,play);

	ms_start(dummysync);
	ms_AVencoder_set_pix_fmt(MS_AVENCODER(enc),PIX_FMT_RGB24);
	ms_AVencoder_set_width(MS_AVENCODER(enc),320);
	ms_AVencoder_set_height(MS_AVENCODER(enc),160);
	while(cond)
	{
		sleep(1);
	}
	printf("stoping sync...\n");
	ms_stop(dummysync);
	printf("unlinking filters...\n");
	ms_filter_remove_link(play,0,enc,0,LINK_FIFO);
	ms_filter_remove_link(enc,0,rec,0,LINK_FIFO);
	printf("destroying filters...\n");
	ms_filter_destroy(play);
	ms_filter_destroy(enc);
	ms_filter_destroy(rec);
	return 0;
}
#else

int main()
{
	printf("AVcodec support is not compiled.\n");
	return 0;
}
#endif
