  /*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ms.h"

#include "msosswrite.h"
#include "msossread.h"
#include "mscopy.h"
#include "msnosync.h"
#include "msGSMdecoder.h"
#include "msGSMencoder.h"

#include <signal.h>
#include <sys/types.h>
#include <unistd.h>

static int cond=1;

void stop_handler(int signum)
{
	cond=0;
}

int main()
{
	MSFilter *play,*enc,*dec,*rec;
	MSSync *dummysync;	

	ms_init();
	signal(SIGINT,stop_handler);
	
	play=ms_oss_read_new(0);
	rec=ms_oss_write_new(0);
	enc=ms_GSMencoder_new();
	dec=ms_GSMdecoder_new();
	dummysync=ms_nosync_new();

	ms_filter_add_link(play,0,enc,0,LINK_FIFO);
	ms_filter_add_link(enc,0,dec,0,LINK_FIFO);
	ms_filter_add_link(dec,0,rec,0,LINK_FIFO);
	ms_sync_attach(dummysync,play);

	ms_start(dummysync);
	ms_oss_read_start(MS_OSS_READ(play));
	ms_oss_write_start(MS_OSS_WRITE(rec));
	while(0)
	{
		sleep(1);
	}
	sleep(5);
	ms_oss_read_stop(MS_OSS_READ(play));
	ms_oss_write_stop(MS_OSS_WRITE(rec));
	printf("stoping sync...\n");
	ms_stop(dummysync);
	printf("unlinking filters...\n");
	ms_filter_remove_link(play,0,enc,0,LINK_FIFO);
	ms_filter_remove_link(enc,0,dec,0,LINK_FIFO);
	ms_filter_remove_link(dec,0,rec,0,LINK_FIFO);
	printf("destroying filters...\n");
	ms_filter_destroy(play);
	ms_filter_destroy(enc);
	ms_filter_destroy(dec);
	ms_filter_destroy(rec);
	return 0;
}
