/*
  The oRTP library implements the RTP protocol (rfc1889).
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#include "rtp.h"
#include "rtpsession.h"


void rtp_parse(RtpSession *session,mblk_t *mp)
{
	gint i;
	mblk_t *mdata;
	rtp_header_t *rtp;
	
	g_return_if_fail(mp->b_datap!=NULL);
	
	rtp=(rtp_header_t*)mp->b_rptr;
	if (rtp->version!=2)
	{
		g_message("Receiving rtp packet with version number !=2...discarded");
		freemsg(mp);
		return;
	}
	if (session->ssrc!=0)
	{
		/*the ssrc is set, so we must check it */
		if (session->ssrc!=rtp->ssrc){
			g_message("rtp_parse: bad ssrc - %i",rtp->ssrc);
			
		}
	}
	/* convert all header data from network order to host order */
	rtp->seq_number=ntohs(rtp->seq_number);
	rtp->timestamp=ntohl(rtp->timestamp);
	rtp->ssrc=ntohl(rtp->ssrc);
	/* convert csrc if necessary */
	for (i=0;i<rtp->cc;i++)
		rtp->csrc[i]=ntohl(rtp->csrc[i]);
	
	mp->b_wptr=mp->b_rptr+RTP_FIXED_HEADER_SIZE+ (4*rtp->cc);
	/* creates a M_DATA mblk_t to be linked with the header (M_PROTO) */
	mdata=dupb(mp);
	mdata->b_rptr+=RTP_FIXED_HEADER_SIZE+ (4*rtp->cc);
	/* link proto with data */
	mp->b_cont=mdata;
	/* and then add the packet to the queue */
	rtp_putq(session->rtp.rq,mp);
	/* make some checks: q size must not exceed RtpStream::max_rq_size */
	if (session->rtp.rq->q_count> session->rtp.max_rq_size)
	{
		/* remove the oldest mblk_t */
		mp=getq(session->rtp.rq);
		g_message("rtp_putq: Queue is full. Discarding message with ts=%i",((rtp_header_t*)mp->b_rptr)->timestamp);
		freemsg(mp);
	}
}



















