 /*
  The oRTP LinPhone RTP library intends to provide basics for a RTP stack.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef RTPSESSION_H
#define RTPSESSION_H


#include "../config.h"

#include "rtp.h"


#ifndef TARGET_IS_HPUXKERNEL

#include <sys/errno.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/time.h>
#include <stdio.h>
#include "str_utils.h"

#else
/* include the HPUX streams header*/
#include <sys/stream.h>
#include <sys/socket.h>
#include <netinet/in.h> 
#endif

typedef struct _RtpStream
{
	gint socket;
	struct timeval *timeout;
	struct timeval _timeout;
	gint jitt_comp;   /* the user jitt_comp in miliseconds*/
	gint jitt_comp_time; /* the jitt_comp converted in rtp time (same unit as timestamp) */
	gint max_rq_size;
	queue_t _rq;  /* should not be accessed direclty, use RtpStream::rq instead */
	queue_t _wq;  /* should not be accessed direclty, use RtpStream::wq instead */
	queue_t *rq;
	queue_t *wq;
	struct sockaddr_in loc_addr;
	struct sockaddr_in rem_addr;
	guint32 snd_ts_offset;
	guint32 snd_ts;
	guint32 rcv_ts_offset;
	guint32 rcv_ts;
	guint16 rcv_seq;
	guint16 snd_seq;
	rtp_stats_t stats;
}RtpStream;

typedef struct _RtcpStream
{
	gint socket;
	queue_t rq;
	queue_t wq;
	struct sockaddr_in loc_addr;
	struct sockaddr_in rem_addr;
} RtcpStream;

struct _RtpSession
{
	guint32 ssrc;
	gint profile;
	gint highest_fd;
	fd_set scanfd;
	RtpStream rtp;
	RtcpStream rtcp;
	guint32 flags;
#define RTP_SESSION_NO_SYNC (1)  /*means that the caller process uses its own synchronisation*/
#define RTP_SESSION_SEND (1<<1)
#define RTP_SESSION_RECV (1<<2)
#define RTP_SESSION_DUPLEX (1<<3)
#define RTP_SESSION_IS_BLOCKING  (1<<4)
/* private flags*/
#define RTP_SESSION_IS_STARTING  (1<<5)

};
	
typedef struct _RtpSession RtpSession;

void ortp_init();
#ifndef TARGET_IS_HPUXKERNEL
void ortp_set_debug_file(FILE *file);
#endif

void rtp_session_init(RtpSession *session, gint type);
/* for use in the HPUX kernel */
#define rtp_session_set_rq(s,q) (s)->rtp.rq=(q)
#define rtp_session_set_wq(s,q) (s)->rtp.wq=(q)

RtpSession *rtp_session_new(gint type);
#define rtp_session_set_flag(session,flag) (session)->flags|=(flag)
#define rtp_session_unset_flag(session,flag) (session)->flags&=~(flag)
int rtp_session_set_ssrc(RtpSession *session, guint32 ssrc);
int rtp_session_set_jitter_compensation(RtpSession *session, int milisec);
#ifndef TARGET_IS_HPUXKERNEL
/* setting a local address is not provided for the HPUX kernel environement */
int rtp_session_set_local_addr(RtpSession *session,gchar *addr, gint port);
#endif

int rtp_session_set_remote_addr(RtpSession *session,gchar *addr, gint port);
#define rtp_session_set_profile(session,_profile) (session)->profile=(_profile)
void rtp_session_set_timeout(RtpSession *session,guint timeout);
int rtp_session_recv_with_ts(RtpSession *session, gchar *buffer, gint len, guint32 time);
int rtp_session_send_with_ts(RtpSession *session, gchar *buffer, gint len, guint32 time);
int rtp_session_send(RtpSession *session, gchar *buffer, gint len);
int rtp_session_recv(RtpSession *session, gchar *buffer, gint len);
void rtp_session_uninit(RtpSession *session);
void rtp_session_destroy(RtpSession *session);



void rtp_parse(RtpSession *session, mblk_t *mp);
#endif
