/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <errno.h>
#include <osip/osip.h>
#include <osip/sdp.h>
#include "osipua.h"
#include "utils.h"
#include "uatransaction.h"

void cb_killtransaction(transaction_t *tr)
{
	osip_trace(TRACE_LEVEL2,("Transaction %i killed.\n",tr->transactionid));
	ua_transaction_free(tr);
}

void cb_endoftransaction(transaction_t *tr)
{
	osip_trace(TRACE_LEVEL2,("Transaction %i finished.\n",tr->transactionid));
	ua_transaction_free(tr);
}

void cb_connection_refused(transaction_t *trn)
{
	OsipCallLeg *call;
	OsipUA *ua;
	int error=-ECONNREFUSED;
	osip_trace(TRACE_LEVEL0,("OnEvent_connection_refused! \n"));
	call=osip_call_leg_find_with_tid(trn->transactionid);
	if (call==NULL)
	{
		osip_trace(TRACE_LEVEL2,("Connection refused for an inexistant call leg! \n"));
		return ;
	}
	ua=(OsipUA*)call->ua;
	if (ua->faillure!=NULL)
		ua->faillure(call,NULL,&error);

  return ; /* ok */
}

void cb_network_error(transaction_t *trn)
{
  OsipCallLeg *call;
	OsipUA *ua;
	int error=-errno;
	osip_trace(TRACE_LEVEL0,("OnEvent_connection_refused! \n"));
	call=osip_call_leg_find_with_tid(trn->transactionid);
	if (call==NULL)
	{
		osip_trace(TRACE_LEVEL2,("Connection refused for an inexistant call leg! \n"));
		return ;
	}
	ua=(OsipUA*)call->ua;
	if (ua->faillure!=NULL)
		ua->faillure(call,NULL,&error);

  return ; /* ok */
}


void cb_rcv1xx(sipevent_t *sipevent,transaction_t *trn)
{
	OsipCallLeg *call;
	OsipUA *ua;
	int inf;
   char *strinf;
	
  	osip_trace(TRACE_LEVEL1,("OnEvent_New_Incoming1xxResponse!\n"));

	call=ua_transaction_get_call_leg(trn);
	if (call==NULL)
	{
		osip_trace(TRACE_LEVEL2,("1xx response for an inexistant call leg! \n"));
		return ;
	}
	strinf=sipevent->sip->strtline->statuscode;
	if (strinf!=NULL) inf=satoi(strinf);
	ua=(OsipUA*)call->ua;
	if (ua->informative!=NULL)
		ua->informative(call,sipevent->sip,&inf);
	
  return ; /* ok */
}

void cb_rcv2xx(sipevent_t *sipevent, transaction_t *trn)
{
	sip_t *sipmsg=sipevent->sip;
	OsipCallLeg *call;
	OsipUA *ua;
	int err,i;
	MediaDesc *desc;
	sdp_t *remote_sdp=NULL;
	body_t *body;
	
	
	osip_trace(TRACE_LEVEL1,("OnEvent_New_Incoming2xxResponse!\n"));

  call=ua_transaction_get_call_leg(trn);
	if (call!=NULL)
	{
		osip_call_leg_update_from_response(call,sipevent->sip);
		/* an existing call-leg was found, so process it */
		ua=osip_call_leg_get_ua(call);
		
		if (MSG_IS_INVITE(trn->lastrequest))
		{
			call->status=CALL_RUNNING;
   			/* get first body, hoping it is a sdp*/
      	body=(body_t*)list_get(sipmsg->bodies,0);
       if (body!=NULL)
			{
				/* we must check if there is a sdp, and check if it is compatible*/
         	remote_sdp=smalloc(sizeof(sdp_t));
			  	sdp_init(remote_sdp);
				err=sdp_parse(remote_sdp,body->body);
      		if (err==-1) sfree(remote_sdp);
			} else err=-1;
			if (err==-1)
			{
				osip_trace(TRACE_LEVEL2,("Syntax error in sdp or sdp not found.\n"));
				 				/* we should return something here TODO (maybe BAD REQUEST)*/ 	
				return ;
			}
			err=osip_call_leg_extract_media(call,remote_sdp);
			if (err==-1)
			{
				osip_trace(TRACE_LEVEL2,("Incompatible media description in 200 OK!!!!\n"));
				return ;
			}
			/* check if there is one profile for each media stream*/
			for (i=0;i<call->media_list->nb_elt;i++)
			{
				desc=(MediaDesc*) list_get(call->media_list,i);
				if (desc->profile_list->nb_elt==0)
				{
					osip_trace(TRACE_LEVEL2,("No profile found in 200 OK!!!!\n"));
					return ;
				}
				else if (desc->profile_list->nb_elt>1)
				{
					osip_trace(TRACE_LEVEL2,("Warning: more than one profile in 200 OK!!!!\n"));
				}
			}
			
			/* indicate the user of the acceptation of the invite */
			if (ua->invite_accepted!=NULL)
				ua->invite_accepted(call,sipmsg,(void*)sipmsg);
			/* respond ACK to the server */
			osip_call_leg_ack(call,trn);
			osip_trace(TRACE_LEVEL2,("Sending ACK!\n"));
		}
		else if (MSG_IS_REGISTER(trn->lastrequest))
		{
			int inf=200;
			char *tmp;
			RegistrationCtxt *ctx=call->reg_context;
			
			/* add  the address of records to the list of aliases if the expire!=0 else try to remove it */
			contact_2char( (contact_t*)trn->lastrequest->to,&tmp);
			if (ctx!=NULL){
				if (ctx->expires!=0) osip_ua_add_alias(ua,tmp);
				else osip_ua_remove_alias(ua,tmp);
				sfree(tmp);
				if (ua->informative!=NULL)
					ua->informative(call,sipevent->sip,(void*)&inf);
				/* destroys the registration context (well dec the ref count) */
				call->reg_context==NULL;
				registration_ctxt_destroy(ctx);
			}
		}
	}
	else
	{
		osip_trace(TRACE_LEVEL1,("200 OK for an inexistant call-leg !\n"));
	}			
  return ; /* ok */

}

void cb_rcv3xx(sipevent_t *sipevent,transaction_t *trn)
{
	char *strinf,*tmp;
	int code=0,err;
	OsipCallLeg *call;
	OsipUA *ua;
	contact_t *contact=NULL;

	osip_trace(TRACE_LEVEL1,("OnEvent_New_Incoming3xxResponse!\n"));
	call=ua_transaction_get_call_leg(trn);
	if (call==NULL)
	{
		osip_trace(TRACE_LEVEL2,("3xx response for an inexistant call leg! \n"));
		return ;
	}
	ua=osip_call_leg_get_ua(call);
	strinf=sipevent->sip->strtline->statuscode;
	if (strinf!=NULL) code=satoi(strinf);
	osip_trace(TRACE_LEVEL2,("Sending ack.\n"));
	osip_call_leg_ack(call,trn);
	if (ua->informative!=NULL)
		ua->informative(call,sipevent->sip,&code);
	switch(code)
	{
	case 300: /*Multiples choices*/
		break;
	case 301: /*Moved permanently */
		break;
	case 302: /*Moved temporarily */
		osip_trace(TRACE_LEVEL1,("User has moved temporarily.\n"));
		/* get first contact field*/
		err=msg_getcontact(sipevent->sip,0, &contact);
		if (contact!=NULL)
		{  /* do redirection*/
			call->status=CALL_NEW;
			/* remove params like action=redirect*/
			listofchar_free(contact->gen_params);
			contact_2char(contact,&tmp);
			osip_call_leg_invite(call,tmp);
			sfree(tmp);
		}	
		break;
	case 305: /* Use proxy */
		break;
	}
	
	/* then we invite using the address in the contact header*/
	return ; /* ok */
}

void cb_rcv4xx(sipevent_t *sipevent,transaction_t *trn)
{
  /* this is to avoid infinite loop upon registration always refused
     with 401 or 407... */
  static time_t time_of_last_authentication = 0;
  time_t current_time;
	OsipCallLeg *call;
	OsipUA *ua;
	char *strerr;
	int error;
	
	osip_trace(TRACE_LEVEL1,("OnEvent_New_Incoming4xxResponse!\n"));
	call=ua_transaction_get_call_leg(trn);
	if (call==NULL)
	{
		osip_trace(TRACE_LEVEL2,("4xx response for an inexistant call leg! \n"));
		return ;
	}
	ua=osip_call_leg_get_ua(call);
	/* patch from Aymeric: for linphone-0.5.0
        Not only 2xx requests for INVITE need ACK! 3xx,4xx,5xx and 6xx
          also need it.
       */
	if (MSG_IS_INVITE(trn->lastrequest))
	{
    osip_trace(TRACE_LEVEL2,("Sending ACK!\n"));
    osip_call_leg_ack(call,trn);
	}
	strerr=sipevent->sip->strtline->statuscode;
	if (strerr!=NULL) error=satoi(strerr);
	switch(error)
	  {
	  case 407: /*Multiples choices*/
	  case 401: /*Multiples choices*/

	    /* THIS IS USED TO AVOID REPEATING BAD REQUEST WHICH MAKES A LOOP */
	    current_time = time(NULL);
	    if (time_of_last_authentication==0)
	      time_of_last_authentication=current_time;
	    else
	      if (current_time-time_of_last_authentication<10)
		{ /* stop answering because the second authentication failed */
		  time_of_last_authentication=0; /* next auth will be done */
		  return ; /* We have failed to authenticate twice in the
				last */
		}
	  
	    time_of_last_authentication = current_time;
	    if (MSG_IS_INVITE(trn->lastrequest))
	      {
		osip_trace(TRACE_LEVEL1,("User need to authenticate to INVITE!\n"));
		call->status=CALL_NEW;
		osip_call_leg_invite_with_authentication(call,sipevent->sip);
	      }
	    else if (MSG_IS_REGISTER(trn->lastrequest))
	      {
		osip_trace(TRACE_LEVEL1,("User need to authenticate to REGISTER!\n"));
		call->status=CALL_NEW;
		osip_call_leg_register_with_authentication(call,sipevent->sip);
	      }
	    else
	      {
		osip_trace(TRACE_LEVEL1,("User need to authenticate to this method? NOT IMPLEMENTED!\n"));
		break;
	      }
	    return ;
	  }

	if (ua->faillure!=NULL)
		ua->faillure(call,sipevent->sip,&error);
	
	osip_call_leg_release(call);
  return ; /* ok */
}

void cb_rcv5xx(sipevent_t *sipevent,transaction_t *trn)
{
	OsipCallLeg *call;
	OsipUA *ua;
	char *strerr;
	int error;
	
	osip_trace(TRACE_LEVEL1,("OnEvent_New_Incoming5xxResponse!\n"));
	call=ua_transaction_get_call_leg(trn);
	if (call==NULL)
	{
		osip_trace(TRACE_LEVEL2,("5xx response for an inexistant call leg! \n"));
		return ;
	}
	ua=osip_call_leg_get_ua(call);
	/* patch from Aymeric: for linphone-0.5.0
        Not only 2xx requests for INVITE need ACK! 3xx,4xx,5xx and 6xx
          also need it.
       */
	if (MSG_IS_INVITE(trn->lastrequest))
	{
		osip_call_leg_ack(call,trn);
    osip_trace(TRACE_LEVEL2,("Sending ACK!\n"));
	}
	strerr=sipevent->sip->strtline->statuscode;
	if (strerr!=NULL) error=satoi(strerr);
	if (ua->faillure!=NULL)
		ua->faillure(call,sipevent->sip,&error);
	osip_call_leg_release(call);
  return ; /* ok */
}

void cb_rcv6xx(sipevent_t *sipevent,transaction_t *trn)
{	
	OsipCallLeg *call;
	OsipUA *ua;
	char *strerr;
	int error;
	
	osip_trace(TRACE_LEVEL1,("OnEvent_New_Incoming6xxResponse!\n"));
	call=ua_transaction_get_call_leg(trn);
	if (call==NULL)
	{
		osip_trace(TRACE_LEVEL2,("6xx response for an inexistant call leg! \n"));
		return ;
	}
	ua=osip_call_leg_get_ua(call);
	/* patch from Aymeric: for linphone-0.5.0
        Not only 2xx requests for INVITE need ACK! 3xx,4xx,5xx and 6xx
          also need it.
       */
	if (MSG_IS_INVITE(trn->lastrequest))
	{
		
    osip_trace(TRACE_LEVEL2,("Sending ACK!\n"));
    osip_call_leg_ack(call,trn);
	}
	strerr=sipevent->sip->strtline->statuscode;
	if (strerr!=NULL) error=satoi(strerr);
	if (ua->faillure!=NULL)
		ua->faillure(call,sipevent->sip,&error);
	osip_call_leg_release(call);
  return ; /* ok */
}
