/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <osip/osip.h>
#include <osip/sdp.h>

#include "osipua.h"
#include "osipmanager.h"
#include "resolver.h"
#include "utils.h"
#include "uatransaction.h"



void cb_rcvinvite(sipevent_t *sipevent,transaction_t *trn)
{
	int err;
	sip_t *sipmsg=sipevent->sip;
	OsipCallLeg *call;
	OsipUA *ua;
	sdp_t *remote_sdp;
	body_t *body;
	

  osip_trace(TRACE_LEVEL1,("OnEvent_New_IncomingInvite!\n"));
  call=osip_call_leg_find(sipmsg);
	if (call!=NULL)
	{
		/* an existing call-leg was found, so process it */
		osip_trace(TRACE_LEVEL1,("Receiving new invite for an existent call-leg!\n"));
		call->cseq_number++;
	}
	else
	{
		/* send a 100 trying */
		osip_trace(TRACE_LEVEL2,("Sending 100 trying.\n"));
		respond_to_request(def_manager->config, trn,100,NULL);
		/* we must find a user agent interested by this new call */
		call=osip_call_leg_new_from_incoming_trn(trn);
		if (call!=NULL)
 	  { /* the call-leg was created, there is a user agent accepting it */
 	  	ua=osip_call_leg_get_ua(call);
 	    /* get first body, hoping it is a sdp.... more
 	       things to do here*/
 	    if (ua->presence_mode!=200)
 	      /* if the ua has presence features... */
 	    {
 		    sip_t *response;
 		    char *reason;
 		
 		    msg_init(&response);
 		    reason = msg_getreason(ua->presence_mode);
 		
 		    if (1==msg_makereply(ua->presence_mode,reason,
 					 sipevent->sip,
 					 response,
 					 NULL, /*local_body*/
 					 NULL, /*apptype */
 					 0))
 		      {
 						char *val;
 						
 						sfree(reason);
 						if ((ua->presence_mode==486
 			     		||ua->presence_mode==480
 			     		||ua->presence_mode==600)
 			   			 &&ua->presence_delay>0)
 			  		{
 			    		val = smalloc(8);
 			    		sprintf(val,"%i",ua->presence_delay);
 			    		msg_setheader(response,"Retry-After",val);
 			    		free(val);
 			  		}
 						if ((ua->presence_mode==302	||ua->presence_mode==380)		
 			    		&& ua->presence_contact_url!=NULL)
 			  		{
 			    		int i;
 			    		i = msg_setcontact(response,ua->presence_contact_url);
 			    		if (i!=0)
					  osip_trace(TRACE_LEVEL0,( "Error in contact url: %s\n",ua->presence_contact_url));
 			  		}
//				if (ua->presence_mode==380				
//				    &&ua->presence_contact_url!=NULL)
//				{
//					/* the contact may not be a sip url. As osip does not undestand other
//					things that sip url for the moment, we can set contact header in
//					the basic way:*/
//					msg_setheader(response,"Contact",ua->presence_contact_url);
//				}
 						osip_call_leg_send_response(call,trn,response);
 		      }
 		    	else
 		      {
 						osip_trace(TRACE_LEVEL1,("Could not create response for current status\n"));
 						sfree(reason);
 		      }
 		    return ;
 		  }
		
			body=list_get(sipmsg->bodies,0);
			 if (body!=NULL)
			{
			   /* check if we have an sdp in this invite*/
			   remote_sdp=smalloc(sizeof(sdp_t));
			   sdp_init(remote_sdp);
			   err=sdp_parse(remote_sdp,body->body);
      		} else err=-1;
			if (err==-1)
			{
				osip_trace(TRACE_LEVEL2,("Syntax error in sdp or sdp not found.\n"));
				/* Aymeric: 415 Unsupported Media Type
				   Here media is SDP see draft-...rfc2543-05.txt */
				respond_to_request(ua->config, trn, 415, NULL) ;
				return ;
			}
			/* check if user agent has session properies */
			if ((ua->maudio->profile_list->nb_elt==0) && (ua->mvideo->profile_list->nb_elt==0))
			{
				osip_trace(TRACE_LEVEL2,("User agent has NO session properties! Must reject an invite...\n"));
				/* respond that there is an internal error*/
				respond_to_request(ua->config, trn, 500, NULL) ;
				return;
			}
			call->remote_sdp=remote_sdp;
			err=osip_call_leg_extract_media(call,remote_sdp);
			if (err==-1)
			{
				/*  send a 606 not acceptable response */
				osip_trace(TRACE_LEVEL2,("Must reject invite because of incompatible media type.\n"));
				osip_call_leg_respond(call,606,NULL);
				/* to do*/
				return ;
			}
			/* inform the user about this invite*/
			if (ua->invite!=NULL)
				err=ua->invite(call,sipevent->sip,(void*)sipmsg);
			/* if err==0 send 180 ringing since we are successfully informing the user...*/
			if (err==0) osip_call_leg_respond(call,180,NULL);
		}		
	}			
  return ; /* ok */
}

void cb_rcvack(sipevent_t *sipevent,transaction_t *trn)
{
	OsipCallLeg *call;
	OsipUA *ua;
	
	osip_trace(TRACE_LEVEL1,("OnEvent_New_IncomingAck!\n"));

  call=ua_transaction_get_call_leg(trn);
	if (call!=NULL)
	{
		/* an existing call-leg was found, so process it */
		ua=osip_call_leg_get_ua(call);
		if (osip_call_leg_get_state(call)==CALL_INVITED)
		{
			osip_call_leg_set_state(call,CALL_RUNNING);
			/* indicate the user of the beginning of the audio session ???*/
			
		}
	}
	else
	{
		osip_trace(TRACE_LEVEL1,("Ack for an inexistant call-leg !\n"));
		/* do not respond*/
	}			

  return ; /* ok */
}

void cb_rcvbye(sipevent_t *sipevent,transaction_t *trn)
{
	sip_t *sipmsg=sipevent->sip;
	OsipCallLeg *call;
	OsipUA *ua;
	
	osip_trace(TRACE_LEVEL1,("OnEvent_New_IncomingBye!\n"));

  call=osip_call_leg_find(sipmsg);
	if (call!=NULL)
	{
		/* an existing call-leg was found, so process it */
		ua=osip_call_leg_get_ua(call);
		ua_transaction_incoming_set_call_leg(trn,call);
		call->last_tid=trn->transactionid;
		/* indicate the user of the end of the call-leg*/
		if (ua->bye!=NULL)
				ua->bye(call,sipevent->sip,(void*)sipmsg);
		/* respond 200 OK to the client */
		osip_call_leg_respond(call,200,NULL);
		/* then delete the call*/
		osip_call_leg_release(call);
	}
	else
	{
		osip_trace(TRACE_LEVEL1,("Bye for an inexistant call-leg !\n"));
		ua=(OsipUA*)list_get(&ua_list,0);
		respond_to_request(ua->config, trn, 481, NULL) ;
		/* return an Inexistant Call-Leg*/
	}			
  	return ; /* ok */
}

void cb_rcvcancel(sipevent_t *sipevent,transaction_t *trn)
{
  osip_trace(TRACE_LEVEL1,("<callback.c> OnEvent_New_IncomingCancel!\n"));
  //uaapp_annouceincrequest(sipevent->sip, transaction_id);
  return ; /* ok */
}



void cb_rcvunkrequest(sipevent_t *sipevent,transaction_t *trn)
{
  osip_trace(TRACE_LEVEL1,("OnEvent_New_IncomingUnknownRequest!\n"));
  //uaapp_annouceincrequest(sipevent->sip, transaction_id);
  return ; /* ok */
}




