/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <osip/osip.h>
#include <osip/smsg.h>

#include "osipmanager.h"
#include "osipua.h"
#include "utils.h"
#include "uatransaction.h"
#include "resolver.h"
#include "digcalc.h"

list_t callleg_list={0,NULL};

void osip_call_leg_init(OsipCallLeg *new_call, OsipUA *ua)
{
	contact_t *ctt;
	
	memset(new_call,0,sizeof(OsipCallLeg));
	/*set status for this new call*/
	new_call->status=CALL_NEW;
	/* init transaction list*/
	list_init(&new_call->incoming_transactions);
	list_init(&new_call->outgoing_transactions);
  /* set the ua reference on the newly created call-leg*/
	new_call->ua=ua;
	/* add the call-leg to the list of call-legs handled by the stack*/
	list_add(&callleg_list,(void*)new_call,-1);
	/* add the call-leg to the list of call-legs managed by the ua*/
	list_add(&ua->call_list,(void*)new_call,-1);
	ctt=(contact_t*) list_get(ua->alias,0);

	if (ctt!=NULL)
	{
		from_clone((from_t*)ctt,&new_call->from);
	}
	else
	{
		from_clone((from_t*) ua->contact,&new_call->from);
	}
	
	/*
	{
		char *tmp;
		from_init(&new_call->from);
		from_2char(ctt,&tmp);
		from_parse(new_call->from,tmp);
		sfree(tmp);
	}
	*/
	/* set from tag */
	from_tag_add(new_call->from,sgetcopy(ua->fromtag));
	/* set default registrar if given*/
	if (ua->registrar!=NULL)
	{
		url_clone(ua->registrar,&new_call->registrar);
	}
	/* BKO: initialize empty route list */
	list_init(&new_call->route_list);
}

/* this is the init function that initialize the call-leg if it is created
locally: callid, ceq... are choosed randomly */
void osip_call_leg_init2(OsipCallLeg *new_call, OsipUA *ua)
{
	char *number;
	
	new_call->sdp_sessid=rand();
	
	/* set up callid*/
	call_id_init(&new_call->callid);
	call_id_sethost(new_call->callid,sgetcopy(ua->ua_ip4addr));
	number=smalloc(17);
	snprintf(number,15,"%i",rand());
	call_id_setnumber(new_call->callid,number);
	/* set to ?? REVISIT */
	to_init(&new_call->to);

	/* BKO: initialize empty route list */
	list_init(&new_call->route_list);
}

/* internal constructor ! do not use !*/
OsipCallLeg *osip_call_leg_alloc(OsipUA *ua)
{
	OsipCallLeg *new_call;
	
	new_call=smalloc(sizeof(OsipCallLeg));
	osip_call_leg_init(new_call,ua);
	return new_call;
}

/* this is called by the application to create a new call-leg */

/**
 * osip_call_leg_new:
 * @ua: an existing user agent.
 *
 * Creates a new #OsipCallLeg object managed by the user agent @ua.
 *
 * Returns: a new call leg.
 */
OsipCallLeg *osip_call_leg_new(OsipUA *ua)
{
	OsipCallLeg *new_call;
	new_call=osip_call_leg_alloc(ua);
	osip_call_leg_init2(new_call,ua);
	return(new_call);
}



/* create a new call-leg from an initial incoming request . The function
assumes that there is no existing call-leg that match the new request.
You should use osip_call_leg_find() to check this */

/**
 * osip_call_leg_new_from_incoming_trn:
 * @trn: an incoming transaction
 *
 * Create a new call-leg from an initial incoming request. The function
 * assumes that there is no existing call-leg that match the new request.
 * You should use #osip_call_leg_find() to check this.
 *
 * Returns: a new call leg.
 */
OsipCallLeg *osip_call_leg_new_from_incoming_trn(transaction_t *trn)
{
	OsipCallLeg *call;
	OsipUA *ua;
	sip_t *sipmsg;
	contact_t *ctt;
	char *from;
	char *to;
	int i;
	
	call=ua_transaction_get_call_leg(trn);
	if (call!=NULL)
	{
		osip_trace(TRACE_LEVEL3,("info: A call-leg already exists for this transaction. (%x)",trn));
		return NULL;
	}
	
	/* first find a ua for this call-leg */
	ua=osip_ua_find(trn->lastrequest);

	i = from_2char(trn->lastrequest->from,&from);
	if (i!=0) return NULL;
	osip_trace(TRACE_LEVEL1,("info: %s has called at %i.\n",from,time(NULL)));
	sfree(from);
	

	if (ua==NULL)
	{
	  i = to_2char(trn->lastrequest->to,&to);
	  if (i!=0) return NULL;
	  /* we should return a 404 Not Found*/
	  osip_trace(TRACE_LEVEL0,("error: Requested user (%s) does not exist.\n",to));
	  sfree(to);
		/* take the first user agent of the list to respond to this request  :  Maybe revisit*/
		respond_to_request(def_manager->config, trn, 404, NULL) ;
		return NULL;
	}
	/* check if the user agent can answer to the call */
 	if (ua->call_list.nb_elt>=ua->max_calls)
 	{
 		/*respond a 486 busy here*/		
 		respond_to_request(ua->config, trn,486,NULL);
 		return NULL;
 	}
	call=osip_call_leg_alloc(ua);
	/* then setup the call_leg according the initial incoming request */
	sipmsg=trn->lastrequest;
	ua_transaction_incoming_set_call_leg(trn,call);

	if (MSG_IS_INVITE(sipmsg) )
		osip_call_leg_set_state(call,CALL_INVITED);
	else if (MSG_IS_REGISTER(sipmsg))
		osip_call_leg_set_state(call,CALL_INVITED);
	else
	{
		osip_trace(TRACE_LEVEL1,("error: Unsupported new incoming request."));
		osip_call_leg_release(call);
		return NULL;
	}	
	call_id_clone(sipmsg->call_id,&call->callid);
	call->last_tid=trn->transactionid;
	msg_getcontact(sipmsg,0,&ctt);
	if (ctt!=NULL)
	{
	    /* BKO: only set contact if we have not done so already */
	    if(call->peer != NULL)
	    {
		contact_clone(ctt,&call->peer);
	    }
	}else
	{
		osip_trace(TRACE_LEVEL2,("warning: No contact field in incoming request.\n"));
	}
	from_clone(sipmsg->from,&call->to); /* The from from the incoming request
	will become the to the future outgoing request (including tag)*/
	call->cseq_number=atoi(sipmsg->cseq->number);
	return call;
}

/**
 * osip_call_leg_update_from_response:
 * @call: a call leg.
 * @resp: a sip response.
 *
 * This function should be called each time a new response is received with
 * a call-leg. It updates the context of the call-leg according to the
 * response.
 *
 */
void osip_call_leg_update_from_response(OsipCallLeg *call,sip_t *resp)
{
	if (!call->tag_set)
	{
		/* if the call-leg is new, it has no tag parameter in to. If the server (maybe a proxy) give us one (it should)
		then we must remember it */
		from_copy_tag((from_t*)call->to,(from_t*)resp->to);
		call->tag_set=1;
	}
	/*
	 * GRS
	 * if there is a contact: header in here, set peer in
	 * the call control block
	 */

	/* BKO: if there is a record-route list here, save it so we
	 can use it */
	{
	    record_route_t *rr;
	    route_t *rr2;
	    int pos;
	    int i;
	    char *tmp;

	    if(!list_eol(resp->record_routes,0))
	    {
		for(pos=resp->record_routes->nb_elt-1;pos>=0;pos--)
		{
		    rr=(record_route_t*)list_get(resp->record_routes,pos);
		    i=record_route_2char(rr,&tmp); 
/*		    printf("********** route: %s\n", tmp); */
		    route_init(&rr2);
		    route_parse(rr2,tmp);
		    list_add(&call->route_list,rr2,0);
		    sfree(tmp);
		}
	    }
	}	   

	/* BKO: if there is a contact, append to the route list */
	{
	    contact_t *ctt;
	    char *tmp;
	    int i;
	    route_t *rr2;
	    msg_getcontact(resp,0,&ctt);
	    if (ctt!=NULL)
	    {
		osip_trace(TRACE_LEVEL3,("info: Contact in 2xx - adding to route list\n"));
		i=contact_2char(ctt,&tmp);
		route_init(&rr2);
		route_parse(rr2, tmp);
		list_add(&call->route_list,rr2,-1);
	    }
	}
	

	if (call->peer==NULL)
	{
	    /* BKO: set to the first item from the route list */
	    if(!list_eol(&call->route_list,0))
	    {
		contact_t *p;
		char *tmp;

		route_t* route = (route_t*) list_get(&call->route_list,0);
		/* must also pop the route */
		list_remove(&call->route_list, 0);

		route_2char(route, &tmp);

		contact_init(&p);
		contact_parse(p, tmp);

		call->peer = p;
	    }
	}
}

/* accept an invitation for an existing call-leg*/
/**
 * osip_call_leg_accept_invite:
 * @call_leg: a call-leg that is in the CALL_INVITED state.
 *
 * On reception of the 200 Ok for invite, then the osipua library calls
 * the "INVITE" signal handler. Then the user can choose to accept the call
 * using this function.
 * If the user wants to reject the call, then it should call
 * #osip_call_leg_bye
 *
 * Returns: the list of the media descriptor (MediaDesc) choosen by
 * the sdp negociation.
 */
list_t * osip_call_leg_accept_invite(OsipCallLeg *call_leg)
{
	OsipUA *ua;
	char *sdpmesg;
	/* check if we have good status for this call to answer 200OK*/
	if (osip_call_leg_exists(call_leg)==0)
	{
		osip_trace(TRACE_LEVEL0,("error: SIP call leg does not exist.\n"));
		return NULL;
	}
	ua=(OsipUA*)call_leg->ua;
	if (call_leg->status!= CALL_INVITED)
	{
		osip_trace(TRACE_LEVEL0,("error: Attempt to accept an inexistant invite.\n"));
		return(NULL);
	}
   /* make an sdp message with ua capabilities*/
	sdpmesg=sdp_compose(call_leg->sdp_sessid,ua->contact->url->username,ua->ua_ip4addr,call_leg->remote_sdp->s,call_leg->media_list);
	if (sdpmesg==NULL) return(NULL); 	/* respond a 200 OK with the above sdp message*/
	osip_call_leg_respond(call_leg,200, sdpmesg);

	/* returns the list of medias with their profile*/
	return(call_leg->media_list);
}

/* register your identity to a registrar */

/**
 * osip_call_leg_register:
 * @call_leg: a call-leg.
 * @context: a registration context.
 *
 * Sends a REGISTER request to a registrar.
 *
 * Returns: 0 if the REGISTER was sent, a negative value if not.
 */
int osip_call_leg_register(OsipCallLeg *call_leg, RegistrationCtxt *context)
{
	sip_t *sipmesg;
	char *tmp;
	url_t *rgt;
	from_t *addr_of_rec;
	int err;
	
	if (context->registrar==NULL)
	{
		osip_trace(TRACE_LEVEL0,("error: You have to specify a registrar.\n"));
		return(-1);
	}
	
 	url_init(&rgt);
 	err=url_parse(rgt,context->registrar);
 	if (err!=0)
 	{
 		osip_trace(TRACE_LEVEL0,("error: Bad registrar address (%s).\n",context->registrar));
 		sfree(rgt);
 		return -EINVAL;
 	}
	
	if (context->address_of_record!=NULL){
		from_init(&addr_of_rec);
		err=from_parse(addr_of_rec,context->address_of_record);	
		if (err!=0){
			osip_trace(TRACE_LEVEL0,("error: Bad address of record (%s).\n",context->address_of_record));
			sfree(addr_of_rec);
			return -EINVAL;
		}
		/* set to=from= address_of_record */
		from_free(call_leg->from);
		sfree(call_leg->from);
		call_leg->from=addr_of_rec;
		from_parse(call_leg->to,context->address_of_record);
	}
	else
	{
    	/* if no address of record is given in the context, then use the default from field of the call-leg */
 		to_2char((from_t*)call_leg->from,&tmp);
		to_parse(call_leg->to,tmp);
		sfree(tmp);
	}
  	/* remove the tag in the to feild! */
  	if (!list_eol(call_leg->to->gen_params,0))
	{
		generic_param_t *gp = list_get(call_leg->to->gen_params,0);
    	list_remove(call_leg->to->gen_params,0);
    	generic_param_free(gp);
    	sfree(gp);
  	}
  	call_leg->reg_context=context;
  	context->ref_count++;  /* context is in use ! */
	call_leg->cseq_number++;
	sipmesg=osip_call_leg_compose_request(call_leg,REGISTER,NULL);
	if (sipmesg==NULL) return(-1);
	osip_call_leg_send_request(call_leg,sipmesg);
	osip_call_leg_set_state(call_leg,CALL_REGISTERING);
	return 0;
}

int osip_call_leg_register_with_authentication(OsipCallLeg *call_leg,
						sip_t *previous_answer)
 {
	 sip_t *sipmesg;
	 authorization_t *aut = NULL;
	 proxy_authorization_t *proxy_aut = NULL;
	 int i;
	 char *uri;

	 /* set action */
	 call_leg->cseq_number++;
	 sipmesg=osip_call_leg_compose_request(call_leg,REGISTER,NULL);
	 if (sipmesg==NULL) return(-1);

	 /* ADD AUTHENTICATION STUFF.... */
	 url_2char(sipmesg->strtline->rquri,&uri);
	 /*	if (MSG_TEST_CODE(previous_answer,407)
		 ||MSG_TEST_CODE(previous_answer,401))
		 { */
	 if ((previous_answer->proxy_authenticate!=NULL)
	     &&(previous_answer->www_authenticate!=NULL))
	   return -1;
	 if (previous_answer->proxy_authenticate!=NULL)
	   {
	     i = osip_create_proxy_authorization_header(call_leg,
							previous_answer,
							uri,&proxy_aut);
	     if (i!=0) {
	       osip_trace(TRACE_LEVEL0,("error: could not create authorization header.\n"));
	       return -1;
	     }
	   }
	 
	 if (previous_answer->www_authenticate!=NULL)
	   {
	     i = osip_create_authorization_header(call_leg,
						  previous_answer,
						  uri,&aut);
	     if (i!=0) {
	       osip_trace(TRACE_LEVEL0,("error: could not build the proxy_authorization header.\n"));
	       return -1;
	     }
	   }
	 /* TODO */
	 /* here we should copy the proxy_authorization header
	    from the previous_answer to the new request. This
	    proxy_authorization is the response given by the
	    first proxy made for the second proxy!
	    oSIP must be fixed here to support multiple
	    proxy_authorization, authorization, www_authenticate,
	    and proxy_authenticate in one message.
	    jack@atosc.org
	    */
	 /* !TODO */
	 
	 
	 sipmesg->authorization = aut;
	 if (proxy_aut!=NULL)
	   list_add(sipmesg->proxy_authorizations, proxy_aut,-1);
	 /* ADD AUTHENTICATION STUFF.... */
	 
	 osip_call_leg_send_request(call_leg,sipmesg);
	 osip_call_leg_set_state(call_leg,CALL_REGISTERING);
	 return 0;
 }

/**
 * osip_call_leg_unregister:
 * @call: a call-leg.
 * @ctxt: a registration context that has been used to make a previous registration.
 *
 * Sends a REGISTER request to a registrar with a expire=0, so that
 * the registration is cancelled.
 * Warning: the ctxt structure may be changed. Do not rely on it to make a new registration.
 *
 * Returns: 0 if the REGISTER was sent, a negative value if not.
 */
int osip_call_leg_unregister(OsipCallLeg *call, RegistrationCtxt *ctxt)
{
	int error;
	ctxt->expires=0;
	error=osip_call_leg_register(call,ctxt);
	return error;
}

/* invite somebody */
/**
 * osip_call_leg_invite:
 * @call_leg: a call-leg.
 * @callee: the URI of the person to invite.
 *
 * Sends an invite to @callee.
 *
 * Returns: 0 if the invite was sent, a negative value if not.
 */
int osip_call_leg_invite(OsipCallLeg *call_leg, char *callee)
{
	list_t *mds;
	OsipUA *ua=osip_call_leg_get_ua(call_leg);
	char *sdpmesg;
	sip_t *sipmesg;
	char *callee_addr;
	int err;

	if (call_leg->status!=CALL_NEW) return(-ESRCH);
	/* make a list MediaDesc: for the moment only audio*/
	mds=smalloc(sizeof(list_t));
	list_init(mds);
	list_add(mds,ua->maudio,-1);
	/* set to */
	callee_addr=sgetcopy(callee);      /* useful because we write on calle_addr*/
 	err=from_parse((from_t*)(call_leg->to),callee_addr);
  	sfree(callee_addr); 	
  	if (err==SIP_SYNTAX_ERROR)
	{
		osip_trace(TRACE_LEVEL0,("error: Invalid sip address.\n"));
		return -EINVAL;
	}

	/* make the sdp message*/
	sdpmesg=sdp_compose(call_leg->sdp_sessid,ua->contact->url->username,ua->ua_ip4addr,"Our conversation",mds); 	/* should free the list*/

	if (sdpmesg==NULL) return(-1);
	/* compose an invite with the above sdp*/
	call_leg->cseq_number++;
	sipmesg=osip_call_leg_compose_request(call_leg,INVITE,sdpmesg);
	if (sipmesg==NULL)
	{
		osip_trace(TRACE_LEVEL0,("error: could not build sip message.\n"));
		return(-1);
	}
	/* and send it*/
	osip_call_leg_send_request(call_leg,sipmesg);
	osip_call_leg_set_state(call_leg,CALL_INVITING);
	return(0);
}

/* invite somebody */
int osip_call_leg_invite_with_authentication(OsipCallLeg *call_leg,
					     sip_t *previous_answer)
{
	list_t *mds;
	OsipUA *ua=osip_call_leg_get_ua(call_leg);
	char *sdpmesg;
	sip_t *sipmesg;
	char *uri;
	authorization_t *aut = NULL;
	proxy_authorization_t *proxy_aut = NULL;
	

	if (call_leg->status!=CALL_NEW) return(-ESRCH);
	/* make a list MediaDesc: for the moment only audio*/
	mds=smalloc(sizeof(list_t));
	list_init(mds);
	list_add(mds,ua->maudio,-1);

	/* make the sdp message*/
	sdpmesg=sdp_compose(call_leg->sdp_sessid,ua->contact->url->username,ua->ua_ip4addr,"Our conversation",mds); 	/* should free the list*/

	if (sdpmesg==NULL) return(-1);
	/* compose an invite with the above sdp*/
	call_leg->cseq_number++;
	sipmesg=osip_call_leg_compose_request(call_leg,INVITE,sdpmesg);
	if (sipmesg==NULL)
	{ osip_trace(TRACE_LEVEL0,("error: could not build sip message.\n")); return(-1); }


	 /* ADD AUTHENTICATION STUFF.... */
	 url_2char(sipmesg->strtline->rquri,&uri);
	 /*	if (MSG_TEST_CODE(previous_answer,407)
		 ||MSG_TEST_CODE(previous_answer,401))
		 { */
	 if ((previous_answer->proxy_authenticate!=NULL)
	     &&(previous_answer->www_authenticate!=NULL))
	   return -1;
	 if (previous_answer->proxy_authenticate!=NULL)
	   {
	     int i = osip_create_proxy_authorization_header(call_leg,
							previous_answer,
							uri,&proxy_aut);
	     if (i!=0) {
	       osip_trace(TRACE_LEVEL0,("error: could not build the authorization header.\n"));
	       return -1;
	     }
	   }
	 
	 if (previous_answer->www_authenticate!=NULL)
	   {
	     int i = osip_create_authorization_header(call_leg,
						  previous_answer,
						  uri,&aut);
	     if (i!=0) {
	       osip_trace(TRACE_LEVEL0,("error: could not build the proxy_authorization header.\n"));
	       return -1;
	     }
	   }
	 /* TODO */
	 /* here we should copy the proxy_authorization header
	    from the previous_answer to the new request. This
	    proxy_authorization is the response given by the
	    first proxy made for the second proxy!
	    oSIP must be fixed here to support multiple
	    proxy_authorization, authorization, www_authenticate,
	    and proxy_authenticate in one message.
	    jack@atosc.org
	    */
	 /* !TODO */
	 
	 
	 sipmesg->authorization = aut;
	 if (proxy_aut!=NULL)
	   list_add(sipmesg->proxy_authorizations, proxy_aut,-1);
	 /* !ADD AUTHENTICATION STUFF.... */

	/* and send it*/
	osip_call_leg_send_request(call_leg,sipmesg);
	osip_call_leg_set_state(call_leg,CALL_INVITING);
	return(0);

}


/* bye and destroy the call-leg*/

/**
 * osip_call_leg_bye:
 * @call_leg: a call-leg.
 *
 *
 * Sends a bye and ends the call-leg.
 *
 * Returns: 0 if the bye was sent, a negative value if not.
 */
int osip_call_leg_bye(OsipCallLeg *call_leg)
{
	sip_t *sipmesg;

	if (call_leg->status==CALL_NEW)
	{
		osip_trace(TRACE_LEVEL0,("error: no call established. (use CANCEL instead)\n"));
		return(-ESRCH);
	}
	call_leg->cseq_number++;
	sipmesg=osip_call_leg_compose_request(call_leg,BYE,NULL);
	if (sipmesg==NULL) return(-1); /* to be set...*/
	osip_call_leg_send_request(call_leg,sipmesg);
	osip_call_leg_release(call_leg);
	return(0);
}




/* returns 1 if call leg exists*/
int osip_call_leg_exists(OsipCallLeg *call)
{
	int pos;
	OsipCallLeg *callleg;

	for (pos=0;pos<callleg_list.nb_elt;pos++)
	{
		callleg=(OsipCallLeg*)list_get(&callleg_list,pos);
		if (callleg==call) return 1;
  	}
	return 0;
}


/**
 * osip_call_leg_find:
 * @sipmsg: an incoming request.
 *
 *
 * Tries to find a call-leg that may be concerned by the incoming request.
 *
 * Returns: The call leg, NULL if none was found.
 */
OsipCallLeg *osip_call_leg_find(sip_t *sipmsg)
{
	int pos=0;
	int cond=0;
	OsipCallLeg *call=NULL;

  	/*find an existant call-leg that may match  this new invite */
  	while( !list_eol(&callleg_list,pos)&&(!cond))
  	{
  		call=(OsipCallLeg*)list_get(&callleg_list,pos);
  		/* there are much more things to check: callid, from tags, to etc...*/
		if ( call_id_match(sipmsg->call_id,call->callid)==0)
		{
			
			if (MSG_IS_REQUEST(sipmsg))
			{
				if (MSG_IS_ACK(sipmsg))
				{
					if (atoi(sipmsg->cseq->number)>=call->cseq_number)
						cond=1;
					else osip_trace(TRACE_LEVEL4,("info: Ack cseq does not match !!!!!!!!!!!!!\n"));
				}
				else
				{
					if (atoi(sipmsg->cseq->number)>call->cseq_number)
						cond=1;
					else osip_trace(TRACE_LEVEL4,("info: Request Cseq does not match !!!!!!!!!!!!!\n"));
				}
			}
			else
			{
				if (atoi(sipmsg->cseq->number)>=call->cseq_number)
					cond=1;
				else osip_trace(TRACE_LEVEL4,("info: Response Cseq does not match !!!!!!!!!!!!!\n"));
			}
		}
		pos++;
	}
	if (cond)
	  {
	    osip_trace(TRACE_LEVEL4,("info: call-leg has been found!\n"));
	    /* an existing call-leg was found */
	    return(call);
	}
	else return(NULL);	
}


/* find an existing call leg using a transaction id*/
OsipCallLeg *osip_call_leg_find_with_tid(int tid)
{
	int i;
	OsipCallLeg *call;
	for (i=0;i<callleg_list.nb_elt;i++)
	{
		call=(OsipCallLeg*)list_get(&callleg_list,i);
		if (tid==call->last_tid)
		return(call);
	}
	return(NULL);
}

/**
 * osip_call_leg_send_response:
 * @call: a call-leg
 * @trn: a transaction
 * @resp: a sip response
 *
 *
 * Sends the response @resp that will be part of the transaction @trn,
 * trn being part of the call-leg @call.
 *
 */
void osip_call_leg_send_response(OsipCallLeg *call,transaction_t *trn, sip_t *resp)
{
	osip_send_response(call->ua->manager,trn,resp);	
}


/* respond to a request that is part from a call-leg : ex 200OK (INVITE)*/
/**
 * osip_call_leg_respond:
 * @call: a call-leg.
 * @code: the code of the response.
 * @body: a sdp body to be attached to the response.
 *
 *
 * This function creates a sip response in the context of the call-leg @call
 * for the last transaction and sends it automatically.
 * A sdp body can be optionnaly appended to the response.
 *
 */
void osip_call_leg_respond(OsipCallLeg *call, int code, char *body)
{
	char *reason;
	char *apptype=NULL;
	sip_t *response;
	transaction_t *transaction;	
	OsipUA *ua=osip_call_leg_get_ua(call);
	int tid=call->last_tid;
	osip_t *config=ua->config;
	transaction = osip_find_byid(config, tid);

	msg_init(&response);
	reason = msg_getreason(code);

	if (body!=NULL) apptype=sgetcopy("application/sdp");

  if (1==msg_makereply(code,reason,
		 transaction->lastrequest,
		 response,
		 body, /*local_body*/
		 apptype, /*apptype */
		 0))
	{
		/* must add a tag in the to field, I use the one the user agent */
		{
		    /* BKO: this should be To:, not From: */
/*			from_t *from;
			from=msg_getfrom(response);
			from_tag_add(from,sgetcopy(ua->fromtag));
*/
			to_t *to;
			to=msg_getto(response);
			to_tag_add(to,sgetcopy(ua->fromtag));
		}	
		/* BKO: we must save and mark the peer as well as the
		 * route here, from the record-route.  */
		{
		    sip_t *req = transaction->lastrequest;

		    record_route_t *rr;
		    route_t *rr2;
		    int pos;
		    int i;
		    char *tmp;
		    
		    /* this should only be done the first time! */
		    if(!list_eol(req->record_routes,0) && 
		       list_eol(&call->route_list,0))
		    {
			for(pos=req->record_routes->nb_elt-1;pos>=0;pos--)
			{
			    rr=(record_route_t*)list_get(req->record_routes,pos);
			    i=record_route_2char(rr,&tmp); 
/*			    printf("**** route %d: %s\n", pos, tmp); */
			    route_init(&rr2);
			    route_parse(rr2,tmp);
			    /* the route list is the reverse of the
                               code for a response we receive
                               vs. response we send */
			    list_add(&call->route_list,rr2,-1);
			    sfree(tmp);
			}

			/* BKO: if there is a contact, append to the
                           route list */
			{
			    contact_t *ctt;
			    char *tmp;
			    int i;
			    route_t *rr2;
			    msg_getcontact(req,0,&ctt);
			    if (ctt!=NULL)
			    {
				osip_trace(TRACE_LEVEL3,("info: Contact in 2xx - adding to route list\n"));
				i=contact_2char(ctt,&tmp);
				route_init(&rr2);
				route_parse(rr2, tmp);
				list_add(&call->route_list,rr2,-1);
			    }
			}
		    }
		}

		/* we must set the peer from the route list */
		if (call->peer==NULL)
		{
		    /* set to the first item from the route list */
		    if(!list_eol(&call->route_list,0))
		    {
			contact_t *p;
			char *tmp;
			
			route_t* route = (route_t*) list_get(&call->route_list,0);
			/* must also pop the route */
			list_remove(&call->route_list, 0);

			route_2char(route, &tmp);
			
			contact_init(&p);
			contact_parse(p, tmp);
			
			call->peer = p;
		    }
		}
		


     /* the user agent MUST set a contact header in last RFC*/
 		{	
       char *tmp = (char *)smalloc(60*sizeof(char)+strlen(ua->contact->url->username));
  		sprintf(tmp,"<sip:%s@%s:%i>",ua->contact->url->username,ua->ua_ip4addr,ua->ua_port);
  		msg_setcontact(response,tmp);
  	}
    	
		osip_call_leg_send_response(call,transaction,response);
	}
	else
	{
		osip_trace(TRACE_LEVEL0,("error: Could not build SIP response.\n"));
		sfree(response);
		sfree(reason);
		if (apptype!=NULL)
	  	sfree(apptype);
		return ;
      }
	sfree(reason);
	if (apptype!=NULL)
      sfree(apptype);
}

/**
 * osip_call_leg_ack:
 * @call: a call-leg.
 * @trn: a transaction part of the call-leg.
 *
 *
 * This function generates and sends an ACK request that will end the
 * transaction @trn.
 *
 */
void osip_call_leg_ack(OsipCallLeg *call,transaction_t *trn)
{
	sip_t *ack;
	sipevent_t *sipevent;
	char *dest;
	OsipUA *ua=osip_call_leg_get_ua(call);

    printf("******************** sending ack ********************\n");
	
	ack=osip_call_leg_compose_request(call,ACK,NULL);

	/* in acks, the request uri must be the same as in the request
           of transaction, unless call->peer exists*/

 	if (ack->strtline->rquri->host==NULL)
 	{
 		char *tmp=NULL;
		url_2char(trn->lastrequest->strtline->rquri,&tmp);
 		url_parse(ack->strtline->rquri,tmp);
 		sfree(tmp);
 	}
	
	/*sending an ack is a special case: there is no need to create
          a new transaction*/

	sipevent = osip_new_outgoing_sipmessage(ack);
	call->last_tid=sipevent->transactionid = trn->transactionid;
	/* see if we need a name resolution*/		
	if (trn->proxy!=NULL)
		dest=trn->proxy->host;
	else dest=ack->strtline->rquri->host;
	if (inet_addr(dest)==-1)
	{
		async_resolv_and_send(ua->manager,trn,sipevent);
		return;
	}
	//transaction_execute(trn,sipevent);
	fifo_add(trn->transactionff,sipevent);
}

/**
 * osip_call_leg_compose_request:
 * @call_leg: a call-leg.
 * @msgtype: the request type in INVITE, BYE, REGISTER.
 * @sdpbody: an optionnal sdp body to add to the request.
 *
 * Creates a sip request of type @msgtype according to the context of
 * the call-leg @call_leg. The request is not sent.
 *
 * Returns: a sip message.
 */
sip_t * osip_call_leg_compose_request(OsipCallLeg *call_leg,int msgtype,char *sdpbody)
{
	sip_t         *sipmsg;
	char *tmp;
	OsipUA *ua= (OsipUA*)call_leg->ua;
	RegistrationCtxt *context=call_leg->reg_context;
	
	msg_init (&sipmsg);

	/* set to header*/
  {/* mandatory */
    to_2char(call_leg->to,&tmp);
    msg_setto(sipmsg,tmp);
    sfree(tmp);
	}
	/*set from header*/
	{
		from_2char(call_leg->from,&tmp);
  		msg_setfrom(sipmsg,tmp);
    	sfree(tmp);
	}
	/* set via header: use ip4addr instead of name*/
	{
    char *via = (char *)smalloc(60*sizeof(char));
    sprintf(via,"SIP/2.0/UDP %s:%i",ua->ua_ip4addr,ua->ua_port);
    msg_setvia(sipmsg,via);
    sfree(via);
  }
	/*set contact header*/
	contact_2char(ua->contact,&tmp);
	msg_setcontact(sipmsg,tmp);
	sfree(tmp);
	if (msgtype==REGISTER)
	{
		/* add a tag "action" */
		contact_t *ctt;
		if (context==NULL) {
			osip_trace(TRACE_LEVEL0,("registration context is null!\n"))
			return NULL;
		}
		msg_getcontact(sipmsg,0,&ctt);
		if (context->action==REGISTER_ACTION_REDIRECT)
			from_param_add(((from_t*)ctt),sgetcopy("action"),sgetcopy("redirect"));
		else if (context->action==REGISTER_ACTION_PROXY)
			from_param_add(((from_t*)ctt),sgetcopy("action"),sgetcopy("proxy"));
		tmp=smalloc(10);
		sprintf(tmp,"%i",context->expires);
		msg_setexpires(sipmsg,tmp);
		sfree(tmp);
		/* set the from url same as the address of record, because some server don't like other addresses*/
		//sfree(sipmsg->from->url->host);
		//sipmsg->from->url->host=sgetcopy(sipmsg->to->url->host);
		/* remove the tag of the from field, because some server don't like it */
		
		//list_remove(sipmsg->from->gen_params,0);
		
	}

	/* BKO: if there is a routelist, you need to set that, as well */
	if(!list_eol(&call_leg->route_list,0))
	{
	    int pos;
	    route_t *route;
	    char *tmp;
	    int i;
	    int max = call_leg->route_list.nb_elt-1; 
	    for(pos=0; pos <= max; pos++)
	    {
		route=(route_t*)list_get(&call_leg->route_list,pos);
		i=route_2char(route,&tmp);
/*		printf("******************** route: %s\n", tmp); */
/*		route_init(&route2); */
/*		route_parse(&route2,tmp); */
		i = msg_setroute(sipmsg,tmp);
		if(i == -1)
		{
		    osip_trace(TRACE_LEVEL0,("bad setroute!\n"))
		}
/*		list_add(sipmsg->routes,route2,0); */
		sfree(tmp);
	    }
	}
	
	/* set first line*/
	sipmsg->strtline = (startline_t *) smalloc(sizeof(startline_t));
	sipmsg->strtline->sipmethod = (char *)smalloc(14);
	switch(msgtype)
	{
		case INVITE:
			strcpy(sipmsg->strtline->sipmethod,"INVITE");
			break;
		case BYE:
			strcpy(sipmsg->strtline->sipmethod,"BYE");
			break;
		case REGISTER:
			strcpy(sipmsg->strtline->sipmethod,"REGISTER");
			break;
		case ACK:
			strcpy(sipmsg->strtline->sipmethod,"ACK");
			break;
	}

  sipmsg->strtline->sipversion = (char *) smalloc(8*sizeof(char));
  sprintf(sipmsg->strtline->sipversion,"SIP/2.0");
  sipmsg->strtline->statuscode   = NULL;
  sipmsg->strtline->reasonphrase = NULL;
  url_init(&sipmsg->strtline->rquri);

	/* set request URI, depends on message type*/
	switch(msgtype)
	{
	case  REGISTER:
		url_parse(sipmsg->strtline->rquri,context->registrar);
		/* set also the expire field */
		/* set expires*/
		
		break;
	case ACK:

	    /* in acks, the request uri must be the same as in the
	       request of transaction unless there is a
	       OsipCallLeg::peer set by previous response */

	    /* but we don't have the transaction reference here so we
		leave blank the request uri to be set in
		osip_call_leg_ack() */
		
		if (call_leg->peer != NULL)
		{
			url_2char(call_leg->peer->url,&tmp);
			url_parse(sipmsg->strtline->rquri,tmp);
			sfree(tmp);
		}
  break;
	default:
		/*
		 * GRS
     * we need to do the same as above for subsequent messages
     * in the dialog (e.g. BYE)
    */
    if (call_leg->peer == NULL)
      url_2char(call_leg->to->url,&tmp);
    else
      url_2char(call_leg->peer->url,&tmp);
		url_parse(sipmsg->strtline->rquri,tmp);
		sfree(tmp);
	}
	
  /* remove unwanted information... They do not appear on request URI */
  /* to do */

	/* set user agent information */
	msg_setuser_agent(sipmsg,"oSIP/Linphone-"VERSION);
	
	/* set call-id*/
  {
    char *callid;

		call_id_2char(call_leg->callid,&callid);
		msg_setcall_id(sipmsg,callid);
		sfree(callid);
	}
	/* set cseq*/
  {
    char *cseq = (char *)smalloc(31*sizeof(char));
    snprintf(cseq,30,"%i %s",call_leg->cseq_number,sipmsg->strtline->sipmethod);
    msg_setcseq(sipmsg,cseq);
    sfree(cseq);
  }
	/* maybe need to set a subject ...*/
	/* to do*/
	
	/* set application type and sdp body, if given*/
	if (sdpbody!=NULL)
  {
   	char   *ctypev;
   	char size[10];

   	ctypev = sgetcopy("application/sdp\0");
   	msg_setcontent_type(sipmsg,ctypev);

   	sfree(ctypev);
    	
   	msg_setbody(sipmsg,sdpbody);
   	sprintf(size,"%i",strlen(sdpbody));
   	sfree(sdpbody);  /* a voir*/
   	msg_setcontent_length(sipmsg,size);
	}
	else msg_setcontent_length(sipmsg,"0");
	return(sipmsg);
}

/**
 * osip_call_leg_send_request:
 * @call_leg: a call-leg.
 * @sipmsg: a sip request.
 *
 *
 * Sends a sip request. The transaction is created to wrap the request and
 * to the list of transaction of call-leg @call_leg.
 *
 * Returns: 0 if successfull.
 */
int osip_call_leg_send_request(OsipCallLeg *call_leg,sip_t *sipmsg)
{
	sipevent_t    *sipevent;
	transaction_t *transaction;
	char *dest;
	OsipUA * ua=(OsipUA*)call_leg->ua;

	if (!MSG_IS_ACK(sipmsg))
	{
		sipevent = osip_new_outgoing_sipmessage(sipmsg);
		transaction = ua_transaction_new(call_leg,sipmsg);
	}
	else
	{
	  osip_trace(TRACE_LEVEL0,("error: you must not use osip_call_leg_send_request() to send acks."));
	  /* "Acks must only be sent using osip_call_leg_ack()" */
	  exit(1);
	}
	call_leg->last_tid=sipevent->transactionid = transaction->transactionid;
	/* see if we need a name resolution*/		
	if (transaction->proxy!=NULL)
		dest=transaction->proxy->host;
	else dest=sipmsg->strtline->rquri->host;
	if (inet_addr(dest)==-1)
		return (async_resolv_and_send(ua->manager,transaction,sipevent));
	
	//transaction_execute(transaction,sipevent);
	fifo_add(transaction->transactionff,sipevent);
	return transaction->transactionid;
}

/**
 * osip_call_leg_release:
 * @call: a call-leg.
 *
 *
 * Schedule the end of the call-leg. The call-leg object will be destroyed as
 * soon as it the osip stack will terminate its last transaction.
 *
 */


/* Destroy a call-leg and remove it from ua list and stack list*/

/**
 * osip_call_leg_destroy:
 * @call: a call-leg.
 *
 *
 * Destroy a call-leg and all data associated with it.
 * BUT: this function should not be called directly. You should use
 * #osip_call_leg_release instead to schedule the destruction of the call
 * leg.
 *
 * Returns 0.
 */

int osip_call_leg_destroy(OsipCallLeg *call)
{
	int err=0;
	struct  _OsipUA *ua=call->ua;
	
	err=list_remove_el(&callleg_list,call);
	if (err<0)  osip_trace(TRACE_LEVEL1,("error: Could not remove call-leg from stack list.\n"));
	err=list_remove_el(&ua->call_list,call);
	osip_trace(TRACE_LEVEL1,("Call leg is removed. It remains %i call legs in the ua list.\n",list_size(&ua->call_list)));
	if (err<0)  osip_trace(TRACE_LEVEL1,("error: Could not remove call-leg from ua list.\n"));
	if (err) return(-EFAULT);
	if( call->to!=NULL)
	{
		from_free(call->to);
		sfree(call->to);
	}
	if( call->from!=NULL)
	{
		from_free(call->from);
		sfree(call->from);
	}
	if (call->callid!=NULL)
	{
		call_id_free(call->callid);
		sfree(call->callid);
	}
	if (call->remote_sdp!=NULL)
	{
		sdp_free( call->remote_sdp);
		sfree(call->remote_sdp);
	}
	/* free the media_list */
	if (call->media_list!=NULL)
	{
		list_free(call->media_list,LIST_ELEM_DESTRUCTOR(media_desc_destroy));
		sfree(call->media_list);
	}	
	if(call->sdp_caddr!=NULL) sfree(call->sdp_caddr);
	if (call->registrar!=NULL)
	{
		url_free(call->registrar);
		sfree(call->registrar);
	}
	sfree(call);
	return(err);
}


/* extract all media descriptors from a sdp_t, and find compatible profiles for each one*/
/* if compatible profile is found for ALL medias, return 0, else -1. The list of MediaDesc is stored in OsipCallLeg::media_list*/
int osip_call_leg_extract_media(OsipCallLeg *call,sdp_t *remote_sdp)
{
	list_t *media_list;
	mediad_t *md;
	MediaDesc *mrem,*mloc=NULL;
	OsipUA *ua=(OsipUA*)call->ua;
	int prof=-1,error=0,i;
	char *dest_addr,*p;
	
	/* get c field and extract ip address*/
	dest_addr=smalloc(strlen(remote_sdp->c));
	sscanf(remote_sdp->c,"%*s %*s %s",dest_addr);
	p=strchr(dest_addr,'/');
	if (p!=NULL) *p='\0';
	call->sdp_caddr=dest_addr;
	
	media_list=smalloc(sizeof(list_t));
	list_init(media_list);
	for(i=0;i<remote_sdp->mediads->nb_elt;i++)
	{
		/* get all streams and their properties, and store them in a list of MediaDesc OsipCallLeg::media_list */
		printf("Get stream\n");
		md=(mediad_t*)list_get(remote_sdp->mediads,i);
		mrem=media_desc_new_from_mediad(md);
		if (mrem->type==STREAM_AUDIO)
		{
			prof=get_compatible_profile(ua->maudio->profile_list,mrem->profile_list);
			mloc=media_desc_new(STREAM_AUDIO);
			mloc->loc_port=ua->maudio->loc_port;
			mloc->rem_port=mrem->rem_port;
		}
		else if ( mrem->type==STREAM_VIDEO)
		{
			prof=get_compatible_profile(ua->mvideo->profile_list,mrem->profile_list);
			mloc=media_desc_new(STREAM_VIDEO);
			mloc->loc_port=ua->mvideo->loc_port;
			mloc->rem_port=mrem->rem_port;
		}
		media_desc_destroy(mrem);	
		mloc->rem_ip4addr=dest_addr;
		if (prof!=-1)
		{
			/* if we have found a compatible profile, then add the media descriptor to the list maintened by the call-leg*/
			media_desc_append_profile(mloc,prof);
			list_add(media_list,mloc,-1);
		}
		else
		{
			/* we get uncompatible profile : send an unsupported media type*/
			osip_trace(TRACE_LEVEL0,("info: Uncompatible media type in incoming message.\n"));
			media_desc_destroy(mloc);
			error=-1;
		}
		osip_trace(TRACE_LEVEL3,("info: End of get stream\n"));
	}	
	call->media_list=media_list;
	return(error);
}	

/* get the list media streams with their profile when a call is starting*/
/**
 * osip_call_leg_get_streams:
 * @call_leg: a call-leg.
 *
 * After a successfull invite (means that the application received the
 * "INVITE_ACCEPTED" signal, then this function can be call to get
 * a list of media descriptors of the media streams that have been choosen
 * during the sdp negociation.
 *
 * Returns: a list of MediaDesc that describes the media streams.
 */
list_t * osip_call_leg_get_streams(OsipCallLeg *call_leg)
{
	if (call_leg->status!=CALL_RUNNING)
	{
		osip_trace(TRACE_LEVEL0,("error: Call-leg has no local media.\n"));
		return(NULL);
	
	}
	return(call_leg->media_list);
}

/**
 * osip_call_leg_release:
 * @call: a call-leg.
 *
 * Release a call-leg. You can't use it anymore, and the object will be
 * destroyed by the stack when the last transaction of the call-leg
 * will terminate, either by successfull completion or timeout.
 *
 *
 */
