/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef OSIPUA_H
#error "You must not include osipcallleg.h in an application. Use osipua.h instead."
#endif

#ifndef OSIP_CALL_LEG_H
#define OSIP_CALL_LEG_H

#include <osip/smsgtypes.h>

#include "regctxt.h"


/* status for call-legs*/
enum { CALL_NEW, CALL_INVITING, CALL_REGISTERING, CALL_INVITED, CALL_RUNNING,
			 CALL_TERMINATED};

/* the list of call-legs handled by the stack*/
extern list_t callleg_list;


/* create a new call-leg managed by an existing use agent*/
OsipCallLeg *osip_call_leg_new(struct _OsipUA *ua);

OsipCallLeg *osip_call_leg_new_from_incoming_trn(transaction_t *trn);

void osip_call_leg_update_from_response(OsipCallLeg *call,sip_t *resp);

#define osip_call_leg_set_state(call, state) (call)->status=(state)

#define osip_call_leg_get_state(call) ((call)->status)

/* returns 1 if call leg exists*/
int osip_call_leg_exists(OsipCallLeg *call);

/* returns the ua that owns the call leg*/
#define osip_call_leg_get_ua(call) ((struct _OsipUA*)(call->ua))

/* find an existing call_leg in the stack list using an incmoing sip message. Returns NULL if not found*/
OsipCallLeg *osip_call_leg_find(sip_t *sipmsg);

/* find an existing call leg using a transaction id*/
OsipCallLeg *osip_call_leg_find_with_tid(int tid);


/* accept an invitation for an existing call-leg*/
list_t * osip_call_leg_accept_invite(OsipCallLeg *call_leg);

/* invite somebody */
int osip_call_leg_invite(OsipCallLeg *call_leg, char *callee);
int osip_call_leg_invite_with_authentication(OsipCallLeg *call_leg,
					     sip_t *previous_answer);

/* register your identity to a registrar */
int osip_call_leg_register(OsipCallLeg *call_leg, RegistrationCtxt *ctxt);
/* action can be :*/

int osip_call_leg_register_with_authentication(OsipCallLeg *call_leg,sip_t *previous_answer);

/* unregister user's identity to a registrar*/
int osip_call_leg_unregister(OsipCallLeg *call, RegistrationCtxt *ctxt);

/* get the list media streams with their profile when a call is starting*/
list_t * osip_call_leg_get_streams(OsipCallLeg *call_leg);

void osip_call_leg_ack(OsipCallLeg *call,transaction_t *trn);
/* bye and destroy the call-leg*/
int osip_call_leg_bye(OsipCallLeg *call_leg);

sip_t * osip_call_leg_compose_request(OsipCallLeg *call_leg,int msgtype,char *sdpbody);

int osip_call_leg_send_request(OsipCallLeg *call_leg,sip_t *sipmsg);

/* respond to a request that is part from a call-leg : ex 200OK (INVITE)*/
void osip_call_leg_respond(OsipCallLeg *call, int code, char *body);

void osip_call_leg_send_response(OsipCallLeg *call,transaction_t *trn, sip_t *resp);

#define osip_call_leg_release(call) osip_call_leg_set_state(call,CALL_TERMINATED);

/* Destroy a call-leg and remove it from ua list and stack list*/
int osip_call_leg_destroy(OsipCallLeg *call);

/* extract all media descriptors from a sdp_t, and find compatible profiles for each one*/
/* if compatible profile is found for ALL medias, return 0, else -1. The list of MediaDesc is stored in OsipCallLeg::media_list*/
int osip_call_leg_extract_media(OsipCallLeg *call,sdp_t *remote_sdp);

/* possible requests*/
#define INVITE 0
#define BYE 1
#define REGISTER 2
#define ACK 3

/* returns 1 if call is handled by ua*/
int osip_ua_call_leg_exists(OsipUA *ua,OsipCallLeg *call);


#endif
