/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef OSIPMANAGER_H
#define OSIPMANAGER_H

#include <osip/osip.h>
#include <osip/port.h>
#include <osip/thread.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#define OSIP_MAX_UDP_PORTS 5



typedef struct _OsipManager
{
	osip_t *config;
	int send_sock;											/* the socket used to send requests and responses */
	int send_port;												/* the port from which requests and responses are sent*/
	int udpports[OSIP_MAX_UDP_PORTS];  /* table of listening ports*/
	int udpfds[ OSIP_MAX_UDP_PORTS];  /* the corresponding table of file descripor */
	fd_set udpfdset;                           /* set of scanned file descripors */
	int max_udpfd;											/* highest file descriptor */
	int udp_control_fd;									  /* a control file descriptor to interrupt select() when needed*/
	int udp_unblock_fd;
	int udp_run_cond;										/* the running condition for the sipd thread*/
	int resolv_run_cond;									/* the running condition for the resolution thread */
	char *udp_buf;													/* the buffer used for receiving udp messages*/
	sthread_t udp_thread;                        /* the udp thread control structure*/
	sthread_t resolv_thread;							/*the name resolution thread */
	struct timeval recv_tout;                     /* the select timeout in udp_receive */
	fifo_t *resolv_fifo;
} OsipManager;

OsipManager* osip_manager_new();

int osip_manager_add_udpport(OsipManager *manager, int port);

int osip_manager_remove_udpport(OsipManager *manager, int port);

void osip_manager_start_udp_daemon(OsipManager *manager);

void osip_manager_stop_udp_daemon(OsipManager *manager);

void osip_manager_start_resolver(OsipManager *manager);

void osip_manager_stop_resolver(OsipManager *manager);

int osip_manager_set_send_port(OsipManager *manager,int port);

void osip_manager_destroy(OsipManager *manager);

void cb_rcvinvite  (sipevent_t *se,transaction_t *tr);
void cb_rcvack     (sipevent_t *se,transaction_t *tr);
void cb_rcvbye     (sipevent_t *se,transaction_t *tr);
void cb_rcvcancel  (sipevent_t *se,transaction_t *tr);
void cb_rcvinfo    (sipevent_t *se,transaction_t *tr);
void cb_rcvoptions (sipevent_t *se,transaction_t *tr);
void cb_rcvregister(sipevent_t *se,transaction_t *tr);
void cb_rcvprack   (sipevent_t *se,transaction_t *tr);
void cb_rcvunkrequest(sipevent_t *se,transaction_t *tr);

void cb_sndinvite  (sipevent_t *se,transaction_t *tr);
void cb_sndack     (sipevent_t *se,transaction_t *tr);
void cb_sndbye     (sipevent_t *se,transaction_t *tr);
void cb_sndcancel  (sipevent_t *se,transaction_t *tr);
void cb_sndinfo    (sipevent_t *se,transaction_t *tr);
void cb_sndoptions (sipevent_t *se,transaction_t *tr);
void cb_sndregister(sipevent_t *se,transaction_t *tr);
void cb_sndprack   (sipevent_t *se,transaction_t *tr);
void cb_sndunkrequest(sipevent_t *se,transaction_t *tr);

void cb_rcv1xx(sipevent_t *se,transaction_t *tr);
void cb_rcv2xx(sipevent_t *se,transaction_t *tr);
void cb_rcv3xx(sipevent_t *se,transaction_t *tr);
void cb_rcv4xx(sipevent_t *se,transaction_t *tr);
void cb_rcv5xx(sipevent_t *se,transaction_t *tr);
void cb_rcv6xx(sipevent_t *se,transaction_t *tr);

void cb_snd1xx(sipevent_t *se,transaction_t *tr);
void cb_snd2xx(sipevent_t *se,transaction_t *tr);
void cb_snd3xx(sipevent_t *se,transaction_t *tr);
void cb_snd4xx(sipevent_t *se,transaction_t *tr);
void cb_snd5xx(sipevent_t *se,transaction_t *tr);
void cb_snd6xx(sipevent_t *se,transaction_t *tr);

void cb_rcvresp_retransmission(transaction_t *tr);
void cb_sndreq_retransmission(transaction_t *tr);
void cb_sndresp_retransmission(transaction_t *tr);
void cb_rcvreq_retransmission(transaction_t *tr);

void cb_killtransaction(transaction_t *tr);
void cb_endoftransaction(transaction_t *tr);

void cb_connection_refused(transaction_t *tr);
void cb_network_error(transaction_t *tr);

extern OsipManager *def_manager;

/* send a sip response from a transaction */
void osip_send_response(OsipManager *mgr,transaction_t *transaction,sip_t *response);


#endif




