#include <string.h>
#include <unistd.h>
#include <signal.h>
#include "osipua.h"

void server(void);
void client(void);
void stop_func(int signum);
int invite_cb(OsipCallLeg *cl,sip_t *msg,void*p);
int bye_cb(OsipCallLeg *cl,sip_t *msg,void*p);
int invite_accepted_cb(OsipCallLeg *cl,sip_t *msg,void*p);
int run_cond=1;

static char *mediatype[2]={"AUDIO","VIDEO"};

int main(int argc, char *argv[])
{
   osipua_init();
	signal(SIGINT,stop_func);
	if (argc!=2)
	{
		printf("usage: osipua_tester [-c] [-s]\n");
		return(-1);
	}
	if (strcmp(argv[1],"-s")==0) server();
	else if (strcmp(argv[1],"-c")==0) client();
	else return(-1);
	return(0);
}

static char *ua_contact= "<sip:pablo@Delphine:5062>";
static char *callee="<sip:simon.morlat@linphone.no-ip.org>";
static char *localip="80.14.24.171";

void client(void)
{
	OsipUA *ua;
	OsipCallLeg *call_leg;
	ua_params_t params;


	memset(&params,0,sizeof(ua_params_t));
	params.contact=ua_contact;
	params.ua_ip4addr=localip;
	params.ua_port=5062;
	params.max_calls=1;
	params.dbg=NULL;

	ua=osip_ua_new(&params);
	/* add rtp profile capabilities */
	osip_ua_add_profile(ua,STREAM_AUDIO,3);
	osip_ua_add_profile(ua,STREAM_AUDIO,4);
	osip_ua_set_rtpport(ua,STREAM_AUDIO,7000);
	osip_ua_signal_connect(ua,"INVITE_ACCEPTED",invite_accepted_cb);
	osip_ua_signal_connect(ua,"BYE",bye_cb);
	osip_trace(TRACE_LEVEL3,("info: Inviting %s\n",callee));
	call_leg=osip_call_leg_new(ua);
	osip_call_leg_invite(call_leg,callee);

	osip_trace(TRACE_LEVEL3,("info: sleeping...\n"));
	while (run_cond) sleep(1);
	//scanf("%i",&err);
	osip_call_leg_bye(call_leg);
	sleep(2);
	osip_trace(TRACE_LEVEL3,("info: exiting...\n"));
	osip_ua_destroy(ua);
	exit(1);
}
void server(void)
{
	OsipUA *ua;
	ua_params_t params;

	memset(&params,0,sizeof(ua_params_t));
	params.contact=ua_contact;
	params.ua_ip4addr=localip;
	params.ua_port=5064;
	params.max_calls=1;
	params.dbg=NULL;

	ua=osip_ua_new(&params);
	osip_ua_add_profile(ua,0,4);
	osip_ua_set_rtpport(ua,STREAM_AUDIO,7002);
	osip_ua_signal_connect(ua,"INVITE",invite_cb);
	osip_ua_signal_connect(ua,"BYE",bye_cb);
	osip_trace(TRACE_LEVEL3,("sleeping...\n"));
	while (run_cond) sleep(1);
	//scanf("%i",&err);
	osip_trace(TRACE_LEVEL3,("exiting...\n"));
	osip_ua_destroy(ua);
	exit(1);
}

int invite_cb(OsipCallLeg *cl,sip_t *msg,void*p)
{
	sip_t *sipmsg=(sip_t*)p;
	char *from;
	list_t *streams;
	MediaDesc *desc;
	int i,*prof;
	
	from_2char(sipmsg->from,&from);
	osip_trace(TRACE_LEVEL0,("Receiving invite from %s.\n",from));
	sfree(from);
	sleep(1);
	osip_trace(TRACE_LEVEL3,("Accepting invite.\n"));
	streams=osip_call_leg_accept_invite(cl);
	if (streams==NULL)
	  {
	    osip_trace(TRACE_LEVEL3,("error: Could not process the incoming invite\n"));
	  }
	else
	  {
	    osip_trace(TRACE_LEVEL3,("Invite is accepted with the following sdp:\n"));
	    for (i=0;i<streams->nb_elt;i++)
	      {
		desc=(MediaDesc*)list_get(streams,i);
		prof=list_get(desc->profile_list,0);
		osip_trace(TRACE_LEVEL0,("info: Type: %s   port: %i     profile:%i\n",mediatype[desc->type],desc->rem_port,prof[0]));
	      }
	  }
	return(0);
}

int bye_cb(OsipCallLeg *cl,sip_t *msg,void*p)
{
	sip_t *sipmsg=(sip_t*)p;
	char *from;
	from_2char(sipmsg->from,&from);
	osip_trace(TRACE_LEVEL0,("info: Receiving bye from %s.\n",from));
	sfree(from);
	exit;
	return(0);
}	

int invite_accepted_cb(OsipCallLeg *cl,sip_t *msg,void*p)
{
	int i;
	int *prof;
	MediaDesc *desc;
	list_t *streams;
	
	osip_trace(TRACE_LEVEL0,("info: Invite accepted by remote server!\n"));
	streams=osip_call_leg_get_streams(cl);
	if (streams==NULL)
	  {
	    osip_trace(TRACE_LEVEL0,("error: No streams properties !!!!\n"));
	  }
	else
	{
		osip_trace(TRACE_LEVEL3,("info: Invite was accepted with the following sdp:\n" ));
		for (i=0;i<streams->nb_elt;i++)
		{
			desc=(MediaDesc*)list_get(streams,i);
			prof=list_get(desc->profile_list,0);
			osip_trace(TRACE_LEVEL3,("info: Type: %s   port: %i     profile:%i\n",mediatype[desc->type],desc->rem_port,prof[0]));
		}
	}
	return(0);
}

void stop_func(int signum)
{
	run_cond=0;
}

