/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "resolver.h"
#include "osipmanager.h"
#include "osipua.h"
#include <osip/smsg.h>
#include <osip/smsgtypes.h>
#include <osip/port.h>
#include <osip/fifo.h>


#ifndef __VXWORKS_OS__
#ifdef WIN32
#include <winsock.h>
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif
#endif

#ifdef __VXWORKS_OS__
#include <vxWorks.h>
#include <sockLib.h>
#include <inetLib.h>
#include <hostLib.h>
#include <resolvLib.h>
#endif



struct hostent *resolv(char *name) {
	unsigned long int addr;
	struct hostent *hp = NULL;

  if ((int)(addr = inet_addr(name)) == -1)
    {
    /* The address is not of the form a.b.c.d */
#ifndef __VXWORKS_OS__
    hp = gethostbyname(name);
    /*    fprintf(stdout,"Address is of type name %li\n",addr); */
#endif
#ifdef __VXWORKS_OS__
    char *pbuf = (char *) smalloc(513*sizeof(char));
    hp = resolvGetHostByName(name,pbuf,512);
    /* we should check errno */
#endif
    }
  else
    {
#ifndef __VXWORKS_OS__
    hp = gethostbyaddr((char *)&addr, sizeof (addr), AF_INET);
    /* fprintf(stdout,"Address is of type a.c.d.e %li\n",addr); */
#endif
#ifdef __VXWORKS_OS__
    char *pbuf = (char *) smalloc(513*sizeof(char));
    /* hp = resolvGetHostByAddr((char *)&addr ,pbuf ,512); */
#endif
    }
  if (hp == NULL) {
    osip_trace(TRACE_LEVEL0,("error: host information for %s not found\n",name));
    return NULL;
  }

  return hp;
}

void * resolver_thread(void *p_manager)
{
	trans_event_t *transevent;
	transaction_t *trans;
	sipevent_t *sipevent;
	sip_t *sipmsg;
	char **name;
	OsipManager *manager=(OsipManager*)p_manager;
	struct hostent *hi;
	
	osip_trace(TRACE_LEVEL4,("info: Entering name resolution thread.\n"));
	while(manager->resolv_run_cond)
	{
		transevent=(trans_event_t *) fifo_get(manager->resolv_fifo);
		osip_trace(TRACE_LEVEL4,("Doing asynchronous name resolution.\n"));
		if (transevent!=NULL)
		{
			trans=transevent->transaction;
			sipevent=transevent->event;
			sipmsg=sipevent->sip;
			sfree(transevent);
			
			if (MSG_IS_REQUEST(sipmsg))
			{
				if ( trans->proxy==NULL)
				{
					/* we must resolve the request-uri */
					name=&sipmsg->strtline->rquri->host;
				}
				else
				{  /* we must resolv the outbound proxy */
					name=&  trans->proxy->host;
				}
				hi=resolv(*name);
				if (hi!=NULL)
				{
					sfree(*name);
					*name=sgetcopy(inet_ntoa(((struct in_addr*)hi->h_addr)[0]));
					fifo_add(trans->transactionff,sipevent);
					/* we need to remove the transaction from the main fifo of the main thread to transaction_execute() here.
					But that needs a fifo_remove(fifo,element) that does not exist for the moment */
					//transaction_execute(trans,sipevent);
					/* so the transaction is executed by osip_execute() in the main thread */
				}
				else trans->state=NETWORK_ERROR;
			}
			else
			{
				/* we must resolv the via field */
				via_t *via;
				via=(via_t*)list_get(sipmsg->vias,0);
				if (via==NULL) osip_trace(TRACE_LEVEL0,( "error: Could not get the first via header.\n"));
				name=&via->host;
				hi=resolv(*name);
				if (hi!=NULL)
				{
					sfree(*name);
					via->host=sgetcopy(inet_ntoa(((struct in_addr*)hi->h_addr)[0]));
					fifo_add(trans->transactionff,sipevent);
					//transaction_execute(trans,sipevent);
				}
				else trans->state=NETWORK_ERROR;
			}
		}
		else osip_trace(TRACE_LEVEL3,("warning: Null event !\n"));
	}
	return NULL;
}


