/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "osipua.h"
#include "utils.h"
#include "uatransaction.h"

transaction_t * ua_transaction_new(OsipCallLeg *call,sip_t *sipmsg)
{
	transaction_t *transaction;
	OsipUA *ua;
	if (call==NULL) return NULL;
	
	ua=(OsipUA*)call->ua;
	transaction = (transaction_t *) smalloc(sizeof(transaction_t));
	transaction_init(ua->config,
	   		transaction,
	   		sipmsg->to,
	   		sipmsg->from,
	   		sipmsg->call_id,
	   		sipmsg->cseq);
	/* if an outbound proxy is set for this call-leg, then set it to the transaction */
	if ((ua->registrar!=NULL) && (ua->flags & OSIP_UA_USE_PROXY) )
	{
		url_clone(ua->registrar,&transaction->proxy);
		/* it must be freed when the transaction is finished !*/
	}
	   		
	ua_transaction_outgoing_set_call_leg(transaction,call);
	
	return transaction;
}

void ua_transaction_incoming_set_call_leg(transaction_t *trn,OsipCallLeg *call)
{
	/* add the transaction to the call-leg list of incoming transaction*/
	list_add(&call->incoming_transactions,trn,-1);
	trn->your_instance=call;
}

void ua_transaction_outgoing_set_call_leg(transaction_t *trn,OsipCallLeg *call)
{
	/* add the transaction to the call-leg list of outgoing transaction*/
	list_add(&call->outgoing_transactions,trn,-1);
	trn->your_instance=call;
}


void ua_transaction_free(transaction_t *transaction)
{
	OsipCallLeg *call;
	OsipUA *ua;
	int error;
	if (transaction->proxy!=NULL)
	{
		url_free(transaction->proxy);
		sfree(transaction->proxy);
	}
	/* check if the call_leg that the transaction is part of should not be deleted too*/
	call=ua_transaction_get_call_leg(transaction);
	if (call==NULL)
	{
		osip_trace(TRACE_LEVEL2,("error: CallLeg==NULL ! This must not happen!"));
		return;
	}
	/* remove the transaction from the call leg list of transaction*/
	/* FIXME: we try to remove it on the two list, but we know it can
	be only in one of them */
	list_remove_el(&call->incoming_transactions,(void*)transaction);
	list_remove_el(&call->outgoing_transactions,(void*)transaction);
	ua=osip_call_leg_get_ua(call);
	/* now if it was the last active transaction of the call-leg, delete the call_leg*/
	if (list_size(&call->incoming_transactions)+list_size(&call->outgoing_transactions)==0)
	{
  	switch (call->status)
  	{
  		case CALL_REGISTERING:
  		case CALL_INVITING:
  			error=-ETIMEDOUT;
  			/* the invite times out. The call-leg is lost. */
  			if (ua->faillure!=NULL)
  				ua->faillure(call,NULL,&error);
  			/* then destroy the call-leg*/
  			osip_call_leg_destroy(call);
  		break;
  		case CALL_TERMINATED:
  			osip_call_leg_destroy(call);
  		break;
  	}
  }
}

