/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <osip/osip.h>
#include <osip/timers.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include "udp.h"
#include "osipmanager.h"
#include "osipua.h"

extern OsipManager *def_manager;

static int udp_receive(OsipManager *manager);

void *sipd_thread(void *managerp)
{
	long int time_elapsed;
	struct timeval t1,t2;
	transaction_t *transaction;
	OsipManager *manager=(OsipManager*)managerp;
	
    /* SIP_MESSAGE_MAX_LENGTH should be set to 66001
	 	to improve interopperability */
	/* the main loop */
	osip_trace(TRACE_LEVEL4,("info: Entering osipua thread.\n"));
	gettimeofday(&t1,NULL);
	while(manager->udp_run_cond)
	{
		udp_receive(manager);
		gettimeofday(&t2,NULL);
		
		time_elapsed= ((t2.tv_sec-t1.tv_sec)*1000) +  ((t2.tv_usec-t1.tv_usec)/1000);
		
		/* execute timers if needed */
		//printf("Time elapsed: %i\n", time_elapsed);
		if ( time_elapsed > 500)   /*ms*/
		{
			//printf("Executing timers\n");
			t1.tv_sec=t2.tv_sec;
			t1.tv_usec=t2.tv_usec;
			timers_execute(manager->config->uas_transactions);
			timers_execute(manager->config->uac_transactions); 	
		}
		
		transaction = (transaction_t *)fifo_tryget(manager->config->uas_timerff);
		while (transaction!=NULL)
		{
			list_add(manager->config->uas_transactions,transaction,-1);
			transaction = (transaction_t *)fifo_tryget(manager->config->uas_timerff);
		}

		transaction = (transaction_t *)fifo_tryget(manager->config->uac_timerff);
		while (transaction!=NULL)
		{
			list_add(manager->config->uac_transactions,transaction,-1);
			transaction = (transaction_t *)fifo_tryget(manager->config->uac_timerff);
		}
		//printf("Executing osip stack..\n");
		osip_execute(manager->config);
		
	}
	osip_trace(TRACE_LEVEL4,("info: Exiting osipua thread.\n"));
	return NULL;
}

/* this function waits 100 ms until a packet arrives , return 1 if a packet is received, 0 else */
static int udp_receive(OsipManager *manager)
{
	int i,j,k,nb;
	fd_set osipfdset;
	struct timeval timeout;
	
	memcpy(&osipfdset,&manager->udpfdset,sizeof(fd_set));
	timeout.tv_sec=manager->recv_tout.tv_sec;               /* timeout can be modified by linux !!*/
	timeout.tv_usec=manager->recv_tout.tv_usec;
	nb=select(manager->max_udpfd+1,&osipfdset,NULL,NULL,&timeout);
	/* if something to read on the control file descriptor, then exit thread*/
	if (nb>0)
	{	
		if (FD_ISSET(manager->udp_unblock_fd,&osipfdset))
		{
			//printf("Receiving something on unblock_fd\n");
			read(manager->udp_unblock_fd, manager->udp_buf, SIP_MESSAGE_MAX_LENGTH);
		}
		//printf("Something to read !!!!!!!!!!!!!!!!!\n");
		for (j=0,k=0;(j<OSIP_MAX_UDP_PORTS) && (k<nb);j++)
		{
			if (FD_ISSET(manager->udpfds[j],&osipfdset))
			{
				k++;
				i = recv(manager->udpfds[j], manager->udp_buf, SIP_MESSAGE_MAX_LENGTH ,  0);
				if( i > 0 )
				{
					/* Message might not end with a "\0" but we know the number of */
					/* char received! */
					transaction_t *transaction=NULL;
					sipevent_t *sipevent;
					manager->udp_buf[i]='\0';
					osip_trace(TRACE_LEVEL2,("info: RECEIVING UDP MESSAGE:\n%s\n",manager->udp_buf));
					sipevent = osip_parse(manager->udp_buf);
					if (sipevent!=NULL)
						transaction = osip_distribute_event(manager->config,sipevent);
#if 0
#ifdef OSIP_MT
					if (transaction!=NULL)
					{
						if (transaction->your_instance==NULL)
						{
							transaction_mt_t *transaction_mt;
							transaction_mt_init(&transaction_mt);
							transaction_mt_set_transaction(transaction_mt,transaction);
							transaction_mt_start_transaction(transaction_mt);
						}
					}
#endif
#endif
				}
				else  /* if i>0 */
				{
					TRACE(trace(__FILE__,__LINE__,TRACE_LEVEL0,NULL,"UDP listener failed while receiving data!\n"));
				}
			}
		}
		return 1;   /* messages have been received */
	}
	
	return 0;  /* no messages : timeout  */
}

static int udp_send(char *message,char *host,int port)
{
  struct sockaddr_in addr;
  unsigned long int  one_inet_addr;


  if ((int)(one_inet_addr = inet_addr(host)) == -1)
    {
      osip_trace(TRACE_LEVEL0,("error: There is no support for name resolution in osip.\n"));
    }
  else
    {
      addr.sin_addr.s_addr = one_inet_addr;
    }

  addr.sin_port        = htons((short)port);
  addr.sin_family      = AF_INET;

  if (def_manager->send_sock==0) 
    {
      osip_trace(TRACE_LEVEL0,("error: No socket for sending messages. Aborting.\n"));
    }

  if (0  > sendto (def_manager->send_sock, (const void*) message, strlen (message), 0,
		      (struct sockaddr *) &addr, sizeof(addr)))
    {
#ifdef WIN32
      if (WSAECONNREFUSED==WSAGetLastError())
#else
      if (ECONNREFUSED==errno)
#endif
	{
	  TRACE(trace(__FILE__,__LINE__,TRACE_LEVEL3,stdout,
		"SIP_ECONNREFUSED - No remote server.\n"));
	  /* This can be considered as an error, but for the moment,
	     I prefer that the application continue to try sending
	     message again and again... so we are not in a error case.
	     Nevertheless, this error should be announced!
	     ALSO, UAS may not have any other options than retry always
	     on the same port.
	  */
	  return 1;
	}
      else
	{
	  TRACE(trace(__FILE__,__LINE__,TRACE_LEVEL3,stdout,
		"SIP_NETWORK_ERROR - Network error.\n"));
	  /* SIP_NETWORK_ERROR; */
	  return -1;
	}
    }

  /* #endif */
  TRACE(trace(__FILE__,__LINE__,TRACE_LEVEL2, NULL,
	      "SND UDP MESSAGE.\n"));

  return 0;
}

int
udp_send_response(sip_t *response)
{
  char *mess;
  via_t *via;
  int port;
  int i;

  /*  via = (via_t *)smalloc(sizeof(via_t)); */
  if (-1==msg_getvia(response,0,&via))
    {
      osip_trace(TRACE_LEVEL0,("error: Bad via header\n"));
      return -1;
    }
  i = msg_2char(response, &mess);
  if (i==-1)
    {
      osip_trace(TRACE_LEVEL3,("error: could not build message\n"));
      return -1;
    }
  if (via->port!=NULL)
    port = atoi(via->port);
  else
    port = 5060;

  i = udp_send(mess,via->host, port);
  sfree(mess);
  if (i==-1)
    {
      osip_trace(TRACE_LEVEL2,("error: could not send REQUEST\n"));
      return -1; /* error */
    }
  DEBUG(msg_logresponse(response,"SND %s %s (%s) f:%s t:%s cseq:%s callid:%s\n"));
  return 0; /* return 1 on success and i on error */
}

int
udp_send_request(sip_t *request, url_t *proxy)
/* day3: support for proxy usage
   udp_send_request(sip_t *request) */
{
  char *mess;
  url_t *url;
  int port;
  int i;

   if (proxy==NULL)
    url = request->strtline->rquri;
   else
    url = proxy;

  if (url->port!=NULL)
    port = atoi(url->port);
  else
    port = 5060;
  i = msg_2char(request, &mess);
  if (i==-1)
    {
    osip_trace(TRACE_LEVEL2,("error: could not build REQUEST.\n"));
    return -1; /* error */
    }

  i = udp_send(mess, url->host, port);
  sfree(mess);
  if (i==-1)
    {
      osip_trace(TRACE_LEVEL0,("error: Could not send REQUEST.\n"));
      return -1; /* error */
    }
  DEBUG(msg_logrequest(request,"SND %s f:%s t:%s cseq:%s callid:%s\n"));
  return 0;
}

