/*
  The osipua library is a library based on oSIP that implements CallLeg and User Agent
  level.
  Copyright (C) 2001  Simon MORLAT simon.morlat@free.fr
  											Aymeric MOIZARD jack@atosc.org
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef UTILS_H
#define UTILS_H

#include <osip/osip.h>
#include <osip/smsg.h>
#include <osip/port.h>
#include "osipua.h"



/* profiles is a list of profiles supported by remote host, and ua_profile is the list for local user agent. The return value is the preferred profile for ua, compatible with
remote host 's list. */
int get_compatible_profile(list_t *ua_profiles,list_t *profiles);


/* respond to a request*/
void respond_to_request(osip_t *config, transaction_t *transaction, int code, char *body);

int msg_makereply(int status,char *reason,sip_t *request,
	      sip_t *response,char *body,char *content_type
	      ,int options);
	
/* returns an sdp message with properties given as argument*/
char *sdp_compose(int sess_id, char *user, char *ip4addr,char *subject, list_t *media_list);


/*search element el in list, and remove it*/
int list_remove_el(list_t *list,void *el);

/* copy callid_t source to dest, by allocating fields inside dest*/
void callid_getcopy(call_id_t	*dest,call_id_t *source);

#define LIST_ELEM_DESTRUCTOR(func)     ((void (*)(void *))func)
/* free a list with all its elements and nodes. destructor() is used ot free elements*/
void list_free(list_t *li, void (*destructor)(void *));

struct hostent *resolv(char *name);

/* from2 has tag parameter that we should copy to from1 if it does not have one */
int from_copy_tag(from_t *from1,from_t *from2);


/* allocate a random tag string */
#define tag_init(pc) _tag_init( (pc),5)

int _tag_init(char **tag,int length);

#define from_tag_add(_from,_tag) from_param_add((_from),sgetcopy("tag"),(_tag))

#define from_tag_add_random(_from) \
	do \
	{\
		char *_tag; \
		tag_init(&_tag); \
		from_param_add((_from),sgetcopy("tag"),_tag); \
	}while (0)

/* BKO: similar to from_tag_add */
#define to_tag_add(_to,_tag) to_param_add((_to),sgetcopy("tag"),(_tag))

/* this function check is *url has a sip: prefix. If it has not, then it add ones in a newly allocated string. If no correction is made, it just returns
the initial string */
char * check_sipurl(char *url);

/* char **url : does the same but if a new string is created, the old one is freed */
#define remake_sipurl(url) do \
{             \
	char *__newurl; \
	char **__url=url;           \
	__newurl=check_sipurl( *__url ); \
	if ((*__url)!=__newurl) sfree(*__url); \
	*__url=__newurl;                             \
} while(0)	


#endif

