/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifndef NOTGNOME
#include <gnome.h>
#endif

#include <stdlib.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <sys/errno.h>
#include <string.h>
#include <signal.h>

#include "LinphoneMain.h"
#include "../audio/ring.h"
#include "../audio/audio.h"
#include "sip_handlers.h"

/* table of different bandwidth for different connection type*/
const double bandwidths[5]=
{
	28800.0,	/*28k modem*/
	28800.0, /*56k modem*/
	64000.0, /*64k modem (numeris)*/
	128000.0,	/*adsl or cable*/
	10e6, /* ethernet */
};/* carefull: these are the upstream bandwidth !!!*/


/* init object */
void linphone_main_init(LinphoneMain *lpm)
{
	memset(lpm,0,sizeof(LinphoneMain));
	osipua_init();
}
/* if there is no config file, use this functions to set default parameters*/
/* error codes=
	-3: local loopback used
	-4: no networks interface founds
	*/
int linphone_main_first_startup(LinphoneMain *lpm)
{
	int err,retval=0,len,index=0;
	char *ring_path,*p;
	
	/* set network configuration*/
	lpm->net_conf.if_count=err=retrieve_if(lpm);
	switch(err)
	{
		case 1:
			printf("Warning: using local loopback network->\n");
			lpm->net_conf.local_addr="127.0.0.1";
			lpm->net_conf.if_name="lo";
			retval=-3;
			break;
		case -1:
			printf("ERROR: No network interface found.\n");
			retval=-4;
			break;
		default:
			if (strcmp(lpm->net_conf.if_names[0],"lo")==0) index++;
			printf("using %s interface with ip address %s\n",lpm->net_conf.if_names[index],lpm->net_conf.if_addr[index]);
			lpm->net_conf.local_addr=lpm->net_conf.if_addr[index];
			lpm->net_conf.if_sel=index;
			lpm->net_conf.if_name=lpm->net_conf.if_names[index];
			break;
	}
	lpm->net_conf.con_type=1;   /* default to 56k modem*/
	lpm->net_conf.bandwidth=bandwidths[(int)lpm->net_conf.con_type];
	/* set sip configuration*/
	lpm->sip_conf.sip_port=5060;
	p=getenv("LOGNAME");
	if (p==NULL) p="NO_USER_NAME";
	lpm->sip_conf.username=sgetcopy(p);
	p= getenv("HOSTNAME");
	if (p==NULL) p="NO_HOST_NAME";
	lpm->sip_conf.hostname=sgetcopy(p);
	lpm->sip_conf.reg_conf.expires=REG_EXPIRES;  /* 15 minutes*/
		
	/*set rtp configuration*/
	lpm->rtp_conf.jitt_comp=7;
	lpm->rtp_conf.audio_rtp_port=7078;


	/* set codec configuration*/
	linphone_main_codecs_check(lpm);
	
	/* set default sound driver */
#ifdef ALSA
	lpm->sound_conf.driver_mode=1;
	err=audio_alsa_init();
#else
	lpm->sound_conf.driver_mode=0;
	err=audio_oss_init();
#endif
	if (err!=0) printf("Cannot initialise sound support.\n");
	/* set recording source*/
	lpm->sound_conf.source='m';
	set_source('m');
	/* set default levels*/
	lpm->sound_conf.rec_lev=75;
	lpm->sound_conf.play_lev=75;
	
	set_in_level(lpm->sound_conf.rec_lev);
	set_out_level(lpm->sound_conf.play_lev);

	lpm->sound_conf.autokill=0;
	/* loads ring file */
	len=strlen(PACKAGE_DATA_DIR)+strlen(SOUND_REMOTE_RING)+2;
	ring_path=malloc(len);
	snprintf(ring_path,len,"%s/%s",PACKAGE_DATA_DIR,SOUND_LOCAL_RING);
	open_ringfile(ring_path);
	snprintf(ring_path,len,"%s/%s",PACKAGE_DATA_DIR,SOUND_REMOTE_RING);
	open_ringbackfile(ring_path);
	free(ring_path);
	return(retval);
}


/* error codes=
	-1: config file not found.
	-2: previous network interface not found
	-3: local loopback used
	-4: no networks interface founds
	*/	
/* this functions loads linphone configuration from a config file. If the file does not exists, returns -1.*/
int linphone_main_read_config(LinphoneMain *lpm)
{
	int err=0,i,retval=0;
	int test,len,tmp;
	char * ring_path;
	GList *list;
	char **vector;
	codec_info_t *codec;
	gchar *ab_entry=g_malloc(100);

#ifndef NOTGNOME	
	/* do a test to find if there is a config file*/
	test=gnome_config_has_section("/linphone/global");
	if (!test) return(-1);
	/* get version of this config file*/
	test=gnome_config_get_int("/linphone/version/version_id");
	if (test==0)
	{
		/* version not found, old linphone*/
		printf("Old config file found: deleted.\n");
		gnome_config_drop_all();
		gnome_config_clean_file("/linphone");
		gnome_config_sync();
		return(-1);
	}
	
	/* there is a config file, so read it*/
	/*network config*/
	lpm->net_conf.local_addr=gnome_config_get_string("/linphone/global/local_addr");
	lpm->net_conf.if_name=gnome_config_get_string("/linphone/global/if_name");
	lpm->net_conf.con_type=gnome_config_get_int("/linphone/global/con_type");
	lpm->net_conf.bandwidth=bandwidths[(int)lpm->net_conf.con_type];
	/* rtp config*/
	lpm->rtp_conf.audio_rtp_port=gnome_config_get_int("/linphone/rtp/audio_rtp_port");
	lpm->rtp_conf.jitt_comp=gnome_config_get_int("/linphone/rtp/jitt_comp");
	/* sound config*/
	lpm->sound_conf.driver_mode=gnome_config_get_int("/linphone/audio/driver_mode");
	lpm->sound_conf.rec_lev=gnome_config_get_int("/linphone/audio/rec_lev");
	lpm->sound_conf.play_lev=gnome_config_get_int("/linphone/audio/play_lev");
	lpm->sound_conf.source=gnome_config_get_int("/linphone/audio/source");
	lpm->sound_conf.autokill=gnome_config_get_int("/linphone/audio/autokill");
	/* sip config*/
	lpm->sip_conf.sip_port=gnome_config_get_int("/linphone/sip/sip_port");	
	/*temp code for sip config*/
	lpm->sip_conf.username=gnome_config_get_string("/linphone/sip/username");
	if (lpm->sip_conf.username==NULL) lpm->sip_conf.username=sgetcopy("toto");
	lpm->sip_conf.hostname=gnome_config_get_string("/linphone/sip/hostname");
	if (lpm->sip_conf.hostname==NULL) lpm->sip_conf.hostname=sgetcopy("at.home");
	lpm->sip_conf.reg_conf.use_registrar=gnome_config_get_int("/linphone/sip/use_registrar");
	lpm->sip_conf.reg_conf.registrar=gnome_config_get_string("/linphone/sip/registrar");	
	lpm->sip_conf.reg_conf.passwd= gnome_config_get_string("/linphone/sip/reg_passwd");
	lpm->sip_conf.reg_conf.addr_of_rec= gnome_config_get_string("/linphone/sip/addr_of_rec");
	lpm->sip_conf.reg_conf.expires= gnome_config_get_int("/linphone/sip/reg_expires");
	if (lpm->sip_conf.reg_conf.expires==0)  lpm->sip_conf.reg_conf.expires=REG_EXPIRES;
	lpm->sip_conf.reg_conf.as_proxy= (unsigned int) gnome_config_get_int("/linphone/sip/as_proxy");
	lpm->sip_conf.reg_conf.as_redirect= (unsigned int) gnome_config_get_int("/linphone/sip/as_redirect");
	lpm->sip_conf.reg_conf.as_outbound= (unsigned int) gnome_config_get_int("/linphone/sip/as_outbound");
	
	
	
	/* get codecs config*/
	gnome_config_get_vector("/linphone/codecs/audio_codecs",&len,&vector);
	if (len==0) printf("No codec list in config file!\n");
	list=NULL;
	for (i=0;i<len;i++)
	{
		tmp=atoi(vector[i]);
		codec=linphone_main_get_codec(lpm,tmp & 0xFF);
		if (codec!=NULL)
		{
			list=g_list_append(list,codec);
			codec->is_selected=tmp>>8;
		}
		else printf("Warning: unrecognised profile in config file.\n");
	}
	lpm->codecs_conf.audio_codecs=list;
	
	/* get address book entries*/
	len= gnome_config_get_int("/linphone/address_book/entry_count");
	if (len!=-1)
	{
		for (tmp=0;tmp<len;tmp++)
		{
			snprintf(ab_entry,100,"/linphone/address_book/entry%i",tmp);
			lpm->gui_conf.address_list=g_list_append(lpm->gui_conf.address_list,(gpointer)gnome_config_get_string(ab_entry));
		}
	}
	
	/* set codec configuration*/
	linphone_main_codecs_check(lpm);
	
#else
	FILE *cfg_file;
	char *home;
	char cfg_path[100];
	char keyword[30],value[30];
	
	
	home=getenv("HOME");
	snprintf(cfg_path,100,"%s/.gnome/apps/linphone.cfg",home);
	
	cfg_file=fopen(cfg_path,"r");
	if (cfg_file==NULL)
	{
		printf("Cannot open linphone config file. Perhaps it is the first time you run it ?\n");
		return(-1);
	}
	while(fscanf(cfg_file,"%s %s",keyword,value)==2)
	{
		
		if (strcmp(keyword,"audio_port=")==0)
		{
			linphone_conf->rtp_port=atoi(value);
		}
		else if (strcmp(keyword,"sip_port=")==0)
		{
			linphone_conf->sip_port=atoi(value);
		}
		else if (strcmp(keyword,"jitt_comp=")==0)
		{
			linphone_conf->jitt_comp=atoi(value);
		}
		else if (strcmp(keyword,"if_name=")==0)
		{
			strcpy(linphone_conf->if_name,value);
		}
		else if (strcmp(keyword,"profile=")==0)
		{
			 linphone_conf->profile=atoi(value);
		}
		else if (strcmp(keyword,"sound=")==0)
		{
			linphone_conf->sound_driver=atoi(value);
		}
		else if (strcmp(keyword,"source=")==0)
		{
			linphone_conf->source=value[0];
		}
		else if (strcmp(keyword,"rec_lev=")==0)
		{
			linphone_conf->rec_lev=atoi(value);
		}
		else if (strcmp(keyword,"play_lev=")==0)
		{
			linphone_conf->play_lev=atoi(value);
		}
	}
	fclose(cfg_file);
#endif
	
#ifdef ALSA
	if (lpm->sound_conf.driver_mode==1)
		err=audio_alsa_init();
	if (lpm->sound_conf.driver_mode==0)
		err=audio_oss_init();
#else
	if (lpm->sound_conf.driver_mode==0)
		err=audio_oss_init();
	else
	{
		printf("You cannot use alsa drivers: recompile linphone with installed alsa drivers and library for that, or use oss.");
		err=audio_oss_init();
		lpm->sound_conf.driver_mode=0;
	}
#endif
	if (err!=0) printf("Cannot initialise sound support.\n");
	/* loads ring file */
	len=strlen(PACKAGE_DATA_DIR)+strlen(SOUND_REMOTE_RING)+2;
	ring_path=malloc(len);
	snprintf(ring_path,len,"%s/%s",PACKAGE_DATA_DIR,SOUND_LOCAL_RING);
	open_ringfile(ring_path);
	snprintf(ring_path,len,"%s/%s",PACKAGE_DATA_DIR,SOUND_REMOTE_RING);
	open_ringbackfile(ring_path);
	free(ring_path);
	/* set audio source and levels */
	set_source(lpm->sound_conf.source);
	set_in_level(lpm->sound_conf.rec_lev);
	set_out_level(lpm->sound_conf.play_lev);
	
	/* check if saved network configuration is compatible with current one*/
	lpm->net_conf.if_count=err=retrieve_if(lpm);
	switch(err)
	{
		case 0:
			retval=-2;
			printf("Warning: using local loopback network\n");
			lpm->net_conf.local_addr="127.0.0.1";
			/*strcpy(linphone_conf->if_name,"lo");*/
			break;
		case -1:
			printf("ERROR: No network interface found.\n");
			retval=-4;
			break;
		default:
			retval=-2;
			if (strcmp("lo",lpm->net_conf.if_name)==0)
			{	/* find first availlable interface and exits*/
				printf("Using %s interface with ip address %s\n",lpm->net_conf.if_names[0],lpm->net_conf.if_addr[0]);			
				lpm->net_conf.local_addr=lpm->net_conf.if_addr[0];
				lpm->net_conf.if_sel=0;
				lpm->net_conf.if_name=lpm->net_conf.if_names[0];
				retval=0;
			}
			else
			{   /* else try to find previously used network interface */
				for(i=0;i<err;i++)
				{
					if (strcmp(lpm->net_conf.if_name,lpm->net_conf.if_names[i])==0)
					{
						printf("Found %s interface with ip address %s\n",lpm->net_conf.if_names[i],lpm->net_conf.if_addr[i]);			
						lpm->net_conf.local_addr=lpm->net_conf.if_addr[i];
						lpm->net_conf.if_sel=i;
						retval=0;
					}
				}
				if (retval==-2)
				{
					printf("Cannot find interface %s.\n",lpm->net_conf.if_name);
					printf("using %s interface with ip address %s\n",lpm->net_conf.if_names[0],lpm->net_conf.if_addr[0]);
					lpm->net_conf.local_addr=lpm->net_conf.if_addr[0];
					lpm->net_conf.if_sel=0;	
				}
			}
			break;
	}	
	return(retval);
}

/* save current linphone configuration*/
int linphone_main_save_config(LinphoneMain *lpm)
{
	int len,i,tmp;
	char **vector;
	char *table;
	codec_info_t *codec;
	GList *elem;
	gchar *ab_entry=g_malloc(100);
#ifdef NOTGNOME	
	char *home;
	char cfg_path[100];
	
	
	
	FILE *cfg_file;
	home=getenv("HOME");
	snprintf(cfg_path,100,"%s/.gnome/apps/linphone.cfg",home);
	
	cfg_file=fopen(cfg_path,"w");

	if (cfg_file==NULL)
	{
		printf("Cannot save config.\n");
		return(-1);
	}
	fprintf(cfg_file,"local_addr= %s\n",linphone_conf->local_addr);
	fprintf(cfg_file,"sip_port= %i\n",linphone_conf->sip_port);	
	fprintf(cfg_file,"audio_port= %i\n",linphone_conf->rtp_port);	
	fprintf(cfg_file,"jitt_comp= %i\n",linphone_conf->jitt_comp);
	fprintf(cfg_file,"if_name= %s\n",linphone_conf->if_name);
	fprintf(cfg_file,"profile= %i\n",linphone_conf->profile);
	fprintf(cfg_file,"sound= %i\n",linphone_conf->sound_driver);
	fprintf(cfg_file,"rec_lev= %i\n",linphone_conf->rec_lev);
	fprintf(cfg_file,"play_lev= %i\n",linphone_conf->play_lev);
	fprintf(cfg_file,"source= %c\n",linphone_conf->source);
	
	fclose(cfg_file);
#else
	gnome_config_set_int("/linphone/version/version_id",INT_VERSION);
	gnome_config_set_string("/linphone/global/local_addr",lpm->net_conf.local_addr);
	gnome_config_set_string("/linphone/global/if_name",lpm->net_conf.if_name);
	gnome_config_set_int("/linphone/global/con_type",lpm->net_conf.con_type);
	gnome_config_set_int("/linphone/rtp/audio_rtp_port",lpm->rtp_conf.audio_rtp_port);
	gnome_config_set_int("/linphone/rtp/jitt_comp",lpm->rtp_conf.jitt_comp);
	gnome_config_set_int("/linphone/audio/driver_mode",lpm->sound_conf.driver_mode);
	gnome_config_set_int("/linphone/audio/rec_lev",lpm->sound_conf.rec_lev);
	gnome_config_set_int("/linphone/audio/play_lev",lpm->sound_conf.play_lev);
	gnome_config_set_int("/linphone/audio/source",lpm->sound_conf.source);
	gnome_config_set_int("/linphone/audio/autokill",lpm->sound_conf.autokill);
	gnome_config_set_int("/linphone/sip/sip_port",lpm->sip_conf.sip_port);
	gnome_config_set_int("/linphone/sip/use_registrar",lpm->sip_conf.reg_conf.use_registrar);
	gnome_config_set_string("/linphone/sip/username",lpm->sip_conf.username);
	gnome_config_set_string("/linphone/sip/hostname",lpm->sip_conf.hostname);
	if (lpm->sip_conf.reg_conf.registrar==NULL) gnome_config_set_string("/linphone/sip/registrar","");
	else gnome_config_set_string("/linphone/sip/registrar",lpm->sip_conf.reg_conf.registrar);
	if (lpm->sip_conf.reg_conf.passwd==NULL) gnome_config_set_string("/linphone/sip/reg_passwd","");
	else gnome_config_set_string("/linphone/sip/reg_passwd",lpm->sip_conf.reg_conf.passwd);
	if (lpm->sip_conf.reg_conf.addr_of_rec==NULL) gnome_config_set_string("/linphone/sip/addr_of_rec","");
	else gnome_config_set_string("/linphone/sip/addr_of_rec",lpm->sip_conf.reg_conf.addr_of_rec);
	gnome_config_set_int("/linphone/sip/reg_expires",lpm->sip_conf.reg_conf.expires);
	gnome_config_set_int("/linphone/sip/as_proxy",lpm->sip_conf.reg_conf.as_proxy);
	gnome_config_set_int("/linphone/sip/as_redirect",lpm->sip_conf.reg_conf.as_redirect);
	gnome_config_set_int("/linphone/sip/as_outbound",lpm->sip_conf.reg_conf.as_outbound);
	
	/* need to set record codecs list*/
	len=g_list_length(lpm->codecs_conf.audio_codecs);
	vector=malloc(len*sizeof(char*));
	table=(char*)malloc(len*8);
	for (i=0;i<len;i++)
	{
		codec=(codec_info_t*)g_list_nth_data(lpm->codecs_conf.audio_codecs,i);
		tmp=codec->profile | (codec->is_selected << 8);   /* we add a special bit to tell if user wants to use it*/
		snprintf(&table[8*i],7,"%i",tmp);
		vector[i]=&table[8*i];
	}
	gnome_config_set_vector("/linphone/codecs/audio_codecs",len,vector);
	/* record address book entries*/
	len=0;
	elem=lpm->gui_conf.address_list;
	while(elem!=NULL)
	{
		snprintf(ab_entry,100,"/linphone/address_book/entry%i",len);
		gnome_config_set_string(ab_entry,(gchar*)(elem->data));
		elem=g_list_next(elem);
		len++;
	}
	gnome_config_set_int("/linphone/address_book/entry_count",len);
	
	gnome_config_sync ();
#endif
	g_free(ab_entry);
	return(0);
}

/* create the sip user agent and run it*/
int linphone_main_start(LinphoneMain *lpm)
{
	ua_params_t params;
	int err,len;
	OsipUA *ua;
	
	/* set debug output*/
  	//lpm->sip_conf.sip_dbg=fopen("/tmp/lpsip","w");
	memset(&params,0,sizeof(ua_params_t));
	params.contact="sip:toto@bidule";    /* contact is set later*/
	params.ua_ip4addr=lpm->net_conf.local_addr;
	params.ua_port=lpm->sip_conf.sip_port;
	params.max_calls=1;
	params.dbg=NULL;
	ua=osip_ua_new(&params);
	lpm->ua=ua;
  	if (ua==NULL)
  	{
		printf("Cannot create SIP user agent.\n");
    	return(-1);
	}
	linphone_main_set_contact(lpm);    /* set contact according to current config*/
	
  	err=osip_ua_signal_connect(ua,"INVITE_ACCEPTED",invite_accepted_cb);
  	err=osip_ua_signal_connect(ua,"BYE",bye_cb);
  	err=osip_ua_signal_connect(ua,"FAILLURE",faillure_cb);
  	err=osip_ua_signal_connect(ua,"INVITE",invite_cb);
  	err=osip_ua_signal_connect(ua,"INFORMATIVE",informative_cb);
  	
	
	linphone_main_apply_codec_change(lpm);
	osip_ua_set_rtpport(lpm->ua,STREAM_AUDIO,lpm->rtp_conf.audio_rtp_port);
	linphone_main_set_registration_timer(lpm);
	if (lpm->sip_conf.reg_conf.use_registrar)
	{
		if (lpm->sip_conf.reg_conf.registrar!=NULL)
		{
			len=strlen(lpm->sip_conf.reg_conf.registrar);
			if (len!=0)
			{
				//if (lpm->sip_conf.reg_conf.as_redirect) osip_ua_set_registrar(lpm->ua,tmp2,lpm->sip_conf.reg_conf.passwd);
				if (lpm->sip_conf.reg_conf.as_proxy) osip_ua_set_outbound_proxy(lpm->ua,lpm->sip_conf.reg_conf.registrar,lpm->sip_conf.reg_conf.passwd);
			}	
			do_registration(NULL);
		}
	}
	return(0);
}

int linphone_main_set_contact(LinphoneMain *lpm)
{
	char *contact;
	char *user,*domain;
	int err;
	
	if (lpm->ua==NULL) return(-EFAULT);
	
	
 	contact=malloc(strlen(lpm->sip_conf.username)+strlen(lpm->sip_conf.hostname)+12);
 	user=lpm->sip_conf.username;
 	domain=lpm->sip_conf.hostname;

	if (lpm->sip_conf.sip_port==5060)
		sprintf(contact,"<sip:%s@%s>",user,domain);	
	else sprintf(contact,"<sip:%s@%s:%i>",user,domain,lpm->sip_conf.sip_port);
	err=osip_ua_set_contact(lpm->ua,contact);
	printf("Adding %s to the list of alias.\n",contact);
	osip_ua_add_alias(lpm->ua,contact);
	if (err!=0)
	{
		free(contact);
		return err;
	}
	if (lpm->sip_conf.contact!=NULL) free(lpm->sip_conf.contact);
	lpm->sip_conf.contact=contact;
	return 0;
}
		

void linphone_main_set_registration_timer(LinphoneMain *lpm)
{
	/* set timer for redirect server registration*/
	if (lpm->sip_conf.reg_conf.use_registrar)
	{
		if (lpm->sip_conf.reg_conf.expires!=0)
		{
			if ( lpm->gui_conf.timer_id!=0) linphone_main_remove_registration_timer(lpm);
			lpm->gui_conf.timer_id=gtk_timeout_add( lpm->sip_conf.reg_conf.expires*1000,do_registration,NULL);
			printf("Setting timer for %i seconds.\n", lpm->sip_conf.reg_conf.expires);
		}
		else printf("Timer not set because of null timeval.\n");
	}
}

void linphone_main_remove_registration_timer(LinphoneMain *lpm)
{
	/* set timer for redirect server registration*/
	if (lpm->gui_conf.timer_id!=0)
	{
		gtk_timeout_remove(lpm->gui_conf.timer_id);
	}
}

void linphone_main_apply_codec_change(LinphoneMain *lpm)
{
	codec_info_t *codec;
	GList *elem;
	
	linphone_main_codecs_check(lpm);
	if (lpm->ua==NULL) return;
	osip_ua_clean_profiles(lpm->ua,STREAM_AUDIO);
	osip_ua_clean_profiles(lpm->ua,STREAM_VIDEO);
	elem=lpm->codecs_conf.audio_codecs;
	while(elem!=NULL)
	{
		codec=(codec_info_t*)(elem->data);
		if ((codec->is_selected)&&(codec->is_usable))
		{
			printf("Added profile %i\n", codec->profile);
			osip_ua_add_profile(lpm->ua,STREAM_AUDIO,codec->profile);
		}
		elem=g_list_next(elem);
	}
	elem=lpm->codecs_conf.video_codecs;
	while(elem!=NULL)
	{
		codec=(codec_info_t*)(elem->data);
		osip_ua_add_profile(lpm->ua,STREAM_VIDEO,codec->profile);
		elem=g_list_next(elem);
	}
	
}

/* test if codecs are usable compared to bandwidth. */
codec_info_t *linphone_main_codecs_check(LinphoneMain *lpm)
{
	GList *elem;
	codec_info_t *codinfo;
	
	elem=lpm->codecs_conf.codecs;
	while(elem!=NULL)
	{
		codinfo=(codec_info_t*)elem->data;
		codinfo->is_usable=codec_is_usable(codinfo,lpm->net_conf.bandwidth);
		elem=g_list_next(elem);
	}
	return(NULL);
}
	
/* stop all processes properly*/
void linphone_main_stop(LinphoneMain *lpm)
{
	osip_ua_destroy(lpm->ua);
	linphone_main_remove_registration_timer(lpm);
	lpm->ua=NULL;
}


void linphone_main_shutdown(LinphoneMain *lpm, int (*cb)(OsipCallLeg*,sip_t *,void*) )
{
	OsipCallLeg *call;
	/* Before leaving linphone should unregisters itself from the registrar (if used)
	unregistration_ok_cb is called when the unregistration is successfull */
	if (lpm->sip_conf.reg_conf.use_registrar)
	{
		osip_ua_signal_connect(lpm->ua,"INFORMATIVE",cb);
		if (strcmp(lpm->net_conf.if_name,"lo")!=0)
		{
			RegistrationCtxt *ctx;
			ctx=registration_ctxt_new();
			registration_ctxt_set_registrar(ctx,lpm->sip_conf.reg_conf.registrar);
			registration_ctxt_set_password(ctx,lpm->sip_conf.reg_conf.passwd);
			registration_ctxt_set_address_of_record(ctx,lpm->sip_conf.reg_conf.addr_of_rec);		
			call=osip_call_leg_new(lpm->ua);
			osip_call_leg_unregister(call,ctx);
			osip_call_leg_release(call);
		}
	}
}
	


void linphone_main_register_codec(LinphoneMain *lpm,codec_info_t *codec)
{
	lpm->codecs_conf.codecs=g_list_append(lpm->codecs_conf.codecs,codec);
	/* if this codec is not in the preferred lists, we must add it at the end*/
	if (codec->type==CODEC_AUDIO)
	{
		if (g_list_find(lpm->codecs_conf.audio_codecs,codec)==NULL)
			lpm->codecs_conf.audio_codecs=g_list_append(lpm->codecs_conf.audio_codecs,codec);
	}
	else if (codec->type==CODEC_VIDEO)
	{
		if (g_list_find(lpm->codecs_conf.video_codecs,codec)==NULL)
		lpm->codecs_conf.video_codecs=g_list_append(lpm->codecs_conf.video_codecs,codec);
	}
}

static int codec_cmp(gconstpointer codec, gconstpointer profile)
{
	codec_info_t *p= (codec_info_t*) codec;
	return (p->profile!=GPOINTER_TO_INT(profile)) ;
}

codec_info_t * linphone_main_get_codec(LinphoneMain *lpm, gint profile)
{
	GList *l;
	l=g_list_find_custom(lpm->codecs_conf.codecs,GINT_TO_POINTER(profile),codec_cmp);
	if (l==NULL)
	{
		printf("Cannot find codec with profile %i\n",profile);
		return(NULL);
	}
	return((codec_info_t*)(l->data));
}

/* final exit: stops all threads for gracefull shutdown*/
void linphone_main_exit(LinphoneMain *lpm)
{
	osipua_exit();
}

/* this is the retrieve_if function for FreeBSD. It uses getifaddrs system call instead of ioctl in order to get interface names and address.*/
/* Thanks 	to Wolfgang for this .*/
#ifdef HAVE_GETIFADDRS	
int retrieve_if(LinphoneMain *lpm)
{
    struct ifaddrs *ifp;
    int i;

    if(getifaddrs(&ifp) < 0) {
        return -1;
    }
    for(i = 0;
        ifp != NULL;
        ifp = ifp->ifa_next) {

        if(ifp->ifa_addr->sa_family != AF_INET) continue;

printf("ifc %s %s\n",
ifp->ifa_name,
inet_ntoa(
    ((struct sockaddr_in *)ifp->ifa_addr)->sin_addr
)
);
        strncpy(lpm->net_conf.if_names[i],
                ifp->ifa_name,
                sizeof lpm->net_conf.if_names[i]);
        strncpy(lpm->net_conf.if_addr[i],
                inet_ntoa(
                    ((struct sockaddr_in *)ifp->ifa_addr)->sin_addr
                ),
                sizeof lpm->net_conf.if_addr[i]);
        ++i;
    }

    freeifaddrs(ifp);

    return i;
}
#else

int retrieve_if(LinphoneMain *lpm)
{
	int sock,err,if_count,i,lo=0;
	struct ifconf netconf;
	char buffer[32*MAX_IF];
	
	netconf.ifc_len=32*MAX_IF;
	netconf.ifc_buf=buffer;
	sock=socket(PF_INET, SOCK_DGRAM,0);
	err=ioctl(sock,SIOCGIFCONF,&netconf);
	if (err<0) printf("Error in ioctl: %i.\n",errno);
	close(sock);
	
	if_count=netconf.ifc_len/32;
	//printf("Found %i interfaces.\n",if_count);
	for (i=0;i<if_count;i++)
	{
		if (strcmp(netconf.ifc_req[i].ifr_name,"lo")==0) lo=1;

		strncpy(lpm->net_conf.if_names[i],netconf.ifc_req[i].ifr_name,20);
		strncpy(lpm->net_conf.if_addr[i],inet_ntoa(((struct sockaddr_in*)(&netconf.ifc_req[i].ifr_addr))->sin_addr),20);
		//printf("Found interface %s\n",lpm->net_conf.if_names[i]);
	}
	if (lo==0) return(-1);
	return(if_count);
}

#endif
static char lock_name[80];
static char lock_set=0;

/* put a lock file in /tmp. this is called when linphone runs as a daemon*/
int set_lock_file()
{
	FILE *lockfile;
	
	snprintf(lock_name,80,"/tmp/linphone.%i",getuid());
	lockfile=fopen(lock_name,"w");
	if (lockfile==NULL)
	{
		printf("Failed to create lock file.\n");
		return(-1);
	}
	fprintf(lockfile,"%i",getpid());
	fclose(lockfile);
	lock_set=1;
	return(0);
}

/* looks if there is a lock file. If presents return its content (the pid of the already running linphone), if not found, returns -1*/
int get_lock_file()
{
	
	int pid;
	FILE *lockfile;
	
	snprintf(lock_name,80,"/tmp/linphone.%i",getuid());
	lockfile=fopen(lock_name,"r");
	if (lockfile==NULL)
		return(-1);
	fscanf(lockfile,"%i",&pid);
	fclose(lockfile);
	return(pid);
}

/* remove the lock file if it was set*/
int remove_lock_file()
{
	int err=0;
	if (lock_set)
	{
		err=unlink(lock_name);
		lock_set=0;
	}
	return(err);
}

int kill_sound_daemons()
{
	char command[100];
	int err;
	FILE *out;
	pid_t pid;
	
	/* try to kill all artsd*/
	snprintf(command,100,"ps -u %s |grep artsd",getenv("LOGNAME"));
	out=popen(command,"r");
	if (out!=NULL)
	{
		do{
		err=fscanf(out,"%i",&pid);
		if (err==1) kill(pid,SIGINT);
		}while(err==1);
		pclose(out);
	}
	/* do the same with esd*/
	snprintf(command,100,"ps -u %s |grep esd",getenv("LOGNAME"));
	out=popen(command,"r");
	if (out!=NULL)
	{
		do{
		err=fscanf(out,"%i",&pid);
		if (err==1) kill(pid,SIGINT);
		}while(err==1);
		pclose(out);
	}
	return(0);
}

