/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

callbacks.c -- gtk callbacks, and osipua callbacks.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <applet-widget.h>
#include <string.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "state-machine.h"
#include "sip_handlers.h"
#include "../audio/audio.h"
#include "../audio/ring.h"
#include "../osipua/osipua.h"
#include "../osipua/utils.h"
#include "../gsmlib/gsm_wrapper.h"
#include "../lpc10-1.5/lpc10_wrapper.h"
#include "io.h"
#include "LinphoneMain.h"
#include "gui_utils.h"
#include  "propertybox.h"
#include "../pixmaps/linphone2.xpm"

#ifdef HAVE_PANEL_PIXEL_SIZE

#endif



/* widgets*/
extern GtkWidget *applet;
extern GtkWidget *main_window;
extern GtkWidget *appbar1;
extern GtkWidget *addressentry;
LPPropertyBox *prop;

/* LinphoneMain object*/
extern LinphoneMain linpmain;


/* messages strings */
extern gchar *ready;
extern gchar *end;
extern gchar *contacting;
extern gchar *connected;
extern gchar *cancel;
extern gchar *contacted;

#ifndef G_THREADS_ENABLED
#error "pas de treads dans la glib!"
#endif

int state=LP_INIT;  /* this integer represents the state machine: it MUST only be changed using a mutex or something
like*/
G_LOCK_DEFINE_STATIC(state);
char *rem_url=NULL;
GList *if_list;
static unsigned char selected_row=0;
RegistrationCtxt *regctx=NULL;


void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (main_window!=NULL)
	{
		gnome_entry_save_history (GNOME_ENTRY (addressentry));
	}
	else printf("Error: main_window=NULL.\n");
	gui_shutdown(&linpmain);
	
}

void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *about2;
  about2 = create_about2 ();
  gtk_widget_show (about2);
}


void
on_greenbutton_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *appbar1=(GtkWidget*)user_data;
	gchar *url, *barmsg;
	list_t *streams;
	MediaDesc *stream;
	codec_info_t *cod;
	int err,err1;

	G_LOCK(state);
	printf("state=%i\n",state);
	
	if (state==LP_INVITED)
	{
		/*stop ringing */
		stop_ring();
		/* sends a 200 OK */
		streams=osip_call_leg_accept_invite(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),connected);
		gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(addressentry))),rem_url);
		state=LP_INVITED_CONFIRMED;
		
		if (streams==NULL)
		{
			printf("No streams !!!");
			exit(1);
		}
		/* prepare rtp session using sdp paramters*/
		stream=(MediaDesc*)list_get(streams,0);
		err=rtp_prepare(linpmain.rtp_conf.audio_rtp_port,stream->rem_ip4addr,stream->rem_port,profile_1(stream),
									linpmain.rtp_conf.jitt_comp,0);
		if (err==0)
		{
			/* chexk if we can open dsp*/
			try_open_dsp();
			cod=linphone_main_get_codec(&linpmain,profile_1(stream));
			/* start the io loop*/
			err1=start_audio_io(cod);
			/* if we fail, it is because of an unopenable audio device*/
			if (err1==-1)
			{
				printf("Cannot start io!!\n");
			}
		}else printf("Failed to create RTP session.\n");
	} 	
	else if (state==LP_INIT)
	{
		char *correct_url;
		/* sends an invite */
		url=gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(addressentry))));
		if (url==NULL)
		{
			printf("gtk_entry_get_text() returned NULL. This is not normal if some text is present.\n");
			gnome_app_warning(GNOME_APP(main_window),_("Bad formuled sip address."));
			goto end;
		}
		correct_url=check_sipurl(url);
		if (correct_url==NULL)
		{
			gnome_app_warning(GNOME_APP(main_window),_("Bad formuled sip address."));
			goto end;
		}
		if (correct_url!=url) gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(addressentry))),correct_url);
		url=correct_url;
	
		barmsg=g_malloc(strlen(url)+strlen(contacting)+2);
		sprintf(barmsg,"%s %s",contacting,url);
		gnome_appbar_push(GNOME_APPBAR(appbar1),barmsg);
		
		linpmain.call=osip_call_leg_new(linpmain.ua);
		if (linpmain.call==NULL) printf("error in createcall.\n");
		
		err=osip_call_leg_invite(linpmain.call,url);
		if (err!=0)
		{
			printf("error in invite: %s (%i)\n",strerror(err),err);
			//gnome_app_warning(GNOME_APP(main_window),strerror(err));
			gnome_appbar_push(GNOME_APPBAR(appbar1),ready);
			state=LP_INIT;
		}	
		else state=LP_INVITE;
	}
end:
	printf("invite state=%i\n",state);
	G_UNLOCK(state);
}


void
on_redbutton_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *appbar1=(GtkWidget*)user_data;

    G_LOCK(state);
    printf("state end=%i\n",state);
    if (state==LP_INVITED )
    {
       /*stop ringing*/
		stop_ring();
		/* decline the invitation */
		osip_call_leg_bye(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),end);
		state=LP_INIT;
    }
	else if ((state==LP_INVITED_CONFIRMED) || (state==LP_INVITE_CONFIRMED))
	{
    		 /* sends a bye, the call is terminated */
		stop_audio_io();
		osip_call_leg_bye(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),end);
		state=LP_INIT;
	}
	else if (state==LP_INVITE)
	{
     /*sends a cancel or a bye ???*/
       stop_ring();
		osip_call_leg_bye(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),cancel);
		state=LP_INIT;
	}
	linpmain.call=NULL;
	G_UNLOCK(state);
	return;
}

int invite_accepted_cb(OsipCallLeg *call,sip_t *msg,void*p2)
{
	//sip_t *sipmsg= (sip_t *)p2;
	list_t *streams;
	MediaDesc *md;
	int err;
	codec_info_t *cod;

	G_LOCK(state);
	gdk_threads_enter();
	main_window_show();
	gnome_appbar_push(GNOME_APPBAR(appbar1),connected);
	gdk_threads_leave();
	state=LP_INVITE_CONFIRMED;
	printf("ok state=%i\n",state);
	stop_ring();
	G_UNLOCK(state);
	
	streams=osip_call_leg_get_streams(call);
	if (streams==NULL)
	{
		printf("No streams in a remotely acceted call !!!\n");
		exit(1);
	}
	md=(MediaDesc*)list_get(streams,0);  /* get audio stream descriptor*/
	printf("rem_port=%i\n", md->rem_port);
	err=rtp_prepare(linpmain.rtp_conf.audio_rtp_port,md->rem_ip4addr,md->rem_port,profile_1(md),linpmain.rtp_conf.jitt_comp,0);
	if (err==0)
	{
		gdk_threads_enter();
		try_open_dsp();
		gdk_threads_leave();
		cod=linphone_main_get_codec(&linpmain,profile_1(md));
		err=start_audio_io(cod);
		if (err==-1)
		{
			printf("Failed to start io [2]\n");
		}
	}else printf("Failed to prepare RTP session.\n");
	return 0;
}

int bye_cb(OsipCallLeg *call,sip_t *msg,void*p2)
{
	stop_ring();  /*stop ringing if we were ringing*/
	stop_audio_io();     /* io of rtp data if we were in communication*/
	G_LOCK(state);
	gdk_threads_enter();
	main_window_show();
	gnome_appbar_push(GNOME_APPBAR(appbar1),end);
	gdk_threads_leave();
	state=LP_INIT;
	linpmain.call=NULL;
	printf("bye handler: state=%i\n",state);
	G_UNLOCK(state);
	return 0;
}

/* osipua call us in this callback function when an invite arrives*/
int invite_cb(OsipCallLeg *call,sip_t *sipmsg,void *p2)
{
	from_t *from=sipmsg->from;
	char *barmesg;
	int err,addrlen;

	G_LOCK(state);
	linpmain.call=call;
	if (rem_url!=NULL) g_free(rem_url);
	addrlen=9+strlen(from->url->username)+strlen(from->url->host);
	if (from->displayname!=NULL) addrlen+=strlen( from->displayname);
	rem_url=g_malloc(addrlen);
   if (from->displayname!=NULL)
   		sprintf(rem_url,"%s <sip:%s@%s>", from->displayname, from->url->username,  from->url->host);
   else sprintf(rem_url,"<sip:%s@%s>", from->url->username,  from->url->host);
	barmesg=g_malloc(strlen(rem_url)+strlen(contacted)+4);
	sprintf(barmesg,"%s %s.",rem_url,contacted);
	gdk_threads_enter();
	main_window_show();
	gnome_appbar_push(GNOME_APPBAR(appbar1),barmesg);
	gdk_threads_leave();
	state=LP_INVITED;
	/* play the ring */
	/* see if dsp is openable*/
	gdk_threads_enter();
	try_open_dsp();
	gdk_threads_leave();
	err=start_ring();
	G_UNLOCK(state);
	if (err==-1)
	{
		printf("Failed to start ring.\n");
	}
	printf("invite handler done.\n");
	g_free(barmesg);
	return 0;
}

int informative_cb( OsipCallLeg *call,sip_t *msg,void *p2)
{
	int err=((int*)p2)[0];
	contact_t *contact;
	char *ctt;
			
	gdk_threads_enter();
	switch(err)
	{
		case 100:
			break;
		case 180:
			start_ringback();
			break;
		case 200:   /* for the moment only REGISTER request followed by 200 OK are said to the user*/
			 if (regctx->expires) gnome_appbar_push(GNOME_APPBAR(appbar1),_("Registration successfull."));
			 else  gnome_appbar_push(GNOME_APPBAR(appbar1),_("Unregistration successfull."));
		break;
		case 380:
			printf ("Entering 380\n");
			msg_getcontact(msg,0,&contact);
			if (contact!=NULL)
			{			
				contact_2char(contact,&ctt);
			}else ctt=sgetcopy(_("(No contact information !)"));
			alt_ressource_display(ctt);
			sfree(ctt);
			//gtk_main_iteration_do(FALSE);
		break;			
		default:
			printf("unsupported informative message.\n");
	}
	gdk_threads_leave();
	return 0;
}

int faillure_cb(OsipCallLeg *call,sip_t *msg,void *p2)
{
	int err=((int*)p2)[0];
	char *reason;
	header_t *retry;
	char *umsg;
	char *msg486=_("User is busy.");
	char *msg480=_("User is temporarily unavaillable.");
	char *retrymsg=_("%s. Retry after %i minute(s).");
	char *msg600=_("User does not want to be disturbed.");
	char* tmpmsg=msg486;
	
	G_LOCK(state);
	gdk_threads_enter();
	main_window_show();
	switch(err)
	{
		case 404:
			gnome_appbar_push(GNOME_APPBAR(appbar1),_("User cannot be found at given address."));
			state=LP_INIT;
		break;
	  case 480:
	  	tmpmsg=msg480;
		case 486:
			msg_header_getbyname(msg,"retry-after",0,&retry);
			if (retry!=NULL)
			{
				umsg=g_malloc(strlen(tmpmsg)+strlen(retrymsg)+13);
				sprintf(umsg,retrymsg,tmpmsg,atoi(retry->hvalue)/60);
				gnome_app_message(GNOME_APP(main_window),umsg);
				g_free(umsg);
			}		
			gnome_appbar_push(GNOME_APPBAR(appbar1),tmpmsg);
			state=LP_INIT;
		break;
		
		case 600:
			gnome_appbar_push(GNOME_APPBAR(appbar1),msg600);
		  state=LP_INIT;
		break;
		case -110:  /* time out, call leg is lost */
			state=LP_INIT;
			gnome_appbar_push(GNOME_APPBAR(appbar1),_("Timeout."));
		break;
		case -111:
			gnome_appbar_push(GNOME_APPBAR(appbar1),_("Remote host was found but refused connection."));
			state=LP_INIT;
		break;
		
		default:
			if (err>0)
			{
				reason=msg_getreason(err);
				gnome_appbar_push(GNOME_APPBAR(appbar1),reason);
				free(reason);
			}
			else printf("faillure_cb: %s\n",strerror(-err));
			state=LP_INIT;
	}
	linpmain.call=NULL;
	gdk_threads_leave();
	G_UNLOCK(state);
	printf("error handler done.\n");
	return 0;
}

void on_propertybox1_apply (GtkButton       *button,
                                        gint pagenum)
{	
	switch(pagenum)
	{
	case 0:
		net_section_apply(&prop->net,&linpmain);
		break;
	case 1:
		rtp_section_apply(&prop->rtp,&linpmain);
		break;
	case 2:
		sip_section_apply(&prop->sip,&linpmain);
	break;
	case 4:
		sound_section_apply(&prop->sound,&linpmain);
  		break;
  	case 3:
  	 	codec_section_apply(&prop->codec,&linpmain);
  		break;
  	}
}

gint
on_prop1_close                         (GnomeDialog     *gnomedialog,
                                        gpointer         user_data)
{	

	gnome_appbar_clear_stack( GNOME_APPBAR(appbar1));
	return(FALSE);
}

void
on_parametres1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	prop=lp_property_box_new();
	lp_property_box_fill(prop,&linpmain);
  	
	gtk_widget_show(prop->prop);
}


void
on_killesd_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	
	kill_sound_daemons();
}

void kill_esd_answer(gint reply, gpointer data)
{
	if (!reply)
	{
		printf("Killing sound daemons.\n");
		kill_sound_daemons();
	}
}

gboolean
on_hscale1_button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
	return(TRUE);
}

void
on_list1_select_child                  (GtkList         *list,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{
	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_codec_changed                       (GtkEditable     *editable,
                                        gpointer         user_data)
{
	/*
	gchar *sel_cod;
	sel_cod=gtk_editable_get_chars(editable,0,-1);
	if (strcmp(sel_cod,"GSM (13 kbit/s)")==0) temp_config.profile=3;
	else if (strcmp(sel_cod,"LPC10-15 (2.4 kbit/s)")==0) temp_config.profile=4;
	printf("Profile selection changed to %s : %i.\n",sel_cod,temp_config.profile);
	g_free(sel_cod);
	*/
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_sounddriver_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	/*
	gchar *sel_sound;
	
	sel_sound=gtk_editable_get_chars(editable,0,-1);
	if (strcmp(sel_sound,"OSS")==0) temp_config.sound_driver=0;
	else if (strcmp(sel_sound,"ALSA")==0) temp_config.sound_driver=1;
	printf("Sound driver selection changed to %s : %i.\n",sel_sound,temp_config.sound_driver);
	g_free(sel_sound);
	*/
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_user_manual1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	static GnomeHelpMenuEntry help_ref = { "linphone", "manual/index.html" };
	gnome_help_display(NULL,&help_ref);
}


gboolean
on_play_vol_button_release_event       (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	int vol;
	vol=(gtk_range_get_adjustment(GTK_RANGE(widget)))->value;
	set_out_level(vol);
	linpmain.sound_conf.play_lev=vol;
  return FALSE;
}


gboolean
on_rec_vol_button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	int vol;
	vol=(gtk_range_get_adjustment(GTK_RANGE(widget)))->value;
	set_in_level(vol);
	linpmain.sound_conf.rec_lev=vol;
	return FALSE;
}


void
on_prop1_help                          (GnomePropertyBox *gnomepropertybox,
                                        gint             arg1,
                                        gpointer         user_data)
{
	static GnomeHelpMenuEntry ref = { "linphone", "manual/params.html" };
	gnome_help_display(NULL,&ref);
}


void
on_audioport_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
	/*
	gchar *rtpport;
	rtpport=gtk_editable_get_chars(editable,0,-1);
	temp_config.rtp_port=atoi(rtpport);
	*/
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
	return;
}


void
on_sipport_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{

	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
	return;
}


void
on_source_changed                      (GtkEditable     *editable,
                                        gpointer         user_data)
{
	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_fermer1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if ((linpmain.gui_conf.is_applet) ||( linpmain.gui_conf.is_daemon) )
	{
		gtk_widget_destroy(main_window);
		//linphone_config.show_flag=0;
	}
}


void
on_autokill_button1_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	/*
	temp_config.autokill=gtk_toggle_button_get_active(togglebutton);
	*/
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_app1_destroy                        (GtkObject       *object,
                                        gpointer         user_data)
{
  	if ((linpmain.gui_conf.is_applet) ||( linpmain.gui_conf.is_daemon) )
  	{
  		printf("Main window destroyed.\n");
  		main_window=NULL;
  	}
	else
	{
	
    	gnome_entry_save_history (GNOME_ENTRY (addressentry));
    	gui_shutdown(&linpmain);
    	
	}
}

/* this is the callback when the user clicks on the applet*/
gboolean on_applet_clicked(GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer ud)
{
	//GtkWidget *main_window=  (GtkWidget *)ud;
	if (main_window==NULL)
	{
		main_window_show();
	}
	else
	{
		gtk_widget_destroy(main_window);
		main_window=NULL;
	}
	return(TRUE);
}


/*this is when the panel size changes*/
void applet_change_pixel_size(GtkWidget *w, int size, gpointer data)
{
	GtkWidget *pixmap,*button;
	
	pixmap=gtk_object_get_data(GTK_OBJECT(applet),"applet_pixmap");	
	button=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(applet),"applet_button");	
	if (button==NULL)
	{
		printf("Cannot find applet button\n");
		return;
	}
	if (pixmap!=NULL) gtk_widget_destroy(pixmap);
	pixmap = gnome_pixmap_new_from_xpm_d_at_size(linphone2_xpm,
							 size-4, size-4);
	gtk_object_set_data(GTK_OBJECT(applet),"applet_pixmap",pixmap);
  	gtk_widget_show(pixmap);
  	gtk_container_add(GTK_CONTAINER(button), pixmap);	
}

void
on_adresse_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *addbook;
	GtkWidget *item;
	GList *item_list=NULL,*elem=linpmain.gui_conf.address_list;
	GtkWidget *list;
	
	addbook=create_address_book();
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	while(elem!=NULL)
	{
		item=gtk_list_item_new_with_label ((gchar*)(elem->data));
		item_list=g_list_append(item_list,item);
		gtk_widget_show(item);
		elem=g_list_next(elem);
	}
	gtk_list_append_items(GTK_LIST(list), item_list);
	gtk_widget_show(addbook);
}

void on_address_book_show(GtkWidget *widget,gpointer user_data)
{
	
}



void on_address_list_select_child(GtkList         *list,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void on_add_address_clicked(GtkButton *button,gpointer user_data)
{
	gchar *address,*full_address,*displayname;
	GtkWidget *list,*entry;
	GtkWidget *addbook=(GtkWidget*)user_data;
	from_t *sipaddr;
	int error,namelen;
	GList *item_list=NULL;
	GtkWidget *item;
	
	entry=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_entry");
	address=  gtk_editable_get_chars (GTK_EDITABLE(entry),0,-1);
	entry=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"name_entry");
	displayname=  gtk_editable_get_chars (GTK_EDITABLE(entry),0,-1);
	from_init(&sipaddr);
	error=from_parse(sipaddr,address);
	if (error!=0)
	{
		printf("Sip address is incorrect.\n");
		free(sipaddr);
		g_free(address);
		g_free(displayname);
		return;
	}
	namelen=strlen(displayname);
	full_address=g_malloc(strlen(address)+namelen+6);
	if (namelen!=0)
		sprintf(full_address,"%s <%s>",displayname,address);
	else sprintf(full_address,"%s",address);
	linpmain.gui_conf.address_list=g_list_append( linpmain.gui_conf.address_list, (gpointer)full_address);
	item=gtk_list_item_new_with_label (full_address);
	item_list=g_list_append(item_list,item);
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	gtk_list_append_items(GTK_LIST(list), item_list);
	gtk_widget_show(item);
	free(sipaddr);
	g_free(address);
	g_free(displayname);
}

void on_remove_address_clicked(GtkButton *button,gpointer user_data)
{
	GtkWidget *addbook=(GtkWidget*)user_data;
	GList *elem;
	GtkWidget *list;
	gint selected_address;
	
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	g_return_if_fail(list!=NULL);
	g_return_if_fail(GTK_LIST(list)->selection!=NULL);
	selected_address=g_list_index (GTK_LIST(list)->children,GTK_LIST(list)->selection->data);
	gtk_list_clear_items(GTK_LIST(list), selected_address,selected_address+1);
	elem= g_list_nth(linpmain.gui_conf.address_list,selected_address);
	if (elem==NULL)
	{
		printf("Error: cannot retrieve address\n");
		return;
	}
	g_free(elem->data);
	linpmain.gui_conf.address_list=g_list_remove_link( linpmain.gui_conf.address_list,elem);
	g_list_free_1(elem);
	
}

void on_select_address_clicked(GtkButton *button,gpointer user_data)
{
	GtkWidget *addbook=(GtkWidget*)user_data;
	gchar *address;
	GtkWidget *list;
	gint selected_address;
	
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	g_return_if_fail(list!=NULL);
	g_return_if_fail(GTK_LIST(list)->selection!=NULL);
	selected_address=g_list_index (GTK_LIST(list)->children,GTK_LIST(list)->selection->data);
 	address=(gchar*)g_list_nth_data(linpmain.gui_conf.address_list, selected_address);
	
	gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(addressentry))),address);
	gtk_widget_destroy(addbook);
}




void
on_aucodec_up_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	if (selected_row>0)
	{
		gtk_clist_row_move(clist,selected_row,selected_row-1);
		selected_row--;
		gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
	}
}


void
on_aucodec_down_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	if (selected_row < clist->rows-1)
	{
		gtk_clist_row_move(clist,selected_row,selected_row+1);
		selected_row++;
		gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
	}
}


void
on_aucodec_enable_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	gtk_clist_set_text(clist,selected_row,2,_("Enabled"));
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_aucodec_disable_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	gtk_clist_set_text(clist,selected_row,2,_("Disabled"));
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_au_codec_list_select_row            (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	selected_row=row;
}


void
on_au_codec_list_row_move              (GtkCList        *clist,
                                        gint             arg1,
                                        gint             arg2,
                                        gpointer         user_data)
{
	//gtk_clist_row_move(clist,arg1,arg2);
	//gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
	//printf("moved!");
}




void
on_registrar_checked_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean state;
	
	state=gtk_toggle_button_get_active(togglebutton);
	sip_section_enable_registrar(&prop->sip,&linpmain, state);	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_user_name_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkWidget *address_of_record=(GtkWidget *) user_data;
	gchar *tmp;
	/* address of record domain should usually be also the name of the registrar when it is used*/
	tmp=gtk_editable_get_chars(editable,0,-1);
	gtk_entry_set_text(GTK_ENTRY(address_of_record),tmp);
	g_free(tmp);
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_domain_name_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_registrar_addr_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkWidget *address_of_record=(GtkWidget *) user_data;
	GtkWidget *username=prop->sip.username;
	url_t *url1,*url2;
	gchar *hostname,*addrrec,*tmp;
	gint err=0;
	/* address of record domain host part should usually be also the name of the registrar when it is used*/
	tmp=gtk_editable_get_chars(editable,0,-1);
	if ((tmp!=NULL) && (strlen(tmp)!=0))
	{
		url_init(&url1);
		err=url_parse(url1,tmp);
		g_free(tmp);
		if (err<0) {
			sfree(url1);
			return;
		}
	}
	else return;
	hostname=url1->host;
	addrrec=gtk_editable_get_chars(GTK_EDITABLE(address_of_record),0,-1);
	url_init(&url2);
	if ((addrrec!=NULL) && (strlen(addrrec)!=0)){
		err=url_parse(url2,addrrec);
		//printf("address of record parsed with err=%i\n",err);
		if (err<0) {url2->host=NULL;url2->username=NULL;};
	}
	if (url2->host!=NULL) sfree(url2->host);
	if ((hostname!=NULL) && (strlen(hostname)!=0)) url2->host=sgetcopy(hostname);
	else url2->host=gtk_editable_get_chars(GTK_EDITABLE(prop->sip.hostname),0,-1);
	//printf ("url2->host=%s\n",url2->host);
	if (url2->username==NULL) url2->username=gtk_editable_get_chars(GTK_EDITABLE(username),0,-1);
	url_setscheme(url2,sgetcopy("sip"));
	url_2char(url2,&tmp) ;
	if (tmp!=NULL){
		//printf("New address of record is %s\n",tmp);
		gtk_entry_set_text(GTK_ENTRY(address_of_record),tmp);
		sfree(tmp);
	}
	url_free(url1);
	sfree(url1);
    url_free(url2);
    sfree(url2);
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}

/* send a REGISTER request to the default registrar*/
gint do_registration(gpointer data)
{
	/* prevent from emmitting registration for address 127.0.0.1...*/
	if (strcmp(linpmain.net_conf.local_addr,"127.0.0.1")!=0)
	{
		OsipCallLeg *call=osip_call_leg_new(linpmain.ua);
		if (regctx!=NULL) registration_ctxt_destroy(regctx);
		regctx=registration_ctxt_new();		
		registration_ctxt_set_registrar(regctx,linpmain.sip_conf.reg_conf.registrar);
		registration_ctxt_set_password(regctx, linpmain.sip_conf.reg_conf.passwd);
		registration_ctxt_set_address_of_record(regctx, linpmain.sip_conf.reg_conf.addr_of_rec);
		registration_ctxt_set_expires(regctx,linpmain.sip_conf.reg_conf.expires);
		
		if ((linpmain.sip_conf.reg_conf.as_redirect) && (!linpmain.sip_conf.reg_conf.as_proxy))
			registration_ctxt_set_action(regctx,REGISTER_ACTION_REDIRECT);
		else if ((linpmain.sip_conf.reg_conf.as_proxy) && (!linpmain.sip_conf.reg_conf.as_redirect))
			 registration_ctxt_set_action(regctx,REGISTER_ACTION_PROXY);
		else registration_ctxt_set_action(regctx,REGISTER_ACTION_ANY);
		osip_call_leg_register(call,regctx);
		/* we don't want more from the call-leg:*/
		osip_call_leg_release(call);
		printf("Registration done.\n");
	}
	else printf("Registrations with local loopback address are forbidden in linphone.\n");
	return TRUE;
}
void
on_reg_passwd_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_redirect_button_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_proxy_button_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_obproxy_button_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}



void
on_showmore_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *optioncontrols=(GtkWidget*)user_data;
	gint state;
	state=gtk_toggle_button_get_active(togglebutton);
	if (state) gtk_widget_show(optioncontrols);
	else gtk_widget_hide(optioncontrols);
}



void
on_alt_href_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *url,*sipurl;
	GtkWidget *label;
	label=GTK_BIN(button)->child;
	gtk_label_get(GTK_LABEL(label),&url);
	sipurl=check_sipurl(url);
	if (sipurl==url)
	{ /* it was a sip url, so display it in the entry*/
		gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(addressentry))),url);
	}else sfree(sipurl);
}

void
on_alt_href_realize                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
  GdkCursor *cursor = gdk_cursor_new(GDK_HAND2);
  gdk_window_set_cursor(widget->window, cursor);
  gdk_cursor_destroy(cursor);
}




void
on_address_of_record_changed           (GtkEditable     *editable,
                                        gpointer         user_data)
{
    gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


