/***************************************************************************
                          codec.c - the base class for codecs.
                             -------------------
    begin                : ven mar  30
    copyright            : (C) 2001 by Simon MORLAT
    email                : simon.morlat@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef CODEC_H
#define CODEC_H

#include <glib.h>

#define CODEC_NAMESZ 16

#define CODEC_AUDIO 0
#define CODEC_VIDEO 1


typedef struct audio_info
{
	guint16 rate;     /* sampling frequency*/
}audio_info_t;

typedef struct video_info
{
	guint16 fps;     /* number of frame/seconds */
}video_info_t;



/* this structure is used for registering codecs*/

typedef struct codec_info
{
	gchar name[CODEC_NAMESZ];
	gint fr_size; /* size in char of the uncompressed frame */
	gint	dt_size;	/* size in char of the compressed frame */
	gint bitrate;  /* the minimum bit rate in bits/second */
	union
	{
		audio_info_t a_info;
		video_info_t v_info;
	} spec_info;
	struct _Codec *(*constructor)(void);  /* the constructor to use for this codec*/
	guchar profile;			/* the profile number associated with this codec*/
	guint type:1;   /* CODEC_AUDIO or CODEC_VIDEO */
	guint is_usable:1; /* linphone set this flag to remember if it can use this codec considering the total bandwidth*/
	guint is_selected:1; /* linphone (user) set this flag if he allows this codec to be used*/
}codec_info_t;

/* this is the base class for all codecs. Override the functions pointer to implement new codecs, and
create a constructor named codecname_create() (ex: gsmcodec_create())*/

typedef struct _Codec
{
 /*void (*codec_create)(void);*/
 void (*_getinfo)(struct _Codec *obj,struct codec_info *info);
 void (*_encode)(struct _Codec *obj,char *frame, char *data);
 void (*_decode)(struct _Codec *obj,char *data, char *frame);
 void (*_destroy)(struct _Codec *obj);
} Codec;

/* these are the common method for all codecs implementations */
inline void codec_getinfo(Codec *obj,struct codec_info *info);
inline void codec_encode(Codec *obj,char *frame, char *data);
inline void codec_decode(Codec *obj,char *data, char *frame);
inline void codec_destroy(Codec *obj);

/* return 0 if codec can be used with bandwidth, -1 else*/
int codec_is_usable(codec_info_t *codec,double bandwidth);


#endif

