/*
applet.c - some utils functions that cannot be set in interface.c.

Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <applet-widget.h>
#include <signal.h>
#include <unistd.h>

#include "support.h"
#include "gui_utils.h"
#include "callbacks.h"
#include "../audio/audio.h"
#include "LinphoneMain.h"
#include "presence.h"

extern LinphoneMain linpmain;

extern GtkWidget *appbar1;
extern GtkWidget *main_window;
extern GtkWidget *applet;
extern GtkWidget *addressentry;

void main_window_show()
{
	static char first_time=1;
	GtkWidget *child;
	if (main_window==NULL)
	{
		main_window = create_app1 ();
		addressentry=lookup_widget(main_window,"addressentry");

		appbar1=gtk_object_get_data(GTK_OBJECT(main_window),"appbar1");
		if (first_time)
		{
			first_time=0;
			/* restore entries history */
  			gnome_entry_load_history (GNOME_ENTRY (addressentry));
		}
		child=lookup_widget(main_window,"showmore");
		/* hide the optionnal controls at startup */
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(child),0);
		presence_box_init(&presencebox,main_window);
		gtk_widget_show (main_window);
	}
}

/* creates the applet button*/
GtkWidget *create_applet()
{
  GtkWidget *frame;
  GtkWidget *button;
  GtkWidget *vbox;
  GtkWidget *applet;

  applet = applet_widget_new("linphone_applet");

  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  gtk_widget_show(frame);

  vbox = gtk_vbox_new(FALSE, FALSE);
  gtk_container_add(GTK_CONTAINER(frame), vbox);
  gtk_widget_show(vbox);

  button = gtk_button_new();
	gtk_widget_ref(button);
  GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_DEFAULT);
  GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, TRUE, 0);
	

  	gtk_widget_show(button);
	applet_widget_add (APPLET_WIDGET (applet), frame);
	gtk_object_set_data_full(GTK_OBJECT(applet),"applet_button",button,(GtkDestroyNotify)gtk_widget_unref);
  	gtk_signal_connect(GTK_OBJECT(button), "button_press_event",
                       GTK_SIGNAL_FUNC(on_applet_clicked), NULL);		
  	gtk_signal_connect(GTK_OBJECT(applet), "change_pixel_size",
                       GTK_SIGNAL_FUNC(applet_change_pixel_size), NULL);		 	
	applet_widget_set_tooltip( APPLET_WIDGET (applet),_("linphone"));
	gtk_widget_show(applet);
	return(applet);
};


/* this  just sets level adjustements for startup*/
void set_levels()
{
	GtkWidget *range;
	if (main_window)
	{
		range=gtk_object_get_data(GTK_OBJECT(main_window),"rec_vol");
		gtk_adjustment_set_value (gtk_range_get_adjustment(GTK_RANGE(range)),(gfloat)linpmain.sound_conf.rec_lev);
		range=gtk_object_get_data(GTK_OBJECT(main_window),"play_vol");
		gtk_adjustment_set_value (gtk_range_get_adjustment(GTK_RANGE(range)),(gfloat)linpmain.sound_conf.play_lev);
	}
}

/*try to open dsp. If failed, kill sound daemons if allowed by user, and try agin. If fail again, display the error message. Close it in success*/
int try_open_dsp()
{
	int err1,err2,retval=-1;
	GtkWidget *errmsg;
	
	err1=open_dsp(8000,1,16,0);
	if (err1<0)
	{
		if (linpmain.sound_conf.autokill)
		{
			printf("killing sound daemons.\n");
			kill_sound_daemons();
			sleep(1);   /* wait for sound daemons to be killed...*/
		}
		else
		{
			/* here we sould propose to kill the daemons*/
			/* to do: we required a blocking dialog box here : mutex ??*/
			errmsg=create_messagebox2();
			gtk_widget_show(errmsg);
			return(-1);
		}
		err2=open_dsp(8000,1,16,0);
		if (err2<0)
		{
			errmsg=create_messagebox2();
			gtk_widget_show(errmsg);
			return(-1);
		}
		else retval=0;
	}
	else retval=0;
	
	return(retval);
}
	
		

/*called by SIGUSR1*/
void linphone_wakeup(int signum)
{
	printf("Receiving wake up signal...\n");
	main_window_show();
}


/*called by SIGINT*/
void linphone_exit(int signum)
{
	printf("Exiting gracefully...\n");
	signal(SIGINT,SIG_DFL);
	remove_lock_file();
	applet_widget_gtk_main_quit();
}






static guint timer=0;

static int unregistration_ok_cb(OsipCallLeg *a,sip_t *msg,void *b)
{
	
	if (a!=NULL) gdk_threads_enter(); /* if a!=null we are called by osip */
	if (linpmain.gui_conf.is_applet) applet_widget_remove(APPLET_WIDGET(applet));
	applet_widget_gtk_main_quit();
	
	if (timer!=0)
	{
		gtk_timeout_remove(timer);
		timer=0;
	}
	if (a!=NULL) gdk_threads_leave();
	
	return FALSE;
}

void gui_shutdown(LinphoneMain *lpm)
{
	GtkWidget *wid;
	
	
	if (lpm->sip_conf.reg_conf.use_registrar)
	{
		wid=create_shutdown();
		gtk_widget_show(wid);
		linphone_main_shutdown(lpm,unregistration_ok_cb);
		timer=gtk_timeout_add(2000, (GtkFunction)unregistration_ok_cb,NULL);
	}
	else  unregistration_ok_cb(NULL,NULL,NULL);
}

void alt_ressource_display(gchar *url)
{
	GtkWidget *href;
	GtkWidget *altdisplay;
	GtkLabel *label;
	gchar *pattern;
	
	
	
	altdisplay=create_altressource();
	href=lookup_widget(altdisplay,"alt_href");
	label=GTK_LABEL(GTK_BIN(href)->child);
	gtk_label_set_text(label,url);
	/* pattern used to set underline for string */
  pattern = g_strnfill(strlen(url), '_');
  gtk_label_set_pattern(label,pattern);
  g_free(pattern);
	gtk_widget_show(altdisplay);
}
