/***************************************************************************
                          io.c  - functions that do play,record,recv,send
                          		encode and decode
                             -------------------
    begin                : ven mar  30
    copyright            : (C) 2001 by Simon MORLAT
    email                : simon.morlat@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <pthread.h>
#include <string.h>
#include <stdlib.h>
#include "../audio/audio.h"
#include "../lprtplib/rtp.h"
#include "../gsmlib/gsm_wrapper.h"
#include "../lpc10-1.5/lpc10_wrapper.h"
#include "LinphoneMain.h"
#include "codec.h"


#define RTP_SIZE 160    /* max rtp size allowed for the moment*/


static pthread_t io_audio_thread;
static int iocond=0;
static RtpSession *rtph=NULL;
static void * io_audio_loop(void *);

Codec *audiocodec=NULL;
struct codec_info *codinfo;

int start_audio_io(codec_info_t *cod)
{
	int err;
	
	if (open_dsp==NULL) return(-1); /* check if the driver (oss or alsa) has been choosen*/
	if (rtph==NULL) return(-1); /*check if the rtp handle is valid */
	
	codinfo=cod;
	if (codinfo==NULL)
	{
		printf("Unable to create an unsupported codec:%i.\n",rtph->params.profile);
		return(-1);
	}
	printf("codec: frame_size=%i; data_size=%i;\n",codinfo->fr_size,codinfo->dt_size);
	err=open_dsp(8000,1,16,codinfo->fr_size);
	if (err<0)
	{
		return(-1);
	}
	audiocodec=codinfo->constructor();		/* creates a new codec object*/
	iocond=1;
	err=pthread_create(&io_audio_thread,NULL,&io_audio_loop,rtph);
	if (err!=0) iocond=0;
	return(err);
}

int stop_audio_io()
{
	if (iocond==1)
	{
		iocond=0;
		pthread_join(io_audio_thread,NULL);
    	rtp_destroy_session(rtph);
    	rtph=NULL;
	}
	return(0);
}

static void * io_audio_loop(void *p)
{
	RtpSession* rtph= (RtpSession*)p;
	int err;
	char rtpdata[RTP_SIZE];
	char *aubuf;
	unsigned int pk_count=0;
	
	while(iocond)
	{
		aubuf=get_read_audio_buffer();
		codec_encode(audiocodec,aubuf,rtpdata);
		err=rtp_send(rtph,rtpdata,codinfo->dt_size);
		aubuf=get_write_audio_buffer();
		err=rtp_recv(rtph,rtpdata,codinfo->dt_size);
   	if (err>0) codec_decode(audiocodec,rtpdata,aubuf);
    	else
    	{
    		/*insert a blank*/
    		memset(aubuf,0,codinfo->fr_size);
    		if (err!=0)
    			printf("No rtp packet received. %s %i",rtp_perror(err),err);
    		else
    		{
    			pk_count++;
    			printf("Lost (or late) packets: %i     \r",pk_count);
    		}
    	}
	}
	err=close_dsp();
	codec_destroy(audiocodec);
	return(NULL);
}

int rtp_prepare(int locport,char *remip4,int remport,int profile,int jitt_comp,int red)
{
	int err;
 	rtp_params_t params;

	rtph=rtp_create_session(RTP_DUPLEX);
	if (rtph==NULL)
	{
		printf("Error creating rtp session.\n");
		return(-1);
	}
	bzero(&params,sizeof(rtp_params_t));
	params.profile=profile;
	params.jitt_comp=jitt_comp;
	params.redundancy=red;
	params.rtp_loc_port=locport;
	params.rtp_loc_addr="0.0.0.0";
	params.rtp_rem_port=remport;
	params.rtp_rem_addr=remip4;
	params.stacksize=jitt_comp*2;

	err=rtp_set_parameters(rtph,&params);
	if (err!=0)
	{
		printf("Error setting rtp parameters: %s\n",rtp_perror(err));
		return(-1);
	}
	//if (linpmain.rtp_conf.rtp_dbg!=NULL) rtp_debug(rtph,linpmain.rtp_conf.rtp_dbg);
	return(0);
}

