/***************************************************************************
                          presence.c  -  code for the presence box
                             -------------------
    begin                : Mon Dec 17 2001
    copyright            : (C) 2001 by Simon Morlat
    email                : simon.morlat@linphone.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "callbacks.h"
#include "presence.h"
#include "LinphoneMain.h"
#include "support.h"
#include "../osipua/osipua.h"

/* LinphoneMain object*/
extern LinphoneMain linpmain;
PresenceBox presencebox;

void presence_box_init(PresenceBox *p, GtkWidget *main_window)
{
	GtkWidget *r;
	p->contact_field=lookup_widget(main_window,"contact_field");
	p->minutesaway=lookup_widget(main_window,"minutesaway");
	p->presence_validate=lookup_widget(main_window,"presence_validate");
	r=lookup_widget(main_window,"presence_reachable");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(r),TRUE);
	gtk_widget_set_sensitive(p->presence_validate,FALSE);
}

void presence_box_changed(PresenceBox *p)
{
	gtk_widget_set_sensitive(p->presence_validate,TRUE);
}


void
on_reachable                           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(presencebox.contact_field,FALSE);
	presencebox.toggled_button=PRESENCE_MODE_REACHABLE;
	gtk_widget_set_sensitive(presencebox.minutesaway,FALSE);
	gtk_widget_set_sensitive(presencebox.contact_field,FALSE);
	presence_box_changed(&presencebox);
}


void
on_busy                                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(presencebox.contact_field,FALSE);
	presencebox.toggled_button=PRESENCE_MODE_BUSY;
	gtk_widget_set_sensitive(presencebox.minutesaway,TRUE);
	presence_box_changed(&presencebox);
}


void
on_minutesaway_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	presence_box_changed(&presencebox);
}


void
on_away                                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(presencebox.contact_field,FALSE);
	gtk_widget_set_sensitive(presencebox.minutesaway,TRUE);
	presencebox.toggled_button=PRESENCE_MODE_AWAY;
	presence_box_changed(&presencebox);
}


void
on_do_not_disturb                      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  gtk_widget_set_sensitive(presencebox.contact_field,FALSE);
  gtk_widget_set_sensitive(presencebox.minutesaway,FALSE);
  presencebox.toggled_button=PRESENCE_MODE_NOT_DISTURB;
  presence_box_changed(&presencebox);
}


void
on_moved_tmply                         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  gtk_widget_set_sensitive(presencebox.contact_field,TRUE);
  gtk_widget_set_sensitive(presencebox.minutesaway,FALSE);
  presencebox.toggled_button=PRESENCE_MODE_MOVED;
  presence_box_changed(&presencebox);
}


void
on_alt_serv                            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  gtk_widget_set_sensitive(presencebox.contact_field,TRUE);
  gtk_widget_set_sensitive(presencebox.minutesaway,FALSE);
  presencebox.toggled_button=PRESENCE_MODE_ALT_SERVICE;
  presence_box_changed(&presencebox);
}


void
on_contact_field_changed               (GtkEditable     *editable,
                                        gpointer         user_data)
{
  presence_box_changed(&presencebox);
}


void
on_presence_validate_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *tmp;
	int i;
	/* retrieve the minutes away */
	tmp = gtk_editable_get_chars (GTK_EDITABLE(presencebox.minutesaway),0,-1);
  if (tmp!=NULL && strlen(tmp)>0)
	{
		int delay;
		delay = satoi(tmp);
		g_free(tmp);
      /* linpmain.gui_conf.feild=gtk_entry_get_text(GTK_ENTRY(child)); */
		osip_ua_set_presence_delay(linpmain.ua,delay*60);
		printf("New setting for linpmain.gui_conf.delay: %i\n",linpmain.ua->presence_delay);
	}
	/* retrieve the alternate contact url */
	tmp = gtk_editable_get_chars (GTK_EDITABLE(presencebox.contact_field),0,-1);
	if (tmp!=NULL && strlen(tmp)>0)
	{
		contact_t *co;
		contact_init(&co);
		i = contact_parse(co,tmp);
		osip_ua_set_presence_contact_url(linpmain.ua,tmp);
		if (i==0)
		{
	  	printf("New setting for linpmain.ua->presence_contact_url: %s\n",linpmain.ua->presence_contact_url);
	  	contact_free(co);
	  	sfree(co);
	  }
		else
	  	printf("Error in contact field: could not parse %s. Perhaps this kind of url is not supported.\n",linpmain.ua->presence_contact_url);
    g_free(tmp);
	}
	/* set presence mode */
	switch(presencebox.toggled_button)
	{
		case PRESENCE_MODE_REACHABLE:
			osip_ua_set_presence_mode(linpmain.ua,200);
		break;
		
		case PRESENCE_MODE_BUSY:
			osip_ua_set_presence_mode(linpmain.ua,486);
		break;
		
		case PRESENCE_MODE_AWAY:
			osip_ua_set_presence_mode(linpmain.ua,480);
		break;
		
		case PRESENCE_MODE_NOT_DISTURB:
			osip_ua_set_presence_mode(linpmain.ua,600);
		break;
		
		case PRESENCE_MODE_MOVED:
			osip_ua_set_presence_mode(linpmain.ua,302);
		break;
		
		case PRESENCE_MODE_ALT_SERVICE:
			osip_ua_set_presence_mode(linpmain.ua,380);
		break;
		default:
			printf("unknown presence mode.\n");
	}	
	gtk_widget_set_sensitive(presencebox.presence_validate,FALSE);
}

