/***************************************************************************
                          propertybox.c  -  description
                             -------------------
    begin                : Wed Jan 30 2002
    copyright            : (C) 2002 by Simon Morlat
    email                : simon.morlat@linphone.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "LinphoneMain.h"
#include "callbacks.h"
#include "propertybox.h"
#include "support.h"
#include "../osipua/osipua.h"
#include "../osipua/utils.h"
#include "../audio/audio.h"

extern gint do_registration(gpointer data);
static void codec_list_update(GtkCList *clist,double bandwidth);
extern RegistrationCtxt *regctx;

void net_section_init(NetSection *sec, GtkWidget *prop)
{
	sec->interfaces=lookup_widget(prop, "interfaces");
	sec->connection_type=lookup_widget(prop,"connection_type");
}
/* the callback for interface selection */
void if_selected(GtkCList * clist, gint row, gint column, GdkEventButton *ev,gpointer data)
{
	NetSection *sec=(NetSection*)data;
	sec->if_sel=row;
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sec->root->prop));
}

void on_connection_type_selected(NetSection *sec, gint type)
{
	codec_list_update(GTK_CLIST(sec->root->codec.au_codec_list),bandwidths[type]);
	sec->type_sel=type;
	gnome_property_box_changed(GNOME_PROPERTY_BOX(sec->root->prop));
}

void on_28k_selected(GtkMenuItem *menuitem, gpointer user_data)
{
	on_connection_type_selected((NetSection*)user_data , 0);
}

void on_56k_selected(GtkMenuItem *menuitem, gpointer user_data)
{
	on_connection_type_selected((NetSection*)user_data , 1);
}

void on_128k_selected(GtkMenuItem *menuitem, gpointer user_data)
{
	on_connection_type_selected((NetSection*)user_data , 2);
}

void on_adsl_selected(GtkMenuItem *menuitem, gpointer user_data)
{
	on_connection_type_selected((NetSection*)user_data , 3);
}

void on_ethernet_selected(GtkMenuItem *menuitem, gpointer user_data)
{
	on_connection_type_selected((NetSection*)user_data , 4);
}


void net_section_fill(NetSection *sec,LinphoneMain *lp)
{
	int pos=0;
	GtkWidget *optionmenu=sec->connection_type;
	GtkWidget *menu;
	GtkItem *menu_item;
	GtkMenuShell *menushell;
	GList *item_list,*elem;
	GtkCList *if_list=GTK_CLIST(sec->interfaces);
	gchar *interface[2];
	
	menu=gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu));
	menushell=GTK_MENU_SHELL(menu);
	elem=item_list=menushell->children;
	
	/* 28k menu item*/
	menu_item=(GtkItem*)(elem->data);
	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_28k_selected,(gpointer)sec);
	elem=g_list_next(elem);
	/* 56k menu item */
    menu_item=(GtkItem*)(elem->data);
	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_56k_selected,(gpointer)sec);
	elem=g_list_next(elem);
	/* 128k menu item */
	menu_item=(GtkItem*)(elem->data);
	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_128k_selected,(gpointer)sec);
	elem=g_list_next(elem);
	/* ADSL menu item */
	menu_item=(GtkItem*)(elem->data);
	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_adsl_selected,(gpointer)sec);
	elem=g_list_next(elem);
	/* Ethernet menu item */
	menu_item=(GtkItem*)(elem->data);
	gtk_signal_connect(GTK_OBJECT(menu_item),"activate",(GtkSignalFunc)on_ethernet_selected,(gpointer)sec);
	elem=g_list_next(elem);
	gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu),lp->net_conf.con_type);
	sec->type_sel=lp->net_conf.con_type;
	sec->if_sel=lp->net_conf.if_sel;
	
	/* draw the list of interfaces */
	interface[1]=NULL;
	for(pos=0;pos<lp->net_conf.if_count;pos++)
	{
		interface[0]=lp->net_conf.if_names[pos];
		gtk_clist_append(if_list,interface);
	}
	gtk_clist_select_row(if_list,lp->net_conf.if_sel,0);
	gtk_signal_connect(GTK_OBJECT(if_list),"select-row", (GtkSignalFunc) if_selected,(gpointer)sec);
}



void net_section_apply(NetSection *sec,LinphoneMain *lp)
{
	/* get network interface*/
	lp->net_conf.if_sel=sec->if_sel;
	/* get connection type*/
	lp->net_conf.con_type=sec->type_sel;
	lp->net_conf.local_addr=lp->net_conf.if_addr[sec->if_sel];
	lp->net_conf.if_name=lp->net_conf.if_names[sec->if_sel];
	
	osip_ua_set_ip4addr(lp->ua,lp->net_conf.local_addr);
	lp->net_conf.bandwidth=bandwidths[sec->type_sel];
	linphone_main_apply_codec_change(lp);
}

void rtp_section_init(RtpSection *sec, GtkWidget *prop)
{
	sec->au_port=lookup_widget(prop,"audioport");
	sec->jitt_comp=lookup_widget(prop,"hscale1");
}


void rtp_section_fill(RtpSection *sec, LinphoneMain *lp)
{
	/* put the current value of jitt_comp*/
	gtk_adjustment_set_value (gtk_range_get_adjustment(GTK_RANGE(sec->jitt_comp)),(gfloat)lp->rtp_conf.jitt_comp);
	/* display current rtp port */
  	gtk_spin_button_set_value(GTK_SPIN_BUTTON(sec->au_port),(gfloat)lp->rtp_conf.audio_rtp_port);
}


void rtp_section_apply(RtpSection *sec, LinphoneMain *lp)
{
	/* get the value of jitt_comp*/
	GtkAdjustment *adj=  gtk_range_get_adjustment(GTK_RANGE(sec->jitt_comp));
	lp->rtp_conf.jitt_comp=adj->value;
	/* get  rtp port */
	adj=gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(sec->au_port));
  	lp->rtp_conf.audio_rtp_port=adj->value;
}

void sip_section_init(SipSection *sec,GtkWidget *prop)
{
	sec->port=lookup_widget(prop,"sip_port");
	sec->username=lookup_widget(prop, "user_name");
	sec->hostname=lookup_widget(prop,"domain_name");
	sec->registrar_checked=lookup_widget(prop, "registrar_checked");
	sec->server_label=lookup_widget(prop,"server_label");
	sec->server=lookup_widget(prop,"registrar_addr");
	sec->password=lookup_widget(prop,"reg_passwd");
	sec->password_label=lookup_widget(prop,"regpasslabel");
	sec->addr_of_rec_label=lookup_widget(prop,"addressrec_label");
	sec->addr_of_rec=lookup_widget(prop,"address_of_record");
	sec->registrar_toggle=lookup_widget(prop,"registrar_checked");
	sec->sipfunction_label= lookup_widget(prop,"sipfunction_label");
	sec->redirect_button=lookup_widget(prop,"redirect_button");
	sec->proxy_button=lookup_widget(prop,"proxy_button");
	sec->obproxy_button=lookup_widget(prop, "obproxy_button");
}

void sip_section_enable_registrar(SipSection *sec,LinphoneMain *lp, gboolean state)
{
	gtk_widget_set_sensitive(sec->server_label,state);
	gtk_widget_set_sensitive(sec->server,state);
	gtk_widget_set_sensitive(sec->password_label,state);
	gtk_widget_set_sensitive(sec->password,state);
	gtk_widget_set_sensitive(sec->addr_of_rec_label,state);
	gtk_widget_set_sensitive(sec->addr_of_rec,state);
	gtk_widget_set_sensitive(sec->sipfunction_label,state);
	gtk_widget_set_sensitive(sec->proxy_button, state);
	gtk_widget_set_sensitive(sec->redirect_button,state);
	gtk_widget_set_sensitive(sec->obproxy_button,state);
	
 	if (state) {
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->proxy_button),lp->sip_conf.reg_conf.as_proxy);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->redirect_button),lp->sip_conf.reg_conf.as_redirect);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->obproxy_button),lp->sip_conf.reg_conf.as_outbound);
 	}
 	else {
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->proxy_button),0);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->redirect_button),0);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->obproxy_button),0);
 	}
}

void sip_section_fill(SipSection *sec, LinphoneMain *lp)
{
	/* set sip port */
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(sec->port),(gfloat)lp->sip_conf.sip_port);
	/* set sip username */
	gtk_entry_set_text(GTK_ENTRY(sec->username),lp->sip_conf.username);
	/* set domain name */
	gtk_entry_set_text(GTK_ENTRY(sec->hostname),lp->sip_conf.hostname);
	/* set registrar address*/
	if (lp->sip_conf.reg_conf.registrar!=NULL) gtk_entry_set_text(GTK_ENTRY(sec->server),lp->sip_conf.reg_conf.registrar);
	/* set registrar password */
	if (lp->sip_conf.reg_conf.passwd!=NULL) gtk_entry_set_text(GTK_ENTRY(sec->password),lp->sip_conf.reg_conf.passwd);
	/* set address of record */
	if (lp->sip_conf.reg_conf.addr_of_rec!=NULL) gtk_entry_set_text(GTK_ENTRY(sec->addr_of_rec),lp->sip_conf.reg_conf.addr_of_rec);
	if (lp->sip_conf.reg_conf.use_registrar)
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->registrar_toggle),lp->sip_conf.reg_conf.use_registrar);
		/* putting this button active will send the signal "toggled", and so the registrar_toggled is called to do all the work*/		
	}
}

void sip_section_apply(SipSection *sec, LinphoneMain *lp)
{
	GtkAdjustment *adj;
	gchar *tmp;
	gint state,ind,err;
	
	/* get sip port*/
	adj=gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(sec->port));
	/* if the sip port has changed, we need to restart the ua*/
	if (adj->value!=lp->sip_conf.sip_port)
	{
		linphone_main_stop(lp);
		lp->sip_conf.sip_port=adj->value;
		printf("Restarting user agent.\n");
		lp->sip_conf.sip_port=adj->value;
		err=linphone_main_start(lp);
		if (err<0) printf("Failed to restart user agent!\n");
	}
	tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->username),0,-1);
	if (strlen(tmp)!=0)
	{
		g_free(lp->sip_conf.username);
		lp->sip_conf.username=tmp;
	}
	tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->hostname),0,-1);
	if (strlen(tmp)!=0)
	{
		g_free(lp->sip_conf.hostname);
		lp->sip_conf.hostname=tmp;
	}
	
	state=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sec->registrar_toggle));
	if (!state)
	{
		
		/* unregister if  we were using a registrar*/
		if (lp->sip_conf.reg_conf.use_registrar==1)
		{
			OsipCallLeg *call=osip_call_leg_new(lp->ua);
			osip_call_leg_unregister(call,regctx);
			osip_call_leg_release(call);
		}
		/* tell the osip stack that we have no default registrar*/
		//osip_ua_set_registrar(linpmain.ua,NULL,NULL);
		/*remove registration timer if present*/
		linphone_main_remove_registration_timer(lp);
		/* unset the proxy . if none was used, it has no effect*/
		osip_ua_set_outbound_proxy(lp->ua,NULL,NULL);
	}
	else
	{
		gchar *passwd;
		gchar *addr_of_rec;
		/* we can only do this because unsensitive button are said inactive by default */
		lp->sip_conf.reg_conf.as_proxy=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sec->proxy_button));
		lp->sip_conf.reg_conf.as_redirect=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sec->redirect_button));
		lp->sip_conf.reg_conf.as_outbound=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sec->obproxy_button));
		/*get password*/
		passwd=gtk_editable_get_chars(GTK_EDITABLE(sec->password),0,-1);
		if (lp->sip_conf.reg_conf.passwd!=NULL) g_free(lp->sip_conf.reg_conf.passwd);
		lp->sip_conf.reg_conf.passwd=passwd;
		 /* get registrar name */
		tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->server),0,-1);
		ind=strlen(tmp);
		if (strlen(tmp)!=0)
		{
			remake_sipurl(&tmp);
			if (tmp!=NULL) {
				if (lp->sip_conf.reg_conf.registrar!=NULL) g_free(lp->sip_conf.reg_conf.registrar);
				lp->sip_conf.reg_conf.registrar=tmp;
				lp->sip_conf.reg_conf.use_registrar=1;
				/* set the proxy if user wants one */
				if (lp->sip_conf.reg_conf.as_outbound) osip_ua_set_outbound_proxy(lp->ua,tmp,lp->sip_conf.reg_conf.passwd);
				else osip_ua_set_outbound_proxy(lp->ua,NULL,NULL);
			}
		}
		else
		{
			/* tell the osip stack that we have no default registrar*/
			//osip_ua_set_registrar(linpmain.ua,NULL,NULL);
			/* unset timer if present*/
			linphone_main_remove_registration_timer(lp);
			state=0;  /* the button was checked, but the user did not give us a registrar name!*/
		}
		/* get the address of record */
		tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->addr_of_rec),0,-1);
		addr_of_rec=lp->sip_conf.reg_conf.addr_of_rec;
		if ( addr_of_rec!=NULL) g_free(addr_of_rec);
		if (strlen(tmp)>1) lp->sip_conf.reg_conf.addr_of_rec=tmp;
		else {
			lp->sip_conf.reg_conf.addr_of_rec=NULL;
			g_free(tmp);
		}
	}
	lp->sip_conf.reg_conf.use_registrar=state;
	linphone_main_set_contact(lp);
	if (state)
	{
		linphone_main_set_registration_timer(lp);
		do_registration(NULL);
	}
}

void codec_section_init(CodecSection *sec, GtkWidget *prop)
{
	sec->au_codec_list=lookup_widget(prop,"au_codec_list");
}

/* construct a codec GtkCList*/
void codec_list_construct(GtkCList *clist,GList *codecs)
{
	GList *elem;
	codec_info_t *codec;
	char bitrate[10];
	char *codec_desc[3];
	int row=0;
	
	elem=codecs;
	while(elem!=NULL)
	{
		codec=(codec_info_t*)(elem->data);
		sprintf(bitrate,"%g",((double)codec->bitrate)/1000.0);
		codec_desc[0]=codec->name;
		codec_desc[1]=bitrate;
		//printf("Appending %s\n",codec->name);
		if (codec->is_selected) codec_desc[2]=_("Enabled");
		else codec_desc[2]=_("Disabled");
		gtk_clist_append(clist,codec_desc);
		gtk_clist_set_row_data(clist,row,(void*)codec);
		if (!codec->is_usable) gtk_clist_set_selectable(clist,row,-1);
		
		elem=g_list_next(elem);
		row++;
	}
	if (row) gtk_clist_select_row(clist,0,0);
}
/* update graphical (only) status for codecs list*/
void codec_list_update(GtkCList *clist,double bandwidth)
{
	int row=0,u;
	codec_info_t *cod;
	GList *list=clist->row_list;
	GdkColor colors[2]={{0,65535,0,0},{0,0,0,65535}};
	
	while(list!=NULL)
	{
		cod=gtk_clist_get_row_data(clist,row);
		u=codec_is_usable(cod,bandwidth);
		gtk_clist_set_selectable(clist,row,u);
		gtk_clist_set_foreground(clist,row,&colors[u]);
		list=g_list_next(list);
		row++;
	}
}


void codec_section_fill(CodecSection *sec, LinphoneMain *lp)
{
	/* display prefered codecs*/
	codec_list_construct(GTK_CLIST(sec->au_codec_list),lp->codecs_conf.audio_codecs);
	codec_list_update(GTK_CLIST(sec->au_codec_list),lp->net_conf.bandwidth);
}
/* get codec_info_t GList from codec GtkCList */
GList *codec_list_retrieve(GtkCList *clist)
{
	int row=0;
	char *text;
	codec_info_t *cod;
	GList *list=NULL;
	
	while(gtk_clist_get_text(clist,row,2,&text)==1)
	{
		cod=gtk_clist_get_row_data(clist,row);
		if (strcmp(text,_("Enabled"))==0) cod->is_selected=1;
		else cod->is_selected=0;
		//free(text);
		list=g_list_append(list,(gpointer)cod);
		row++;
	}
	return(list);
}

void codec_section_apply(CodecSection *sec, LinphoneMain *lp)
{
	/* get codec list*/
  	g_list_free(lp->codecs_conf.audio_codecs);
  	lp->codecs_conf.audio_codecs=codec_list_retrieve(GTK_CLIST(sec->au_codec_list));
  	linphone_main_apply_codec_change(lp);
}

void sound_section_init(SoundSection *sec, GtkWidget *prop)
{
	sec->driver_combo=lookup_widget(prop,"sound_mode");
	sec->source_combo=lookup_widget(prop,"rec_source");
	sec->autokill_button=lookup_widget(prop,"autokill_button");
}

void sound_section_fill(SoundSection *sec, LinphoneMain *lp)
{
	GtkWidget *child;
	/* select used sound drivers*/
  	child=sec->driver_combo;
  	switch(lp->sound_conf.driver_mode)
	{
		case 0:
			gtk_entry_set_text (GTK_ENTRY (child), _("OSS"));
  			break;
  		case 1:
  			gtk_entry_set_text (GTK_ENTRY (child), _("ALSA"));
  			break;
  	}
  	/* select audio source*/
  	child=sec->source_combo;
  	switch(lp->sound_conf.source)
	{
		case 'm':
			gtk_entry_set_text (GTK_ENTRY (child), _("micro"));
  			break;
  		case 'l':
  			gtk_entry_set_text (GTK_ENTRY (child), _("line"));
  			break;
  	}
  	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->autokill_button),lp->sound_conf.autokill);
}

void sound_section_apply(SoundSection *sec, LinphoneMain *lp)
{
	gchar *tmp;
	/* get sound driver mode*/
	tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->driver_combo),0,-1);
	if (strcmp(tmp,"OSS")==0) lp->sound_conf.driver_mode=0;
	else if (strcmp(tmp,"ALSA")==0) lp->sound_conf.driver_mode=1;
	g_free(tmp);
  	/* init the selected sound driver*/
	switch(lp->sound_conf.driver_mode)
  	{
  		case 0:
  			printf ("Init Oss.\n");
  			audio_oss_init();
  			break;
  		case 1:
  		#ifdef ALSA
  			printf("Init Alsa.\n");
  			audio_alsa_init();
  		#else
  			printf("ERROR: Linphone was not compiled with alsa support. Recompile it or use OSS.\n");
  		#endif
  			break;
  	}
  	/* get audio source*/
  	tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->source_combo),0,-1);
	if (strcmp(tmp,_("micro"))==0) lp->sound_conf.source='m';
	else if (strcmp(tmp,_("line"))==0) lp->sound_conf.source='l';
	printf("Sound source selection changed to %c.\n",lp->sound_conf.source);
	g_free(tmp);
  	set_source(lp->sound_conf.source);	
  	lp->sound_conf.autokill=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sec->autokill_button));
}

void lp_property_box_init(LPPropertyBox *box)
{
	GtkWidget *prop=create_prop1();
	box->prop=prop;
	net_section_init(&box->net,prop);
	box->net.root=box;
	rtp_section_init(&box->rtp,prop);
	sip_section_init(&box->sip,prop);
	codec_section_init(&box->codec,prop);
	sound_section_init(&box->sound,prop);
}

LPPropertyBox * lp_property_box_new()
{
	LPPropertyBox *box=g_malloc(sizeof(LPPropertyBox));
	lp_property_box_init(box);
	return box;
}

void lp_property_box_fill(LPPropertyBox * box, LinphoneMain *lp)
{
	net_section_fill(&box->net,lp);
	rtp_section_fill(&box->rtp,lp);
	sip_section_fill(&box->sip,lp);
	codec_section_fill(&box->codec,lp);
	sound_section_fill(&box->sound,lp);
	/* set uchanged state to the prop1 box, because gtk_entry_set_text() causes signal "changed"
  	to be sent */
	gnome_property_box_set_state(GNOME_PROPERTY_BOX(box->prop),0);
}

