/***************************************************************************
                          linphone  - sipomatic.c
This is a test program for linphone. It acts as a sip server and answers to linphone's
call.
                             -------------------
    begin                : ven mar  30
    copyright            : (C) 2001 by Simon MORLAT
    email                : simon.morlat@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <config.h>

#include <unistd.h>
#include <signal.h>
#include "../osipua/osipua.h"
#include "sip_handlers.h"
#include "../audio/ring.h"
#include "../audio/audio.h"
#include "LinphoneMain.h"
#include "io.h"
#include "../gsmlib/gsm_wrapper.h"
#include "../lpc10-1.5/lpc10_wrapper.h"
#include "../G711/g711alaw.h"
#include "../G711/g711ulaw.h"


OsipUA *ua;
OsipCallLeg *call_leg=NULL;
int run_cond=1;
int com_time=-1;  /* nb of second of playing file, 0 if not limited*/

#define COM_TIME 12

void stop_handler(int signum);

void stop_handler(int signum)
{
	run_cond=0;
}

int invite_accepted_cb(OsipCallLeg *call,sip_t *msg,void *p2)
{
	/* we should never be here since sipomatic is unable to to make invites*/
	return(0);
}

int bye_cb(OsipCallLeg *call,sip_t *msg,void *p2)
{
	printf ("stopping audio thread\n");
	stop_audio_io();
	call_leg=NULL;
	com_time=-1;
	return(0);
}

int invite_cb(OsipCallLeg *call,sip_t *msg,void *p2)
{
	/* we have just received an invite, so wait two seconds, then accept it*/
	sip_t *sipmsg= (sip_t*)p2;
	char *p;
	
	from_2char(sipmsg->from,&p);
	printf("Contacted by %s.\n",p);
	sfree(p);
	call_leg=call;
	return(0);
}

void respond_to_call(OsipCallLeg *call)
{
	int err,err1;
	list_t *streams;
	MediaDesc *md;
	
	com_time=COM_TIME; /*14 seconds for file playing*/
	printf("Answering...\n");
	streams=osip_call_leg_accept_invite(call);
	md=list_get(streams,0);
 	/* prepare rtp session using sdp paramters*/
 	printf("Remote rtp address: %s:%i\n", md->rem_ip4addr,md->rem_port);
 	err=rtp_prepare(7071,md->rem_ip4addr,md->rem_port,profile_1(md),7,0);
 	if (err==0)
 	{
 		/* start the io loop*/
 		switch(profile_1(md))
 		{
 		case 3:
 			err1=start_audio_io(&gsm_codec_info);
 			break;
 		case 4:
 			err1=start_audio_io(&LPC10codec_info);
 			break;
 		case 0:
 			err1=start_audio_io(&g711ulaw_info);
 			break;
 		case 8:
 			err1=start_audio_io(&g711alaw_info);
 			break;
 		default:
 			err1=-1;
 			printf("Unsupported profile requested !!");
 		}
 		/* if we fail, it is because of an unopenable audio device*/
 		if (err1==-1)
 		{
 			printf("Cannot start io!!\n");
 		}
 	}else printf("Failed to create RTP session.\n");
	
}


int faillure_cb(OsipCallLeg *call,sip_t *msg,void *p2)
{
	return(0);
}

int main(int argc, char *argv[])
{
	int err;
	char snd_path[100];
	ua_params_t params;
	int presence=200;
	
	if (argc==2) presence=atoi(argv[1]);
	
	signal(SIGINT,stop_handler);
	osipua_init();
	
	memset(&params,0,sizeof(ua_params_t));
	params.contact="<sip:robot@127.0.0.1>";
	params.ua_ip4addr="127.0.0.1";
	params.ua_port=5064;
	params.max_calls=1;
	params.dbg=NULL;
	ua=osip_ua_new(&params);
  	if (ua==NULL)
  	{
		printf("Cannot create SIP user agent.\n");
    	return(-1);
	}
	
  	err=osip_ua_signal_connect(ua,"INVITE_ACCEPTED",invite_accepted_cb);
  	err=osip_ua_signal_connect(ua,"BYE",bye_cb);
  	err=osip_ua_signal_connect(ua,"FAILLURE",faillure_cb);
  	err=osip_ua_signal_connect(ua,"INVITE",invite_cb);

	osip_ua_add_profile(ua,STREAM_AUDIO,3);
	osip_ua_add_profile(ua,STREAM_AUDIO,4);
	osip_ua_add_profile(ua,STREAM_AUDIO,0);
	osip_ua_add_profile(ua,STREAM_AUDIO,8);
	osip_ua_set_rtpport(ua,STREAM_AUDIO,7071);
	
	osip_ua_set_presence_mode(ua,presence);
	osip_ua_set_presence_delay(ua,60);
	osip_ua_set_presence_contact_url(ua,"sip:toto@free.fr");
	
	audio_iodisk_init();  /* select the loopback audio interface*/
	snprintf(snd_path,100,"%s%s",PACKAGE_DATA_DIR,"/sounds/hello.wav");
	select_device(snd_path,"");    /*select input and output file */
	if (err!=0) printf("Failed to create UA.\n");
	while(run_cond)
	{
		if (call_leg!=NULL)
		{
			printf("Calleg!=NULL..com_time=%i\n",com_time);
			if (com_time==-1)
			{
				sleep(5);
				if (call_leg!=NULL) respond_to_call(call_leg);
			}
			if (com_time>0) com_time--;
			if (com_time==0)
			{
				printf ("stopping audio thread\n");
				stop_audio_io();
				printf ("bye...\n");
				osip_call_leg_bye(call_leg);
				call_leg=NULL;
				com_time=-1;
			}
		}
		sleep(1);
		//for (err=0;err<4411111;err++) ;
	}
	osip_ua_destroy(ua);
	//fclose(linphone_config.rtp_dbg);
	return(0);
}

